using System;
using System.Collections.Generic;
using System.Collections;
using System.IO;
using UnityEngine;
using UnityEngine.UI;
using System.Linq;
using TMPro;
using UnityEngine.Events;

namespace Mod
{
	public class EntityCreator : MonoBehaviour
	{
		public string OriginalItemName = "Bowling Pin";
		public string PhysicalPropertiesName = "Human";

		public GameObject entity {get; private set;}
		[SkipSerialisation]
		public SpriteRenderer entityRenderer {get; private set;}

		[SkipSerialisation]
		public Sprite Sprite;
		[SkipSerialisation]
		public Sprite CollisionSprite;
        [SkipSerialisation]
        public Sprite[] RandomSprites;

        [SkipSerialisation]
        public bool IsCircleCollider = false;

		//public AudioClip SoundOnActivation;
        //public AudioClip OverrideImpactSounds = new AudioClip[];

		public int sortingOrder = 0;
		public string sortingLayerName = "Default";

		public float LimbMass = 1f;
        public bool DisplayBloodDecals = true;

		public Vector3 localPosition = Vector3.zero;
        public Vector3 localAngle = Vector3.zero;


		public float hingeJointBreakForce = 600f;
		public Vector3 hingeJointAnchor;
		public int limitsMin;
		public int limitsMax;
		public bool UseLimits = true;

		public void CreateNewEntity()
        {
            entity = GameObject.Instantiate(ModAPI.FindSpawnable(OriginalItemName).Prefab);
            entity.AddComponent<AudioSourceTimeScaleBehaviour>();
            entity.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties(PhysicalPropertiesName);

        //CustomCollision
            if (CollisionSprite != null)
            {
                entityRenderer = entity.GetComponent<SpriteRenderer>();
                entityRenderer.sprite = CollisionSprite;
                foreach (Collider2D ColliderBoxDelete in entity.GetComponents<Collider2D>())
                {
                    GameObject.Destroy(ColliderBoxDelete);
                }
                entity.FixColliders();            
                Debug.Log("CollisionSprite = !null");
            }
            else if (IsCircleCollider != false)
            {
                Destroy(GetComponent<Collider2D>());
                //foreach (Collider2D ColliderBoxDelete in entity.GetComponents<Collider2D>())
              	//{ 
                    //GameObject.Destroy(ColliderBoxDelete);
              	//}
              	entity.AddComponent<CircleCollider2D>();
                entity.FixColliders(); 
                Debug.Log("IsCircleCollider = True");
            }


        //Sprite
            
            if (RandomSprites != null)
            {
                RandomSpriteBehaviour entityRendererRandom = entity.AddComponent<RandomSpriteBehaviour>();
                entityRendererRandom.sprites = RandomSprites;
            }
            else if (Sprite != null)
            {
                entityRenderer.sprite = Sprite;
            }
            entityRenderer.sortingOrder = sortingOrder;
            entityRenderer.sortingLayerName = sortingLayerName;   

			PhysicalBehaviour phys = entity.GetComponent<PhysicalBehaviour>();
			phys.TrueInitialMass = LimbMass;
			phys.InitialMass = LimbMass;
			phys.rigidbody.mass = LimbMass;
            phys.DisplayBloodDecals = DisplayBloodDecals;
            //phys.OverrideImpactSounds = OverrideImpactSounds; 
            phys.RefreshOutline();
            phys.BakeColliderGridPoints();

		//pos, angle, scale
			entity.transform.SetParent(gameObject.transform);
			entity.transform.localEulerAngles = localAngle;
			entity.transform.localPosition = localPosition;
			entity.transform.localScale = Vector3.one;
		
		//HingeJoint
			HingeJoint2D entityhingeJoint = entity.GetOrAddComponent<HingeJoint2D>();
			entityhingeJoint.connectedBody = gameObject.GetComponent<Rigidbody2D>();
			entityhingeJoint.breakForce = hingeJointBreakForce;
			entityhingeJoint.anchor = hingeJointAnchor;
			
			JointAngleLimits2D entitylimits = entityhingeJoint.limits;
			entitylimits.min = limitsMin;
			entitylimits.max = limitsMax;
			entityhingeJoint.limits = entitylimits;
			entityhingeJoint.useLimits = true;

			Debug.Log("New Limb Created");
		}		
	}
}
// Originally uploaded by 'Kyrelink, Quickness'. Do not reupload without their explicit permission.
