using System;
using System.Collections.Generic;
using System.Collections;
using System.IO;
using UnityEngine;
using UnityEngine.UI;
using System.Linq;
using TMPro;
using UnityEngine.Events;

namespace Mod
{
	public class ImprovedDoorBehaviour : MonoBehaviour
    {
        private void Start()
        {
            phys = GetComponent<PhysicalBehaviour>();
            renderer = GetComponent<SpriteRenderer>();
            audioSource = gameObject.AddComponent<AudioSource>();
            audioSource.spatialBlend = 1f;
            if (chosenIndex == -1)
            {
                chosenIndex = UnityEngine.Random.Range(0, Options.Length);
            }
            phys.Properties = ModAPI.FindPhysicalProperties(Options[chosenIndex].PhysicalProperties);
            UpdateActivation();
        }
        private void Use()
        {
            Activated = !Activated;
            if (Activated)
            {
                audioSource.PlayOneShot(Options[chosenIndex].OpenSound);
            }
            else
            {
                audioSource.PlayOneShot(Options[chosenIndex].CloseSound);
            }
            UpdateActivation();
        }
        private void UpdateActivation()
        {
            if (Activated)
            {
                renderer.sprite = Options[chosenIndex].OpenSprite;
                renderer.sortingLayerName = "Foreground";
                renderer.sortingOrder = 20;
                gameObject.layer = 10;
                phys.RefreshOutline();
            }
            else
            {
                renderer.sprite = Options[chosenIndex].CloseSprite;
                gameObject.layer = LayerMask.NameToLayer("Objects");
                renderer.sortingLayerName = "Bottom";
                renderer.sortingOrder = -20;
                phys.RefreshOutline();
            }
        }

        public bool Activated;
        private PhysicalBehaviour phys;
        private SpriteRenderer renderer;
        private AudioSource audioSource;

        [SkipSerialisation]
        public DoorOptions[] Options = new DoorOptions[0];
        public int chosenIndex = -1;
        public struct DoorOptions
        {
            public Sprite CloseSprite;
            public Sprite OpenSprite;
            public AudioClip CloseSound;
            public AudioClip OpenSound;
            public string PhysicalProperties;
            //public string CloseSortingLayerName;
            //public string OpenSortingLayerName;
        }
    }
}
// Originally uploaded by 'Kyrelink, Quickness'. Do not reupload without their explicit permission.
