﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.IO;
using System.Reflection;
using UnityEngine;
using UnityEngine.UI;
using System.Linq;
using TMPro;
using UnityEngine.Events;

namespace Mod
{
    public class Mod : MonoBehaviour
    {
        public static void OpenLink(string url)
		{
			Type type = Type.GetType("UnityEngine.Application, UnityEngine.CoreModule");
			type.GetMethod("OpenURL", BindingFlags.Public | BindingFlags.Static).Invoke(null, new object[1] { url }); //! test
		}

        public static bool NewUpdateWindowHasBeenOpened = false;

		public class OnMapLoad: MonoBehaviour
		{
			public const string LastViewedVersionJsonPath = "The ULTIMATE Minecraft Mod_LastViewedVersion.json";

			public void Start()
			{
				TryCreateInfoWindow();
				Destroy(gameObject);
			}
			private void TryCreateInfoWindow()
			{
				if (NewUpdateWindowHasBeenOpened)
				{
					return;
				}

				string lastVersion = "";
				try
				{
					lastVersion = ModAPI.DeserialiseJSON<string>(LastViewedVersionJsonPath);
				}
				catch
				{
				}

				string thisVersion = ModLoader.LoadedMods.Find((ModMetaData meta) => meta.Name == "The ULTIMATE Minecraft Mod" && meta.Author == "Kyrelink, Quickness").ModVersion;
				NewUpdateWindowHasBeenOpened = thisVersion == lastVersion;

				//* Window on first click on a category
				if (!NewUpdateWindowHasBeenOpened)
				{
					string[] categoryNames = new string[1]
					{
						"Minecraft"
					};
					UnityAction categoryOnClick = () =>
					{
						if (!NewUpdateWindowHasBeenOpened)
						{
							DialogBox discordInvite = DialogBoxManager.Dialog("Thanks for installing this mod! (The ULTIMATE Minecraft Mod) \nYou can join our discord channel for news about this mod and future releases.", new DialogButton[]
                            {
                                new DialogButton("Discord (Int Team)", false, () =>
                                {
                                    OpenLink("https://discord.gg/cCQkWJ9j6W");                                
                                }),
                                new DialogButton("Discord (Kyrelink)", false, () =>
                                {
                                    OpenLink("https://discord.gg/8zsmgsU5je");
                                }),
                                new DialogButton("Trailer", false, () =>
                                {
                                    OpenLink("https://www.youtube.com/watch?v=OcUqkW6O3CI");
                                }),
                                new DialogButton("Close", true)
                            });
							NewUpdateWindowHasBeenOpened = true;
							ModAPI.SerialiseJSON(thisVersion, LastViewedVersionJsonPath);
						}
					};

					GameObject categorys = GameObject.Find("/Canvas/Catalog/CategoryContainer/CategoryPanel(Clone)");
					foreach (CategoryButtonBehaviour categoryButton in categorys.GetComponentsInChildren<CategoryButtonBehaviour>())
					{
						if (categoryNames.Contains(categoryButton.Category.name))
						{
							categoryButton.GetComponent<Button>().onClick.AddListener(categoryOnClick);
						}
					}
				}
			}
		}       
        public static void Main()
        {
            ModAPI.RegisterCategory("Minecraft", "",ModAPI.LoadSprite("Cubes.png"));
            ModAPI.Register<OnMapLoad>();

#region Entities
    //Steve
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Steve",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "Steve. The strongest creature in Minecraft",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/5.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Steve2.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Steve_meat.png");
                        var bone = ModAPI.LoadTexture("Mobs/Skeleton.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        person.SetBruiseColor(0, 0, 0); //main bruise colour. purple-ish by default
                        person.SetSecondBruiseColor(0, 0, 0); //second bruise colour. red by default
                        person.SetThirdBruiseColor(0, 0, 0); // third bruise colour. light yellow by default
                        person.SetBloodColour(0, 0, 0); // blood clour. dark red by default
                        person.SetRottenColour(0, 0, 0); // rotten/zombie colour. light yellow/green by default
                        foreach (var body in person.Limbs)
                        {
                             body.BaseStrength *= 1f;                            
                             body.Health *= 1f;
                             body.BreakingThreshold *= 1f;                               
                        }
                    }
                }
            );
    //Alex
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Alex",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "Also the strongest creature in Minecraft",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/7.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/alex.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Steve_meat.png");
                        var bone = ModAPI.LoadTexture("Mobs/Skeleton.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        person.SetBruiseColor(0, 0, 0); //main bruise colour. purple-ish by default
                        person.SetSecondBruiseColor(0, 0, 0); //second bruise colour. red by default
                        person.SetThirdBruiseColor(0, 0, 0); // third bruise colour. light yellow by default
                        person.SetBloodColour(0, 0, 0); // blood clour. dark red by default
                        person.SetRottenColour(0, 0, 0); // rotten/zombie colour. light yellow/green by default
                        foreach (var body in person.Limbs)
                        {
                             body.BaseStrength *= 1f;                            
                             body.Health *= 1f;
                             body.BreakingThreshold *= 1f;                               
                        }
                    }
                }
            );
    //Herobrine
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Herobrine",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "He's close by. He's outside the window. He's watching you.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/23.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/herobrine.png");
                        var flesh = ModAPI.LoadTexture("Mobs/herobrine Layer Flesh.png");
                        var bone = ModAPI.LoadTexture("Mobs/herobrine Layer Bone.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        person.SetBruiseColor(1, 0, 1); //main bruise colour. purple-ish by default
                        person.SetSecondBruiseColor(20, 0, 20); //second bruise colour. red by default
                        person.SetThirdBruiseColor(50, 0, 50); // third bruise colour. light yellow by default
                        person.SetBloodColour(1, 0, 1); // blood clour. dark red by default
                        person.SetRottenColour(0, 0, 0); // rotten/zombie colour. light yellow/green by default
                        foreach (var body in person.Limbs)
                        {
                             body.Health *= 85485484.6f;
                            body.InitialHealth *= 85485484.6f;
                            body.BreakingThreshold = Mathf.Infinity;
                            body.PhysicalBehaviour.Charge = 0f;
                            body.ShotDamageMultiplier *= 0f;
                            body.Vitality *= 3f;
                            body.BloodDecal = null;
                            body.RegenerationSpeed += 10f;
                            body.Numbness *= 3f;
                            body.CirculationBehaviour.BloodFlow *= 0f;
                            body.ImpactPainMultiplier *= 0f;
                            body.IsLethalToBreak = false;
                            body.PhysicalBehaviour.ReflectsLasers = true;
                            body.PhysicalBehaviour.Properties.Flammability = 0f;
                            body.PhysicalBehaviour.Disintegratable = false;
                            body.PhysicalBehaviour.Deletable = false;
                                            body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                                          ModAPI.LoadSound("Sfx/hero.wav"),                                                                                
                                                        };                             
                        }
                    }
                }
            );
    //Zombie
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Minecraft Zombie",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "Insert Zomibe Sounds here",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/3.png"),
                    AfterSpawn = (Instance) =>
                    {

 
                        var armLeft = Instance.transform.Find("FrontArm");
                        var armRight = Instance.transform.Find("BackArm");

                        armLeft.localRotation = Quaternion.Euler(0f, 0f, 90f);
                        armRight.localRotation = Quaternion.Euler(0f, 0f, 90f);

                        
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Zombie.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Steve_meat.png");
                        var bone = ModAPI.LoadTexture("Mobs/Skeleton.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        person.SetBruiseColor(0, 0, 0); //main bruise colour. purple-ish by default
                        person.SetSecondBruiseColor(0, 0, 0); //second bruise colour. red by default
                        person.SetThirdBruiseColor(0, 0, 0); // third bruise colour. light yellow by default
                        person.SetBloodColour(0, 0, 0); // blood clour. dark red by default
                        person.SetRottenColour(0, 0, 0); // rotten/zombie colour. light yellow/green by default
                        foreach (var body in person.Limbs)
                        {
                             body.BaseStrength *= 1f;                            
                             body.Health *= 1f;
                             body.BreakingThreshold *= 1f;
                             body.IsZombie = false;
                             body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                                          ModAPI.LoadSound("Sfx/zom1.wav"),
                                                          ModAPI.LoadSound("Sfx/zom2.wav"),
                                                          ModAPI.LoadSound("Sfx/zom3.wav"),
                                                          ModAPI.LoadSound("Sfx/zom4.wav"), 
                                                          ModAPI.LoadSound("Sfx/zom5.wav"),                                                                                  
                                                        }; 
                             
                             
                        }
                    }
                }
            );
    //Drowned
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Minecraft Drowned",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "Unfortunately, he drowned himself.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/22.png"),
                    AfterSpawn = (Instance) =>
                    {

 
                        var armLeft = Instance.transform.Find("FrontArm");
                        var armRight = Instance.transform.Find("BackArm");

                        armLeft.localRotation = Quaternion.Euler(0f, 0f, 90f);
                        armRight.localRotation = Quaternion.Euler(0f, 0f, 90f);

                        
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/drowned.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Steve_meat.png");
                        var bone = ModAPI.LoadTexture("Mobs/Skeleton.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        person.SetBruiseColor(0, 0, 0); //main bruise colour. purple-ish by default
                        person.SetSecondBruiseColor(0, 0, 0); //second bruise colour. red by default
                        person.SetThirdBruiseColor(0, 0, 0); // third bruise colour. light yellow by default
                        person.SetBloodColour(0, 0, 0); // blood clour. dark red by default
                        person.SetRottenColour(0, 0, 0); // rotten/zombie colour. light yellow/green by default
                        foreach (var body in person.Limbs)
                        {
                             body.BaseStrength *= 1f;                            
                             body.Health *= 1f;
                             body.BreakingThreshold *= 1f;
                             body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                                          ModAPI.LoadSound("Sfx/dr1.wav"),
                                                          ModAPI.LoadSound("Sfx/dr2.wav"),
                                                          ModAPI.LoadSound("Sfx/dr3.wav"),
                                                          ModAPI.LoadSound("Sfx/dr4.wav"), 
                                                          ModAPI.LoadSound("Sfx/dr5.wav"),    
                                                          ModAPI.LoadSound("Sfx/dr6.wav"),                                                                               
                                                        }; 
                             
                             
                        }
                    }
                }
            );
    //Husk
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Husk",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "Desert Zombie",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Husk.png"),
                    AfterSpawn = (Instance) =>
                    {

 
                        var armLeft = Instance.transform.Find("FrontArm");
                        var armRight = Instance.transform.Find("BackArm");

                        armLeft.localRotation = Quaternion.Euler(0f, 0f, 90f);
                        armRight.localRotation = Quaternion.Euler(0f, 0f, 90f);

                        
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Husk.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Steve_meat.png");
                        var bone = ModAPI.LoadTexture("Mobs/Skeleton.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        person.SetBruiseColor(0, 0, 0); //main bruise colour. purple-ish by default
                        person.SetSecondBruiseColor(0, 0, 0); //second bruise colour. red by default
                        person.SetThirdBruiseColor(0, 0, 0); // third bruise colour. light yellow by default
                        person.SetBloodColour(0, 0, 0); // blood clour. dark red by default
                        person.SetRottenColour(0, 0, 0); // rotten/zombie colour. light yellow/green by default
                        foreach (var body in person.Limbs)
                        {
                             body.BaseStrength *= 1f;                            
                             body.Health *= 1f;
                             body.BreakingThreshold *= 1f;
                             body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                                          ModAPI.LoadSound("Sfx/Huskhurt1.mp3"),
                                                          ModAPI.LoadSound("Sfx/Huskhurt2.mp3"),                                                                              
                                                        }; 
                             
                             
                        }
                    }
                }
            );
    //Skeleton
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Minecraft Skeleton",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "Spooky, scary, Creepy",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/4.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Skeleton.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Skeleton.png");
                        var bone = ModAPI.LoadTexture("Mobs/Skeleton.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        person.SetBruiseColor(0, 0, 0); //main bruise colour. purple-ish by default
                        person.SetSecondBruiseColor(0, 0, 0); //second bruise colour. red by default
                        person.SetThirdBruiseColor(0, 0, 0); // third bruise colour. light yellow by default
                        person.SetBloodColour(0, 0, 0); // blood clour. dark red by default
                        person.SetRottenColour(0, 0, 0); // rotten/zombie colour. light yellow/green by default

                             foreach (var body in person.Limbs)
                             {
                              body.BaseStrength *= 1f;                            
                              body.Health *= 2f;
                              body.BreakingThreshold *= 1f; 
                              body.BloodDecal = null;
                              body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
							{
                                                          ModAPI.LoadSound("Sfx/skel1.wav"),
                                                          ModAPI.LoadSound("Sfx/skel2.wav"),
                                                          ModAPI.LoadSound("Sfx/skel3.wav"),
                                                          ModAPI.LoadSound("Sfx/skel4.wav"),
                                                          ModAPI.LoadSound("Sfx/skelstab.wav"),                                                                                    
                                                        }; 
						                                 
                             }
                    }
                }
            );
    //Stray
        ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Human"),
					NameOverride = "Stray",
					NameToOrderByOverride = "A",
					DescriptionOverride = " ",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Stray.png"),
					AfterSpawn = (Instance) =>
					{
						
						var skin = ModAPI.LoadTexture("Mobs/Stray.png");
						var flesh = ModAPI.LoadTexture("Mobs/Stray Fresh.png");
						var bone = ModAPI.LoadTexture("Mobs/Stray bone.png");
						Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);

						var person = Instance.GetComponent<PersonBehaviour>();
						foreach (var body in person.Limbs)
                        {
                            body.BaseStrength *= 1f;                            
                            body.Health *= 1f;
                            body.BreakingThreshold *= 1f; 
                            body.BloodDecal = null; 
                            body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
							{
                                ModAPI.LoadSound("Sfx/Strhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Strhurt2.mp3"),  
								ModAPI.LoadSound("Sfx/Strhurt3.mp3"),   
								ModAPI.LoadSound("Sfx/Strhurt4.mp3"),                                                                               
                            };                             
                        }
						foreach (var body in person.Limbs)
						{
							 var backpack = new GameObject("Backpack");
							 backpack.transform.SetParent(Instance.transform.Find("Body").Find("LowerBody"));
							 backpack.transform.localPosition = new Vector3(0f, 0f);
							 backpack.transform.localScale = new Vector3(1f, 1f);
							 var backpackSprite = backpack.AddComponent<SpriteRenderer>();
							 backpackSprite.sprite =  ModAPI.LoadSprite("Mobs/StrayDown.png");  
							 backpack.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";                          
						}
					}
				}               
			);
    //Wither Skeleton
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Wither Skeleton",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "Wither danger",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/6.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Skeleton2.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Skeleton2.png");
                        var bone = ModAPI.LoadTexture("Mobs/Skeleton2.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        person.SetBruiseColor(0, 0, 0); //main bruise colour. purple-ish by default
                        person.SetSecondBruiseColor(0, 0, 0); //second bruise colour. red by default
                        person.SetThirdBruiseColor(0, 0, 0); // third bruise colour. light yellow by default
                        person.SetBloodColour(0, 0, 0); // blood clour. dark red by default
                        person.SetRottenColour(0, 0, 0); // rotten/zombie colour. light yellow/green by default
                        foreach (var body in person.Limbs)
                        {
                             body.BaseStrength *= 1f;                            
                             body.Health *= 1f;
                             body.BreakingThreshold *= 1f; 
                             body.BloodDecal = null; 
                             body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
							{
                                                          ModAPI.LoadSound("Sfx/skel1.wav"),
                                                          ModAPI.LoadSound("Sfx/skel2.wav"),
                                                          ModAPI.LoadSound("Sfx/skel3.wav"),
                                                          ModAPI.LoadSound("Sfx/skel4.wav"),
                                                          ModAPI.LoadSound("Sfx/skelstab.wav"),                                                                                    
                                                        };                             
                        }
                    }
                }
            );
    //Pillager
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Pillager",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "Dwells near his outpost. Uses a crossbow as his weapon",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Pillager.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Pillager.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
                        var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        foreach (var body in person.Limbs)
                        {
                            body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                                {
                                    ModAPI.LoadSound("Sfx/Pillager.wav"),
                                    ModAPI.LoadSound("Sfx/Pillager2.wav"),
                                    ModAPI.LoadSound("Sfx/Pillager3.wav"),                                                                                
                                };
                             var ca = new GameObject("ca");
                             ca.transform.SetParent(Instance.transform.Find("Head"));
                             ca.transform.localPosition = new Vector3(0f, 0.18f);
                             ca.transform.localScale = new Vector3(1f, 1f);
                             var caSprite = ca.AddComponent<SpriteRenderer>();
                             caSprite.sprite = ModAPI.LoadSprite("Mobs/Illager Head.png"); 
                             ca.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground"; 
                             ca.GetComponent<SpriteRenderer>().sortingOrder = 1;
                             ca.AddComponent<RemoveAfterParentDie>();   

                             var car = new GameObject("car");
                             car.transform.SetParent(Instance.transform.Find("Head"));
                             car.transform.localPosition = new Vector3(0.17f, -0.11f);
                             car.transform.localScale = new Vector3(1f, 1f);
                             var carSprite = car.AddComponent<SpriteRenderer>();
                             carSprite.sprite = ModAPI.LoadSprite("Mobs/Illager Nose.png"); 
                             car.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground"; 
                             car.GetComponent<SpriteRenderer>().sortingOrder = 1;
                             car.AddComponent<RemoveAfterParentDie>();                             
                        }
                    }
                }
            );
    //Vindicator
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Vindicator",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "A very strong illager. Uses an axe as his primary weapon",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Vindicator.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Vindicator.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
                        var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        foreach (var body in person.Limbs)
                        {
                            body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                                {
                                    ModAPI.LoadSound("Sfx/Vindicator.wav"),
                                    ModAPI.LoadSound("Sfx/Vindicator2.wav"),
                                    ModAPI.LoadSound("Sfx/Vindicator3.wav"),                                                                                
                                };
                             var ca = new GameObject("ca");
                             ca.transform.SetParent(Instance.transform.Find("Head"));
                             ca.transform.localPosition = new Vector3(0f, 0.18f);
                             ca.transform.localScale = new Vector3(1f, 1f);
                             var caSprite = ca.AddComponent<SpriteRenderer>();
                             caSprite.sprite = ModAPI.LoadSprite("Mobs/Illager Head.png"); 
                             ca.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground"; 
                             ca.GetComponent<SpriteRenderer>().sortingOrder = 1; 
                             ca.AddComponent<RemoveAfterParentDie>();   

                             var car = new GameObject("car");
                             car.transform.SetParent(Instance.transform.Find("Head"));
                             car.transform.localPosition = new Vector3(0.17f, -0.11f);
                             car.transform.localScale = new Vector3(1f, 1f);
                             var carSprite = car.AddComponent<SpriteRenderer>();
                             carSprite.sprite = ModAPI.LoadSprite("Mobs/Illager Nose.png"); 
                             car.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";  
                             car.GetComponent<SpriteRenderer>().sortingOrder = 1;
                             car.AddComponent<RemoveAfterParentDie>();                          
                        }
                    }
                }
            );
    //Evoker
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Evoker",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "A magician, a wizard. Lives in a mansion in the dark forest. Uses magic to kill.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Evoker.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Evoker.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
                        var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        foreach (var body in person.Limbs)
                        {
                            body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                                {
                                    ModAPI.LoadSound("Sfx/Evoker.wav"),
                                    ModAPI.LoadSound("Sfx/Evoker2.wav"),                                                                             
                                };
                             var ca = new GameObject("ca");
                             ca.transform.SetParent(Instance.transform.Find("Head"));
                             ca.transform.localPosition = new Vector3(0f, 0.18f);
                             ca.transform.localScale = new Vector3(1f, 1f);
                             var caSprite = ca.AddComponent<SpriteRenderer>();
                             caSprite.sprite = ModAPI.LoadSprite("Mobs/Illager Head.png"); 
                             ca.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";  
                             ca.GetComponent<SpriteRenderer>().sortingOrder = 1;  
                             ca.AddComponent<RemoveAfterParentDie>();  

                             var car = new GameObject("car");
                             car.transform.SetParent(Instance.transform.Find("Head"));
                             car.transform.localPosition = new Vector3(0.17f, -0.11f);
                             car.transform.localScale = new Vector3(1f, 1f);
                             var carSprite = car.AddComponent<SpriteRenderer>();
                             carSprite.sprite = ModAPI.LoadSprite("Mobs/Illager Nose.png"); 
                             car.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             car.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             car.AddComponent<RemoveAfterParentDie>();                           
                        }
                    }
                }
            );
    //Illusioner
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Illusioner",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "The Mysterious Wizard. Appears occasionally in raids on villages. Uses a bow.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Illusioner.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Illusioner.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
                        var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        foreach (var body in person.Limbs)
                        {
                            body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                                {
                                    ModAPI.LoadSound("Sfx/Illusioner.wav"),
                                    ModAPI.LoadSound("Sfx/Illusioner2.wav"),
                                    ModAPI.LoadSound("Sfx/Illusioner3.wav"),                                                                                
                                };
                             var ca = new GameObject("ca");
                             ca.transform.SetParent(Instance.transform.Find("Head"));
                             ca.transform.localPosition = new Vector3(0f, 0.18f);
                             ca.transform.localScale = new Vector3(1f, 1f);
                             var caSprite = ca.AddComponent<SpriteRenderer>();
                             caSprite.sprite = ModAPI.LoadSprite("Mobs/Illusioner Head.png"); 
                             ca.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             ca.GetComponent<SpriteRenderer>().sortingOrder = 1;    
                             ca.AddComponent<RemoveAfterParentDie>();  

                             var car = new GameObject("car");
                             car.transform.SetParent(Instance.transform.Find("Head"));
                             car.transform.localPosition = new Vector3(0.17f, -0.11f);
                             car.transform.localScale = new Vector3(1f, 1f);
                             var carSprite = car.AddComponent<SpriteRenderer>();
                             carSprite.sprite = ModAPI.LoadSprite("Mobs/Illager Nose.png"); 
                             car.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             car.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             car.AddComponent<RemoveAfterParentDie>();                           
                        }
                    }
                }
            );
    //Iceologer
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Iceologer",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "Evil Illager. Kind of like a sorcerer throwing ice floes, but he doesn't own it here.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Iceologer.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Iceologer.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
                        var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        foreach (var body in person.Limbs)
                        {
                            body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                                {
                                    ModAPI.LoadSound("Sfx/Pillager.wav"),
                                    ModAPI.LoadSound("Sfx/Pillager2.wav"),
                                    ModAPI.LoadSound("Sfx/Pillager3.wav"),                                                                                
                                };
                             var ca = new GameObject("ca");
                             ca.transform.SetParent(Instance.transform.Find("Head"));
                             ca.transform.localPosition = new Vector3(0f, 0.18f);
                             ca.transform.localScale = new Vector3(1f, 1f);
                             var caSprite = ca.AddComponent<SpriteRenderer>();
                             caSprite.sprite = ModAPI.LoadSprite("Mobs/Iceologer Head.png"); 
                             ca.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground"; 
                             ca.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             ca.AddComponent<RemoveAfterParentDie>();  

                             var car = new GameObject("car");
                             car.transform.SetParent(Instance.transform.Find("Head"));
                             car.transform.localPosition = new Vector3(0.17f, -0.08f);
                             car.transform.localScale = new Vector3(1f, 1f);
                             var carSprite = car.AddComponent<SpriteRenderer>();
                             carSprite.sprite = ModAPI.LoadSprite("Mobs/Illager Nose.png"); 
                             car.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             car.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             car.AddComponent<RemoveAfterParentDie>();                           
                        }
                    }
                }
            );
    //Illager
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Illager",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "Neutral Illager. He appears to be a worker.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Illager.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Illager.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
                        var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        foreach (var body in person.Limbs)
                        {
                            body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                                {
                                    ModAPI.LoadSound("Sfx/Pillager.wav"),
                                    ModAPI.LoadSound("Sfx/Pillager2.wav"),
                                    ModAPI.LoadSound("Sfx/Pillager3.wav"),                                                                                
                                };
                             var ca = new GameObject("ca");
                             ca.transform.SetParent(Instance.transform.Find("Head"));
                             ca.transform.localPosition = new Vector3(0f, 0.18f);
                             ca.transform.localScale = new Vector3(1f, 1f);
                             var caSprite = ca.AddComponent<SpriteRenderer>();
                             caSprite.sprite = ModAPI.LoadSprite("Mobs/Illager Head.png"); 
                             ca.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             ca.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             ca.AddComponent<RemoveAfterParentDie>();   

                             var car = new GameObject("car");
                             car.transform.SetParent(Instance.transform.Find("Head"));
                             car.transform.localPosition = new Vector3(0.17f, -0.11f);
                             car.transform.localScale = new Vector3(1f, 1f);
                             var carSprite = car.AddComponent<SpriteRenderer>();
                             carSprite.sprite = ModAPI.LoadSprite("Mobs/Illager Nose.png"); 
                             car.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             car.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             car.AddComponent<RemoveAfterParentDie>();                             
                        }
                    }
                }
            );
    //Illager 2
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Illager 2",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "Neutral Illager. He also seems to be a worker.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Illager 2.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Illager 2.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
                        var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        foreach (var body in person.Limbs)
                        {
                            body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                                {
                                    ModAPI.LoadSound("Sfx/Vindicator.wav"),
                                    ModAPI.LoadSound("Sfx/Vindicator2.wav"),
                                    ModAPI.LoadSound("Sfx/Vindicator3.wav"),                                                                                
                                };
                             var ca = new GameObject("ca");
                             ca.transform.SetParent(Instance.transform.Find("Head"));
                             ca.transform.localPosition = new Vector3(0f, 0.18f);
                             ca.transform.localScale = new Vector3(1f, 1f);
                             var caSprite = ca.AddComponent<SpriteRenderer>();
                             caSprite.sprite = ModAPI.LoadSprite("Mobs/Illager Head.png"); 
                             ca.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";  
                             ca.AddComponent<RemoveAfterParentDie>();  

                             ca.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground"; 
                             ca.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             ca.AddComponent<RemoveAfterParentDie>();

                             var car = new GameObject("car");
                             car.transform.SetParent(Instance.transform.Find("Head"));
                             car.transform.localPosition = new Vector3(0.17f, -0.11f);
                             car.transform.localScale = new Vector3(1f, 1f);
                             var carSprite = car.AddComponent<SpriteRenderer>();
                             carSprite.sprite = ModAPI.LoadSprite("Mobs/Illager Nose.png"); 
                             car.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             car.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             car.AddComponent<RemoveAfterParentDie>();                           
                        }
                    }
                }
            );
    //Illager 3
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Illager 3",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "Neutral Illager. He also seems to be a worker.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Illager 3.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Illager 3.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
                        var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        foreach (var body in person.Limbs)
                        {
                            body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                                {
                                    ModAPI.LoadSound("Sfx/Vindicator.wav"),
                                    ModAPI.LoadSound("Sfx/Vindicator2.wav"),
                                    ModAPI.LoadSound("Sfx/Vindicator3.wav"),                                                                                
                                };
                             var ca = new GameObject("ca");
                             ca.transform.SetParent(Instance.transform.Find("Head"));
                             ca.transform.localPosition = new Vector3(0f, 0.18f);
                             ca.transform.localScale = new Vector3(1f, 1f);
                             var caSprite = ca.AddComponent<SpriteRenderer>();
                             caSprite.sprite = ModAPI.LoadSprite("Mobs/Illager Head.png"); 
                             ca.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             ca.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             ca.AddComponent<RemoveAfterParentDie>();  

                             var car = new GameObject("car");
                             car.transform.SetParent(Instance.transform.Find("Head"));
                             car.transform.localPosition = new Vector3(0.17f, -0.11f);
                             car.transform.localScale = new Vector3(1f, 1f);
                             var carSprite = car.AddComponent<SpriteRenderer>();
                             carSprite.sprite = ModAPI.LoadSprite("Mobs/Illager Nose.png"); 
                             car.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             car.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             car.AddComponent<RemoveAfterParentDie>();                           
                        }
                    }
                }
            );
    //Iron Golem
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Android"),
                    NameOverride = "Iron Golem",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "Iron Protector of the Villagers!",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Iron Golem.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Iron Golem.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
                        var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(1.4f * (flipped ? -1f : 1f), 1.4f);
                        //Instance.transform.GetChild(Body).transform.localScale = new Vector3(2f, 1f);
                        foreach (var body in person.Limbs)
                        {
                            body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                                {
                                    ModAPI.LoadSound("Sfx/IrGolemhit1.mp3"),
                                    ModAPI.LoadSound("Sfx/IrGolemhit2.mp3"),
                                    ModAPI.LoadSound("Sfx/IrGolemhit3.mp3"),
                                    ModAPI.LoadSound("Sfx/IrGolemhit4.mp3")                                                                            
                                };
                            
                             var ca = new GameObject("ca");
                             ca.transform.SetParent(Instance.transform.Find("Head"));
                             ca.transform.localPosition = new Vector3(0f, 0.18f);
                             ca.transform.localScale = new Vector3(1f, 1f);
                             var caSprite = ca.AddComponent<SpriteRenderer>();
                             caSprite.sprite = ModAPI.LoadSprite("Mobs/Iron Golem Head.png"); 
                             ca.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             ca.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             ca.AddComponent<RemoveAfterParentDie>(); 

                             var car = new GameObject("car");
                             car.transform.SetParent(Instance.transform.Find("Head"));
                             car.transform.localPosition = new Vector3(0.17f, -0.11f);
                             car.transform.localScale = new Vector3(1f, 1f);
                             var carSprite = car.AddComponent<SpriteRenderer>();
                             carSprite.sprite = ModAPI.LoadSprite("Mobs/Iron Golem Nose.png"); 
                             car.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             car.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             car.AddComponent<RemoveAfterParentDie>();                   
                        }
                    }
                }
            );
    //Warden
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Android"),
                    NameOverride = "Warden",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "A very scary blind creature",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Warden.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Warden/Warden.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Warden/Warden Flesh.png");
                        var bone = ModAPI.LoadTexture("Mobs/Warden/Warden Skeleton.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(1.8f * (flipped ? -1f : 1f), 1.8f);
                        //Instance.transform.GetChild(Body).transform.localScale = new Vector3(2f, 1f);
                        
                        foreach (var body in person.Limbs)
                        {
                            body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                                {
                                    ModAPI.LoadSound("Sfx/WardenHurt1.mp3"),
                                    ModAPI.LoadSound("Sfx/WardenHurt2.mp3"),
                                    ModAPI.LoadSound("Sfx/WardenHurt3.mp3"),
                                    ModAPI.LoadSound("Sfx/WardenHurt4.mp3")                                                                            
                                };
                            
                            var WardenHead = new GameObject("ca");
                            WardenHead.transform.SetParent(Instance.transform.Find("Head"));
                            WardenHead.transform.localPosition = new Vector3(0f, 0f);
                            WardenHead.transform.localScale = new Vector3(1f, 1f);
                            var WardenHeadSprite = WardenHead.AddComponent<SpriteRenderer>();
                            WardenHeadSprite.sprite = ModAPI.LoadSprite("Mobs/Warden/Warden Head.png"); 
                            WardenHead.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";  
                            WardenHead.AddComponent<RemoveAfterParentDie>();  

                            var Rog = new GameObject("Rog");
                            Rog.transform.SetParent(Instance.transform.Find("Head"));
                            Rog.transform.localPosition = new Vector3(0f, 0.2f);
                            Rog.transform.localScale = new Vector3(1f, 1f);
                            var RogSprite = Rog.AddComponent<SpriteRenderer>();
                            RogSprite.sprite = ModAPI.LoadSprite("Mobs/Warden/Warden Rog3.png", 180f); 
                            Rog.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                            Rog.GetComponent<SpriteRenderer>().sortingOrder = 1; 
                            Rog.AddComponent<RemoveAfterParentDie>();  

                            body.Health *= 5000f;
                            //body.Properties = ModAPI.FindPhysicalProperties("Human");
                            body.InitialHealth *= 5000f;
                            body.BreakingThreshold = Mathf.Infinity;
                            body.ShotDamageMultiplier *= 1f;
                            body.Vitality *= 25f;
                            body.BloodDecal = null;
                            body.RegenerationSpeed += 25f;
                            body.Numbness *= 2f;
                            body.PhysicalBehaviour.Properties.Flammability = 0f;                           
                        }
                    }
                }
            );
    //Vex
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Vex -MCI",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "A small flying creature created by Evoker. Uses an iron sword.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Vex.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Vex.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
                        var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(0.8f * (flipped ? -1f : 1f), 0.8f);
                        foreach (var body in person.Limbs)
                        {
                            body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                                {
                                    ModAPI.LoadSound("Sfx/Vex.mp3"),
                                    ModAPI.LoadSound("Sfx/Vex2.mp3"),                                                                               
                                };
                             var ca = new GameObject("ca");
                             ca.transform.SetParent(Instance.transform.Find("Body").Find("UpperBody"));
                             ca.transform.localPosition = new Vector3(0f, 0.18f);
                             ca.transform.localScale = new Vector3(1f, 1f);
                             var caSprite = ca.AddComponent<SpriteRenderer>();
                             caSprite.sprite = ModAPI.LoadSprite("Mobs/Vex Wings.png"); 
                             ca.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             ca.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             ca.AddComponent<RemoveAfterParentDie>();                           
                        }
                    }
                }
            );
    //Villager
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Villager",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "Villagers are passive mobs that inhabit villages.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/17.png"),
                    AfterSpawn = (Instance) =>
                    {
                        
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Villager.png");
                        var flesh = ModAPI.LoadTexture("Mobs/vil_meat.png");
                        var bone = ModAPI.LoadTexture("Mobs/vil_Skel.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);

                        foreach (var body in person.Limbs)
                        {






                             body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                                          ModAPI.LoadSound("Sfx/vil1.wav"),
                                                          ModAPI.LoadSound("Sfx/vil2.wav"),
                                                          ModAPI.LoadSound("Sfx/vil3.wav"),
                                                          ModAPI.LoadSound("Sfx/vil4.wav"), 
                                                          ModAPI.LoadSound("Sfx/vil5.wav"),                                                                                  
                                                        }; 

                             body.BaseStrength *= 1f;                            
                             body.Health *= 1f;
                             body.BreakingThreshold *= 1f;  
                             var ca = new GameObject("ca");
                             ca.transform.SetParent(Instance.transform.Find("Head"));
                             ca.transform.localPosition = new Vector3(0f, 0.18f);
                             ca.transform.localScale = new Vector3(1f, 1f);
                             var caSprite = ca.AddComponent<SpriteRenderer>();
                             caSprite.sprite = ModAPI.LoadSprite("Mobs/head.png"); 
                             ca.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             ca.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             ca.AddComponent<RemoveAfterParentDie>();  

                             var car = new GameObject("car");
                             car.transform.SetParent(Instance.transform.Find("Head"));
                             car.transform.localPosition = new Vector3(0.17f, -0.11f);
                             car.transform.localScale = new Vector3(1f, 1f);
                             var carSprite = car.AddComponent<SpriteRenderer>();
                             carSprite.sprite = ModAPI.LoadSprite("Mobs/nose.png"); 
                             car.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             car.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             car.AddComponent<RemoveAfterParentDie>();                          
                        }
                    }
                }
            );
    //Wandering Trader
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Wandering Trader",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "He wanders in search of a better life. Unfortunately, he ended up with you.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Wandering Trader.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Wandering Trader.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
                        var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        foreach (var body in person.Limbs)
                        {
                            body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                                {
                                    ModAPI.LoadSound("Sfx/Wandering Trader.mp3"),
                                    ModAPI.LoadSound("Sfx/Wandering Trader2.mp3"),
                                    ModAPI.LoadSound("Sfx/Wandering Trader3.mp3"),     
                                    ModAPI.LoadSound("Sfx/Wandering Trader4.mp3"),                                                                             
                                };  
                            body.BaseStrength *= 1f;                            
                             body.Health *= 1f;
                             body.BreakingThreshold *= 1f;  
                             var ca = new GameObject("ca");
                             ca.transform.SetParent(Instance.transform.Find("Head"));
                             ca.transform.localPosition = new Vector3(0f, 0.18f);
                             ca.transform.localScale = new Vector3(1f, 1f);
                             var caSprite = ca.AddComponent<SpriteRenderer>();
                             caSprite.sprite = ModAPI.LoadSprite("Mobs/Wandering Trader Head.png"); 
                             ca.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             ca.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             ca.AddComponent<RemoveAfterParentDie>();  

                             var car = new GameObject("car");
                             car.transform.SetParent(Instance.transform.Find("Head"));
                             car.transform.localPosition = new Vector3(0.17f, -0.11f);
                             car.transform.localScale = new Vector3(1f, 1f);
                             var carSprite = car.AddComponent<SpriteRenderer>();
                             carSprite.sprite = ModAPI.LoadSprite("Mobs/nose.png"); 
                             car.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             car.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             car.AddComponent<RemoveAfterParentDie>();                  
                        }
                    }
                }
            );
    //Zombie Villager
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Zombie Villager",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "A Villager who was infected by a zombie virus. What a shame.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Zombie Villager.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Zombie Villager.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
                        var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        foreach (var body in person.Limbs)
                        {
                            body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                                {
                                    ModAPI.LoadSound("Sfx/ZombieVillagerHurt1.mp3"),
                                    ModAPI.LoadSound("Sfx/ZombieVillagerHurt2.mp3"),                                                                            
                                };  
                            body.BaseStrength *= 1f;                            
                             body.Health *= 1f;
                             body.BreakingThreshold *= 1f;  
                             var ca = new GameObject("ca");
                             ca.transform.SetParent(Instance.transform.Find("Head"));
                             ca.transform.localPosition = new Vector3(0f, 0.18f);
                             ca.transform.localScale = new Vector3(1f, 1f);
                             var caSprite = ca.AddComponent<SpriteRenderer>();
                             caSprite.sprite = ModAPI.LoadSprite("Mobs/Zombie Villager Head.png"); 
                             ca.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             ca.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             ca.AddComponent<RemoveAfterParentDie>();   

                             var car = new GameObject("car");
                             car.transform.SetParent(Instance.transform.Find("Head"));
                             car.transform.localPosition = new Vector3(0.17f, -0.11f);
                             car.transform.localScale = new Vector3(1f, 1f);
                             var carSprite = car.AddComponent<SpriteRenderer>();
                             carSprite.sprite = ModAPI.LoadSprite("Mobs/Zombie Nose.png"); 
                             car.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             car.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             car.AddComponent<RemoveAfterParentDie>();                          
                        }
                    }
                }
            );        
    //Witch
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Witch",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "A witch is an uncommon hostile mob that uses harmful splash potions as ranged weapons and beneficial positive potions on itself.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/25.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/witch.png");
                        var flesh = ModAPI.LoadTexture("Mobs/vil_meat.png");
                        var bone = ModAPI.LoadTexture("Mobs/vil_Skel.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        foreach (var body in person.Limbs)
                        {
                            body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                                          ModAPI.LoadSound("Sfx/witch1.wav"),
                                                          ModAPI.LoadSound("Sfx/witch2.wav"),
                                                          ModAPI.LoadSound("Sfx/witch3.wav"),
                                                          ModAPI.LoadSound("Sfx/witch4.wav"), 
                                                          ModAPI.LoadSound("Sfx/witch5.wav"),                                                                                  
                            };

                             body.BaseStrength *= 1f;                            
                             body.Health *= 1f;
                             body.BreakingThreshold *= 1f;  
                             var ca = new GameObject("ca");
                             ca.transform.SetParent(Instance.transform.Find("Head"));
                             ca.transform.localPosition = new Vector3(0f, 0.32f);
                             ca.transform.localScale = new Vector3(1f, 1f);
                             var caSprite = ca.AddComponent<SpriteRenderer>();
                             caSprite.sprite = ModAPI.LoadSprite("Mobs/hat.png"); 
                             ca.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             ca.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             ca.AddComponent<RemoveAfterParentDie>();   

                             var car = new GameObject("car");
                             car.transform.SetParent(Instance.transform.Find("Head"));
                             car.transform.localPosition = new Vector3(0.17f, -0.11f);
                             car.transform.localScale = new Vector3(1f, 1f);
                             var carSprite = car.AddComponent<SpriteRenderer>();
                             carSprite.sprite = ModAPI.LoadSprite("Mobs/nose2.png"); 
                             car.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             car.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             car.AddComponent<RemoveAfterParentDie>();                    
                        }
                    }
                }
            );
    //Piglin
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Piglin",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "A common walking pig. He is very fond of gold.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Piglin.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Nether/Piglin.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
                        var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        foreach (var body in person.Limbs)
                        {
                            body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                                {
                                    ModAPI.LoadSound("Sfx/Piglinhurt1.mp3"),
                                    ModAPI.LoadSound("Sfx/Piglinhurt2.mp3"),
                                    ModAPI.LoadSound("Sfx/Piglinhurt3.mp3"),                                                                                
                                };  
                             var car = new GameObject("car");
                             car.transform.SetParent(Instance.transform.Find("Head"));
                             car.transform.localPosition = new Vector3(0f, 0f);
                             car.transform.localScale = new Vector3(1f, 1f);
                             var carSprite = car.AddComponent<SpriteRenderer>();
                             carSprite.sprite = ModAPI.LoadSprite("Mobs/Nether/Piglin Nose.png"); 
                             car.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             car.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             car.AddComponent<RemoveAfterParentDie>();                         
                        }
                    }
                }
            );
    //Piglin Brute
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Piglin Brute",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "The Brutal Walking Pig. He's ready to tear you apart.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Piglin Brute.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Nether/Piglin Brute.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
                        var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        foreach (var body in person.Limbs)
                        {
                            body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                                {
                                    ModAPI.LoadSound("Sfx/PiglinBrutehurt1.mp3"),
                                    ModAPI.LoadSound("Sfx/PiglinBrutehurt2.mp3"),
                                    ModAPI.LoadSound("Sfx/PiglinBrutehurt3.mp3"),   
                                    ModAPI.LoadSound("Sfx/PiglinBrutehurt4.mp3"),                                                                              
                                };  
                             var car = new GameObject("car");
                             car.transform.SetParent(Instance.transform.Find("Head"));
                             car.transform.localPosition = new Vector3(0f, 0f);
                             car.transform.localScale = new Vector3(1f, 1f);
                             var carSprite = car.AddComponent<SpriteRenderer>();
                             carSprite.sprite = ModAPI.LoadSprite("Mobs/Nether/Piglin Nose.png"); 
                             car.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             car.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             car.AddComponent<RemoveAfterParentDie>();                          
                        }
                    }
                }
            );
    //Zombified Piglin
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Zombified Piglin",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "Good old pigman.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Zombified Piglin.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Nether/Zombified Piglin.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
                        var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        foreach (var body in person.Limbs)
                        {
                            body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                                {
                                    ModAPI.LoadSound("Sfx/zpighurt1.mp3"),
                                    ModAPI.LoadSound("Sfx/zpighurt2.mp3"),                                                                             
                                };  
                             var car = new GameObject("car");
                             car.transform.SetParent(Instance.transform.Find("Head"));
                             car.transform.localPosition = new Vector3(0f, 0f);
                             car.transform.localScale = new Vector3(1f, 1f);
                             var carSprite = car.AddComponent<SpriteRenderer>();
                             carSprite.sprite = ModAPI.LoadSprite("Mobs/Nether/Piglin Nose.png"); 
                             car.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                             car.GetComponent<SpriteRenderer>().sortingOrder = 1;   
                             car.AddComponent<RemoveAfterParentDie>();                          
                        }
                    }
                }
            );
    //Creeper
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Creeper",
                    DescriptionOverride = "Press F on his head to Explode (aw man)",
                    NameToOrderByOverride = "A",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/1.png"),
                    AfterSpawn = (Instance) =>
                    {
                        
                        var person = Instance.GetComponent<PersonBehaviour>();

                        var skin = ModAPI.LoadTexture("Mobs/creeper.png");
                        var flesh = ModAPI.LoadTexture("Mobs/creeper_meat.png");
                        var bone = ModAPI.LoadTexture("Mobs/creeper_skel.png");
                        person.SetBodyTextures(skin, flesh, bone, 1);

                        person.SetBruiseColor(164, 21, 64); //main bruise colour. purple-ish by default
                        person.SetSecondBruiseColor(164, 21, 21); //second bruise colour. red by default
                        person.SetThirdBruiseColor(131, 18, 18); // third bruise colour. light yellow by default
                        person.SetBloodColour(172, 18, 18); // blood clour. dark red by default
                        person.SetRottenColour(87, 69, 69); // rotten/zombie colour. light yellow/green by default
                        foreach (var body in person.Limbs) 
                        {

                             var legLeft = Instance.transform.Find("FrontLeg").Find("UpperLegFront");
                             var legRight = Instance.transform.Find("BackLeg").Find("UpperLeg");

                             legLeft.localPosition = new Vector3(0.15f, -0.47f, 0f);
                             legRight.localPosition = new Vector3(-0.15f, -0.47f, 0f);

                             var breath = new GameObject("breath");
                             breath.transform.SetParent(Instance.transform.Find("Body").Find("MiddleBody"));
                             breath.transform.localPosition = new Vector3(0, 0f);
                             breath.transform.localScale = new Vector3(1f, 1f);
                             var breathSprite = breath.AddComponent<SpriteRenderer>();
                             breathSprite.sprite = ModAPI.LoadSprite("Mobs/none.png");
                             var breathexpl = breath.AddComponent<ExplosiveBehaviour>().Range = 4f;
                             AudioSource audioSource;
			     AudioClip sound;
                             breath.AddComponent<UseEventTrigger>().Action = () => { 
                             sound = ModAPI.LoadSound("Sfx/css.wav");
							audioSource = breath.AddComponent<AudioSource>();
							audioSource.minDistance = 2;
							audioSource.maxDistance = 50;
							audioSource.clip = sound;
							audioSource.Play();
                                                        audioSource.volume = 0.05f;
                                                                                    };
                                    
                                                         

                             body.BaseStrength *= 0.8f;                            
                             body.Health *= 5f;
                             body.BreakingThreshold *= 50f;
                             body.transform.root.localScale *= 1.04f;  
                             body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
							{
                                                          ModAPI.LoadSound("Sfx/crep1.wav"),
                                                          ModAPI.LoadSound("Sfx/crep2.wav"),
                                                          ModAPI.LoadSound("Sfx/crep3.wav"),
                                                          ModAPI.LoadSound("Sfx/crep4.wav"),                                                                                  
                                                        };   
                                                             
                        //destroy EVERYTHING                        
                        Instance.transform.Find("BackArm").Find("UpperArm").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("BackArm").Find("LowerArm").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontArm").Find("UpperArmFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontArm").Find("LowerArmFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();                       
                        Instance.transform.Find("Head").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("BackLeg").Find("LowerLeg").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontLeg").Find("LowerLegFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();                        
                        Instance.transform.Find("BackLeg").Find("Foot").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontLeg").Find("FootFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("Body").Find("UpperBody").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();                        
                        }

                        
                        var Leg1 = Instance.transform.Find("FrontLeg").Find("UpperLegFront").gameObject;
                        foreach(var hitbox1 in Leg1.GetComponents<Collider2D>()) { GameObject.Destroy(hitbox1); }
                        var floop1 = Leg1.AddComponent<BoxCollider2D>();

                        floop1.size = Vector2.one * 0.18f;

                        floop1.offset = new Vector2(0f, 0.11f);                        


                        var Leg2 = Instance.transform.Find("BackLeg").Find("UpperLeg").gameObject;
                        foreach(var hitbox2 in Leg2.GetComponents<Collider2D>()) { GameObject.Destroy(hitbox2); }
                        var floop2 = Leg2.AddComponent<BoxCollider2D>();

                        floop2.size = Vector2.one * 0.18f;

                        floop2.offset = new Vector2(0f, 0.11f); 

                        var fleebHead = Instance.transform.Find("Body").Find("MiddleBody").gameObject;
                        fleebHead.name = "Head"; //this is so it works with the armor mod :)
                        fleebHead.GetComponent<LimbBehaviour>().HasBrain = true;
                        fleebHead.GetComponent<CirculationBehaviour>().WasInitiallyPumping = true;
                        fleebHead.GetComponent<CirculationBehaviour>().IsPump = true;

                        foreach(var hitbox in fleebHead.GetComponents<Collider2D>()) { GameObject.Destroy(hitbox); }
                        var floop = fleebHead.AddComponent<BoxCollider2D>();
                        floop.size = Vector2.one * 0.25f;
                        floop.offset = new Vector2(0f, -0.05f);

                        var fleebth = fleebHead.AddComponent<GripBehaviour>();
                        fleebth.GripPosition = new Vector3(0.15f, -0.15f, 0.45f);  

                                    
                    }
                }
            );
    //Slimy
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Slime",
                    DescriptionOverride = "Squishy creature",
                    NameToOrderByOverride = "A",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/2.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();

                        var skin = ModAPI.LoadTexture("Mobs/slime.png");
                        var flesh = ModAPI.LoadTexture("Mobs/slime2.png");
                        var bone = ModAPI.LoadTexture("Mobs/slime3.png");
                        person.SetBodyTextures(skin, flesh, bone, 1);

                        person.SetBruiseColor(0, 5, 0); //main bruise colour. purple-ish by default
                        person.SetSecondBruiseColor(0, 10, 0); //second bruise colour. red by default
                        person.SetThirdBruiseColor(0, 50, 0); // third bruise colour. light yellow by default
                        person.SetBloodColour(0, 50, 0); // blood clour. dark red by default
                        person.SetRottenColour(50, 50, 50); // rotten/zombie colour. light yellow/green by default
                        foreach (var body in person.Limbs) 
                        {
                             body.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Gorse");
                             body.BaseStrength *= 0.5f;                            
                             body.Health *= 5f;
                             body.BreakingThreshold *= 50f;
                             body.transform.root.localScale *= 1.05f; 
                             body.BloodDecal = null;
                             body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
							{
                                                          ModAPI.LoadSound("Sfx/slime1.wav"),
                                                          ModAPI.LoadSound("Sfx/slime2.wav"),
                                                          ModAPI.LoadSound("Sfx/slime3.wav"),
                                                          ModAPI.LoadSound("Sfx/slime4.wav"),                                                                                  
                                                        };                    

                        //destroy EVERYTHING                        
                        Instance.transform.Find("BackArm").Find("UpperArm").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("BackArm").Find("LowerArm").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontArm").Find("UpperArmFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontArm").Find("LowerArmFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("Body").Find("UpperBody").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("Body").Find("MiddleBody").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("Head").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("BackLeg").Find("UpperLeg").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("BackLeg").Find("LowerLeg").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontLeg").Find("UpperLegFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontLeg").Find("LowerLegFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate(); 
                        Instance.transform.Find("BackLeg").Find("Foot").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontLeg").Find("FootFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();                       
                        }

                        var fleebHead = Instance.transform.Find("Body").Find("LowerBody").gameObject;
                        fleebHead.name = "Head"; //this is so it works with the armor mod :)
                        fleebHead.GetComponent<LimbBehaviour>().HasBrain = true;
                        fleebHead.GetComponent<CirculationBehaviour>().WasInitiallyPumping = true;
                        fleebHead.GetComponent<CirculationBehaviour>().IsPump = true;

                        foreach(var hitbox in fleebHead.GetComponents<Collider2D>()) { GameObject.Destroy(hitbox); }
                        var floop = fleebHead.AddComponent<BoxCollider2D>();
                        floop.size = Vector2.one * 0.25f;
                        floop.offset = new Vector2(0f, -0.05f);

                        var fleebth = fleebHead.AddComponent<GripBehaviour>();
                        fleebth.GripPosition = new Vector3(0.15f, -0.15f, 0.45f);                      
                    }
                }
            );
    //Magma Cube
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Magma Cube",
                    DescriptionOverride = "lava creature",
                    NameToOrderByOverride = "A",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Magma Cube.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();

                        var skin = ModAPI.LoadTexture("Mobs/Magma Cube.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Magma Cube.png");
                        var bone = ModAPI.LoadTexture("Mobs/Magma Cube.png");
                        person.SetBodyTextures(skin, flesh, bone, 1);

                        person.SetBruiseColor(0, 5, 0); //main bruise colour. purple-ish by default
                        person.SetSecondBruiseColor(0, 10, 0); //second bruise colour. red by default
                        person.SetThirdBruiseColor(0, 50, 0); // third bruise colour. light yellow by default
                        person.SetBloodColour(0, 50, 0); // blood clour. dark red by default
                        person.SetRottenColour(50, 50, 50); // rotten/zombie colour. light yellow/green by default
                        foreach (var body in person.Limbs) 
                        {
                             body.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Gorse");
                             body.BaseStrength *= 0.5f;                            
                             body.Health *= 5f;
                             body.BreakingThreshold *= 50f;
                             body.transform.root.localScale *= 1.05f; 
                             body.BloodDecal = null;
                             body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
							{
                                                          ModAPI.LoadSound("Sfx/jump1.mp3"),
                                                          ModAPI.LoadSound("Sfx/jump2.mp3"),
                                                          ModAPI.LoadSound("Sfx/jump3.mp3"),
                                                          ModAPI.LoadSound("Sfx/jump4.mp3"),                                                                                 
                                                        };                    

                        //destroy EVERYTHING                        
                        Instance.transform.Find("BackArm").Find("UpperArm").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("BackArm").Find("LowerArm").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontArm").Find("UpperArmFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontArm").Find("LowerArmFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("Body").Find("UpperBody").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("Body").Find("MiddleBody").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("Head").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("BackLeg").Find("UpperLeg").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("BackLeg").Find("LowerLeg").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontLeg").Find("UpperLegFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontLeg").Find("LowerLegFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate(); 
                        Instance.transform.Find("BackLeg").Find("Foot").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontLeg").Find("FootFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();                       
                        }

                        var fleebHead = Instance.transform.Find("Body").Find("LowerBody").gameObject;
                        fleebHead.name = "Head"; //this is so it works with the armor mod :)
                        fleebHead.GetComponent<LimbBehaviour>().HasBrain = true;
                        fleebHead.GetComponent<CirculationBehaviour>().WasInitiallyPumping = true;
                        fleebHead.GetComponent<CirculationBehaviour>().IsPump = true;

                        foreach(var hitbox in fleebHead.GetComponents<Collider2D>()) { GameObject.Destroy(hitbox); }
                        var floop = fleebHead.AddComponent<BoxCollider2D>();
                        floop.size = Vector2.one * 0.25f;
                        floop.offset = new Vector2(0f, -0.05f);

                        var fleebth = fleebHead.AddComponent<GripBehaviour>();
                        fleebth.GripPosition = new Vector3(0.15f, -0.15f, 0.45f);                      
                    }
                }
            );
    //Enderman
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Enderman",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "This creature travels through worlds. Do not look him in the eye.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/8.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();
                        var skin = ModAPI.LoadTexture("Mobs/Enderman.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Enderman.png");
                        var bone = ModAPI.LoadTexture("Mobs/Enderman.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
                        person.SetBruiseColor(0, 0, 0); //main bruise colour. purple-ish by default
                        person.SetSecondBruiseColor(0, 0, 0); //second bruise colour. red by default
                        person.SetThirdBruiseColor(0, 0, 0); // third bruise colour. light yellow by default
                        person.SetBloodColour(0, 0, 0); // blood clour. dark red by default
                        person.SetRottenColour(0, 0, 0); // rotten/zombie colour. light yellow/green by default
                        foreach (var body in person.Limbs)
                        {
                             body.BaseStrength *= 1.1f;                            
                             body.Health *= 10f;
                             body.BreakingThreshold *= 1f;
                             body.transform.root.localScale *= 1.01f;    
                              body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                                          ModAPI.LoadSound("Sfx/end1.wav"),
                                                          ModAPI.LoadSound("Sfx/end2.wav"),
                                                          ModAPI.LoadSound("Sfx/end3.wav"),
                                                          ModAPI.LoadSound("Sfx/end4.wav"), 
                                                          ModAPI.LoadSound("Sfx/end5.wav"),     
                                                          ModAPI.LoadSound("Sfx/end6.wav"),                                                                                 
                                                        };                            
                        }
                    }
                }
            );
    //Old Ghast
		ModAPI.Register(
			new Modification()
			{
				OriginalItem = ModAPI.FindSpawnable("Human"),
				NameOverride = "Ghast",
				DescriptionOverride = "A ghast is a large white floating ghost-like hostile mob.",
				NameToOrderByOverride = "A",
				CategoryOverride = ModAPI.FindCategory("Minecraft"),
				ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/37.png"),
				AfterSpawn = (Instance) =>
				{
					var person = Instance.GetComponent<PersonBehaviour>();

					var skin = ModAPI.LoadTexture("Mobs/ghaste.png");
					var flesh = ModAPI.LoadTexture("Mobs/ghaste.png");
					var bone = ModAPI.LoadTexture("Mobs/ghaste.png");
					person.SetBodyTextures(skin, flesh, bone, 1);


					var legLeft = Instance.transform.Find("FrontLeg").Find("UpperLegFront");
					var legRight = Instance.transform.Find("BackLeg").Find("UpperLeg");

					legLeft.localPosition = new Vector3(0.35f, -0.94f, 0f);
					legLeft.localScale = new Vector3(1f, 1.35f);
					legRight.localPosition = new Vector3(-0.35f, -0.87f, 0f);

					var cac = new GameObject("cac");
					cac.transform.SetParent(Instance.transform.Find("Body").Find("LowerBody"));
					cac.transform.localPosition = new Vector3(0f, 0f);
					cac.transform.localScale = new Vector3(3f, 3f);
					var cacSprite = cac.AddComponent<SpriteRenderer>();
					cacSprite.sprite = ModAPI.LoadSprite("Mobs/ghast.png"); 
					cac.GetComponent<SpriteRenderer>().sortingLayerName = "Top"; 
							

					person.SetBruiseColor(0, 5, 0); //main bruise colour. purple-ish by default
					person.SetSecondBruiseColor(0, 10, 0); //second bruise colour. red by default
					person.SetThirdBruiseColor(0, 50, 0); // third bruise colour. light yellow by default
					person.SetBloodColour(0, 50, 0); // blood clour. dark red by default
					person.SetRottenColour(50, 50, 50); // rotten/zombie colour. light yellow/green by default
					
					//destroy EVERYTHING                        
					Instance.transform.Find("BackArm").Find("UpperArm").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
					Instance.transform.Find("BackArm").Find("LowerArm").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
					Instance.transform.Find("FrontArm").Find("UpperArmFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
					Instance.transform.Find("FrontArm").Find("LowerArmFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
					Instance.transform.Find("Body").Find("UpperBody").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
					Instance.transform.Find("Body").Find("MiddleBody").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
					Instance.transform.Find("Head").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
					Instance.transform.Find("BackLeg").Find("Foot").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
					Instance.transform.Find("FrontLeg").Find("FootFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();   
					Instance.transform.Find("FrontLeg").Find("LowerLegFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();          

					GameObject fleebHead = Instance.transform.Find("Body").Find("LowerBody").gameObject;
					fleebHead.name = "Head"; //this is so it works with the armor mod :)
					fleebHead.GetComponent<LimbBehaviour>().HasBrain = true;
					fleebHead.GetComponent<CirculationBehaviour>().WasInitiallyPumping = true;
					fleebHead.GetComponent<CirculationBehaviour>().IsPump = true;

					foreach(var hitbox in fleebHead.GetComponents<Collider2D>()) { GameObject.Destroy(hitbox); }
					var floop = fleebHead.AddComponent<BoxCollider2D>();
					floop.size = Vector2.one * 1.35f;
					floop.offset = new Vector2(0f, 0f);

					var fleebth = fleebHead.AddComponent<GripBehaviour>();
					fleebth.GripPosition = new Vector3(0.15f, -0.15f, 0.45f);    

					foreach (var body in person.Limbs) 
					{
						body.Health *= 85485484.6f;
						body.InitialHealth *= 85485484.6f;
						body.BreakingThreshold = Mathf.Infinity;
						body.PhysicalBehaviour.Charge = 0f;
						body.ShotDamageMultiplier *= 0f;
						body.Vitality *= 3f;
						body.BloodDecal = null;
						body.RegenerationSpeed += 10f;
						body.Numbness *= 3f;
						body.CirculationBehaviour.BloodFlow *= 0f;
						body.ImpactPainMultiplier *= 0f;
						body.IsLethalToBreak = false;
						body.PhysicalBehaviour.ReflectsLasers = true;
						body.transform.root.localScale *= 1.02f; 
						body.gameObject.AddComponent<BeeFlyBehaviour>();
					}          
				}
			}
		);
    //Chicken
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Chicken",
                    DescriptionOverride = "Chickens are common passive mobs found in most grassy biomes, and are the main source of chicken, feathers and eggs.",
                    NameToOrderByOverride = "A",
                    CategoryOverride = ModAPI.FindCategory("deleted"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/1.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();

                        var skin = ModAPI.LoadTexture("Mobs/chicken.png");
                        var flesh = ModAPI.LoadTexture("Mobs/chicken.png");
                        var bone = ModAPI.LoadTexture("Mobs/chicken.png");
                        person.SetBodyTextures(skin, flesh, bone, 1);

                        person.SetBruiseColor(0, 0, 0); //main bruise colour. purple-ish by default
                        person.SetSecondBruiseColor(0, 0, 0); //second bruise colour. red by default
                        person.SetThirdBruiseColor(0, 0, 0); // third bruise colour. light yellow by default
                        person.SetBloodColour(0, 0, 0); // blood clour. dark red by default
                        person.SetRottenColour(0, 0, 0); // rotten/zombie colour. light yellow/green by default
                        foreach (var body in person.Limbs) 
                        {

                             var head = Instance.transform.Find("Body").Find("MiddleBody");
                             

                             head.localPosition = new Vector3(0f, 0f, 0f);                             



                             body.BaseStrength *= 0.5f;                            
                             body.Health *= 5f;
                             body.BreakingThreshold *= 50f;
                             body.transform.root.localScale *= 1f;                    

                        //destroy EVERYTHING                        
                        Instance.transform.Find("BackArm").Find("UpperArm").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("BackArm").Find("LowerArm").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontArm").Find("UpperArmFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontArm").Find("LowerArmFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();                                                                     
                        Instance.transform.Find("BackLeg").Find("LowerLeg").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();              
                        Instance.transform.Find("FrontLeg").Find("LowerLegFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate(); 
                        Instance.transform.Find("BackLeg").Find("Foot").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontLeg").Find("FootFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate(); 
                        Instance.transform.Find("Body").Find("UpperBody").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();                                               
                        }

                        var fleebHead = Instance.transform.Find("Body").Find("MiddleBody").gameObject;
                        fleebHead.name = "Head"; //this is so it works with the armor mod :)
                        fleebHead.GetComponent<LimbBehaviour>().HasBrain = true;
                        fleebHead.GetComponent<CirculationBehaviour>().WasInitiallyPumping = true;
                        fleebHead.GetComponent<CirculationBehaviour>().IsPump = true;

                        foreach(var hitbox in fleebHead.GetComponents<Collider2D>()) { GameObject.Destroy(hitbox); }
                        var floop = fleebHead.AddComponent<BoxCollider2D>();
                        floop.size = Vector2.one * 0.25f;
                        floop.offset = new Vector2(0f, 0.05f);

                        var fleebth = fleebHead.AddComponent<GripBehaviour>();
                        fleebth.GripPosition = new Vector3(0.15f, -0.15f, 0.45f);                      
                    }
                }
            );
    //Pig
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Pig",
                    DescriptionOverride = "A pig is a common passive mob found in most grassy biomes. Pigs are a source of porkchops and can be equipped with a saddle to be driven.",
                    NameToOrderByOverride = "A",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/16.png"),
                    AfterSpawn = (Instance) =>
                    {             
                        var person = Instance.GetComponent<PersonBehaviour>();

                        var skin = ModAPI.LoadTexture("Mobs/Pig/Pig/pig.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Pig/Pig/pig_meat.png");
                        var bone = ModAPI.LoadTexture("Mobs/Pig/Pig/pig_skeleton.png");
                        Sprite PigNoseSprite = ModAPI.LoadSprite("Mobs/Pig/Pig/pignose.png");
                        Sprite PigBack = ModAPI.LoadSprite("Mobs/Pig/Pig/pigback.png");
                        person.SetBodyTextures(skin, flesh, bone, 1);

                        person.SetBruiseColor(164, 21, 64); //main bruise colour. purple-ish by default
                        person.SetSecondBruiseColor(164, 21, 21); //second bruise colour. red by default
                        person.SetThirdBruiseColor(131, 18, 18); // third bruise colour. light yellow by default
                        person.SetBloodColour(172, 18, 18); // blood clour. dark red by default
                        person.SetRottenColour(87, 69, 69); // rotten/zombie colour. light yellow/green by default
                        foreach (var body in person.Limbs) 
                        {                                      
                            body.BaseStrength *= 0.5f;                            
                            body.Health *= 1f;
                            body.BreakingThreshold *= 50f;
                            body.transform.root.localScale *= 1.04f;  
                            body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/pig5.wav"),
                                ModAPI.LoadSound("Sfx/pig6.wav"),
                                ModAPI.LoadSound("Sfx/pig7.wav"),
                                ModAPI.LoadSound("Sfx/pig8.wav"), 
                                ModAPI.LoadSound("Sfx/pig9.wav"),                                                                                    
                            }; 
                        var legRight = Instance.transform.Find("FrontLeg").Find("UpperLegFront");
                        var legLeft = Instance.transform.Find("BackLeg").Find("UpperLeg");
                        var Head1 = Instance.transform.Find("Body").Find("MiddleBody");
                        
                        legLeft.localPosition = new Vector3(0.06f, -0.447f, 0f);
                        legRight.localPosition = new Vector3(-0.26f, -0.45f, 0f);
                        Head1.localPosition = new Vector3(0.20f, -0.05f, 0f);

                        var Nose = new GameObject("Nose");
                        Nose.transform.SetParent(Instance.transform.Find("Body").Find("MiddleBody"));
                        Nose.transform.localPosition = new Vector3(0.142f, -0.03f);
                        Nose.transform.localScale = new Vector3(1f, 1f);
                        var NoseSprite = Nose.AddComponent<SpriteRenderer>();
                        NoseSprite.sprite = PigNoseSprite; 
                        Nose.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                                                            
                    //destroy EVERYTHING                        
                        Instance.transform.Find("BackArm").Find("UpperArm").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("BackArm").Find("LowerArm").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontArm").Find("UpperArmFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontArm").Find("LowerArmFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();                       
                        Instance.transform.Find("Head").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("BackLeg").Find("LowerLeg").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontLeg").Find("LowerLegFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();                        
                        Instance.transform.Find("BackLeg").Find("Foot").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontLeg").Find("FootFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("Body").Find("UpperBody").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();   
                    //                     
                    }

                        
                        var Leg1 = Instance.transform.Find("FrontLeg").Find("UpperLegFront").gameObject;
                        foreach(var hitbox1 in Leg1.GetComponents<Collider2D>()) { GameObject.Destroy(hitbox1); }
                        var floop1 = Leg1.AddComponent<BoxCollider2D>();

                        floop1.size = new Vector2(0.1428571f, 0.2010305f);
                        floop1.offset = new Vector2(0f, 0.1280561f);                        


                        var Leg2 = Instance.transform.Find("BackLeg").Find("UpperLeg").gameObject;
                        foreach(var hitbox2 in Leg2.GetComponents<Collider2D>()) { GameObject.Destroy(hitbox2); }
                        var floop2 = Leg2.AddComponent<BoxCollider2D>();

                        floop2.size = new Vector2(0.1428571f, 0.2010305f);
                        floop2.offset = new Vector2(0f, 0.1280561f); 

                        var Heads = Instance.transform.Find("Body").Find("LowerBody").gameObject;
                            foreach(var hitbox1 in Heads.GetComponents<Collider2D>()) { GameObject.Destroy(hitbox1); }
                            var hada = Heads.AddComponent<BoxCollider2D>();

                            //hada.offset = new Vector2(0f, -0.05732039f);
                            //hada.size = new Vector3(0.2571429f, 0.2282164f);

                            hada.offset = new Vector2(-0.1000142f, -0.05732039f);
                            hada.size = new Vector3(0.4571714f, 0.2282164f);

                        var fleebHead = Instance.transform.Find("Body").Find("MiddleBody").gameObject;
                        fleebHead.name = "Head"; //this is so it works with the armor mod :)
                        fleebHead.GetComponent<LimbBehaviour>().HasBrain = true;
                        fleebHead.GetComponent<CirculationBehaviour>().WasInitiallyPumping = true;
                        fleebHead.GetComponent<CirculationBehaviour>().IsPump = true;
                        fleebHead.GetComponent<SpriteRenderer>().sortingLayerName = "Back"; 

                        foreach(var hitbox in fleebHead.GetComponents<Collider2D>()) { GameObject.Destroy(hitbox); }
                        var floop = fleebHead.AddComponent<BoxCollider2D>();
                        floop.size = new Vector2(0.2571429f, 0.2571429f);
                        floop.offset = new Vector2(0f, 0f);

                        var fleebth = fleebHead.AddComponent<GripBehaviour>();
                        fleebth.GripPosition = new Vector3(0.15f, -0.15f, 0.45f);  

                             var ca = new GameObject("ca");
                             ca.transform.SetParent(Instance.transform.Find("Body").Find("LowerBody"));
                             ca.transform.localPosition = new Vector3(-0.2275f, -0.057f);
                             ca.transform.localScale = new Vector3(1f, 1f);
                             var caSprite = ca.AddComponent<SpriteRenderer>();
                             caSprite.sprite = PigBack;
                             ca.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";                
                    }
                }
            );
    //Muddy Pig
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Muddy Pig",
                    DescriptionOverride = "A pig is a common passive mob found in most grassy biomes. Pigs are a source of porkchops and can be equipped with a saddle to be driven.",
                    NameToOrderByOverride = "A",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Muddy Pig.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var person = Instance.GetComponent<PersonBehaviour>();

                        var skin = ModAPI.LoadTexture("Mobs/Pig/Muddy Pig/pig.png");
                        var flesh = ModAPI.LoadTexture("Mobs/Pig/Muddy Pig/pig_meat.png");
                        var bone = ModAPI.LoadTexture("Mobs/Pig/Muddy Pig/pig_skeleton.png");
                        Sprite PigNoseSprite = ModAPI.LoadSprite("Mobs/Pig/Muddy Pig/pignose.png");
                        Sprite PigBack = ModAPI.LoadSprite("Mobs/Pig/Muddy Pig/pigback.png");
                        person.SetBodyTextures(skin, flesh, bone, 1);

                        person.SetBruiseColor(164, 21, 64); //main bruise colour. purple-ish by default
                        person.SetSecondBruiseColor(164, 21, 21); //second bruise colour. red by default
                        person.SetThirdBruiseColor(131, 18, 18); // third bruise colour. light yellow by default
                        person.SetBloodColour(172, 18, 18); // blood clour. dark red by default
                        person.SetRottenColour(87, 69, 69); // rotten/zombie colour. light yellow/green by default
                        foreach (var body in person.Limbs) 
                        {                                      
                            body.BaseStrength *= 0.5f;                            
                            body.Health *= 1f;
                            body.BreakingThreshold *= 50f;
                            body.transform.root.localScale *= 1.04f;  
                            body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/pig5.wav"),
                                ModAPI.LoadSound("Sfx/pig6.wav"),
                                ModAPI.LoadSound("Sfx/pig7.wav"),
                                ModAPI.LoadSound("Sfx/pig8.wav"), 
                                ModAPI.LoadSound("Sfx/pig9.wav"),                                                                                    
                            }; 
                        var legRight = Instance.transform.Find("FrontLeg").Find("UpperLegFront");
                        var legLeft = Instance.transform.Find("BackLeg").Find("UpperLeg");
                        var Head1 = Instance.transform.Find("Body").Find("MiddleBody");
                        
                        legLeft.localPosition = new Vector3(0.06f, -0.447f, 0f);
                        legRight.localPosition = new Vector3(-0.26f, -0.45f, 0f);
                        Head1.localPosition = new Vector3(0.20f, -0.05f, 0f);

                        var Nose = new GameObject("Nose");
                        Nose.transform.SetParent(Instance.transform.Find("Body").Find("MiddleBody"));
                        Nose.transform.localPosition = new Vector3(0.142f, -0.03f);
                        Nose.transform.localScale = new Vector3(1f, 1f);
                        var NoseSprite = Nose.AddComponent<SpriteRenderer>();
                        NoseSprite.sprite = PigNoseSprite; 
                        Nose.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                                                            
                    //destroy EVERYTHING                        
                        Instance.transform.Find("BackArm").Find("UpperArm").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("BackArm").Find("LowerArm").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontArm").Find("UpperArmFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontArm").Find("LowerArmFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();                       
                        Instance.transform.Find("Head").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("BackLeg").Find("LowerLeg").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontLeg").Find("LowerLegFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();                        
                        Instance.transform.Find("BackLeg").Find("Foot").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("FrontLeg").Find("FootFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
                        Instance.transform.Find("Body").Find("UpperBody").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();   
                    //                     
                    }

                        
                        var Leg1 = Instance.transform.Find("FrontLeg").Find("UpperLegFront").gameObject;
                        foreach(var hitbox1 in Leg1.GetComponents<Collider2D>()) { GameObject.Destroy(hitbox1); }
                        var floop1 = Leg1.AddComponent<BoxCollider2D>();

                        floop1.size = new Vector2(0.1428571f, 0.2010305f);
                        floop1.offset = new Vector2(0f, 0.1280561f);                        


                        var Leg2 = Instance.transform.Find("BackLeg").Find("UpperLeg").gameObject;
                        foreach(var hitbox2 in Leg2.GetComponents<Collider2D>()) { GameObject.Destroy(hitbox2); }
                        var floop2 = Leg2.AddComponent<BoxCollider2D>();

                        floop2.size = new Vector2(0.1428571f, 0.2010305f);
                        floop2.offset = new Vector2(0f, 0.1280561f); 

                        var Heads = Instance.transform.Find("Body").Find("LowerBody").gameObject;
                            foreach(var hitbox1 in Heads.GetComponents<Collider2D>()) { GameObject.Destroy(hitbox1); }
                            var hada = Heads.AddComponent<BoxCollider2D>();

                            //hada.offset = new Vector2(0f, -0.05732039f);
                            //hada.size = new Vector3(0.2571429f, 0.2282164f);

                            hada.offset = new Vector2(-0.1000142f, -0.05732039f);
                            hada.size = new Vector3(0.4571714f, 0.2282164f);

                        var fleebHead = Instance.transform.Find("Body").Find("MiddleBody").gameObject;
                        fleebHead.name = "Head"; //this is so it works with the armor mod :)
                        fleebHead.GetComponent<LimbBehaviour>().HasBrain = true;
                        fleebHead.GetComponent<CirculationBehaviour>().WasInitiallyPumping = true;
                        fleebHead.GetComponent<CirculationBehaviour>().IsPump = true;
                        fleebHead.GetComponent<SpriteRenderer>().sortingLayerName = "Back"; 

                        foreach(var hitbox in fleebHead.GetComponents<Collider2D>()) { GameObject.Destroy(hitbox); }
                        var floop = fleebHead.AddComponent<BoxCollider2D>();
                        floop.size = new Vector2(0.2571429f, 0.2571429f);
                        floop.offset = new Vector2(0f, 0f);

                        var fleebth = fleebHead.AddComponent<GripBehaviour>();
                        fleebth.GripPosition = new Vector3(0.15f, -0.15f, 0.45f);  

                             var ca = new GameObject("ca");
                             ca.transform.SetParent(Instance.transform.Find("Body").Find("LowerBody"));
                             ca.transform.localPosition = new Vector3(-0.2275f, -0.057f);
                             ca.transform.localScale = new Vector3(1f, 1f);
                             var caSprite = ca.AddComponent<SpriteRenderer>();
                             caSprite.sprite = PigBack;
                             ca.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";
                    }
                }
            );
    //Bee
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Human"),
					NameOverride = "Bee",
					DescriptionOverride = "Wow! Those bees are so cute! oh, uh... did they sting you?",
					NameToOrderByOverride = "A",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/26.png"),
					AfterSpawn = (Instance) =>
					{
						Instance.transform.Find("Head").gameObject.AddComponent<BeeFlyBehaviour>();
                        //Instance.transform.Find("Head").gameObject.AddComponent<BoxCollider2D>();					
						PersonBehaviour person = Instance.GetComponent<PersonBehaviour>();

						var skin = ModAPI.LoadTexture("Mobs/bee.png");
						var flesh = ModAPI.LoadTexture("Mobs/bee.png");
						var bone = ModAPI.LoadTexture("Mobs/bee.png");
						person.SetBodyTextures(skin, flesh, bone, 1);
						foreach (LimbBehaviour body in person.Limbs) 
						{

							body.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Gorse");
                            var ItDead = body.gameObject.AddComponent<ItDeadBehaviour>();
                            ItDead.OnDeadSound = ModAPI.LoadSound("Sfx/bee2.wav");
                            ItDead.SoundVolume = 1f;
                            ItDead.OverrideImpactSoundsOnAlive = new AudioClip[] 
							{
								ModAPI.LoadSound("Sfx/bee1.wav"),
								ModAPI.LoadSound("Sfx/bee2.wav"),
								ModAPI.LoadSound("Sfx/bee3.wav"),
								ModAPI.LoadSound("Sfx/bee4.wav"),
							};
							body.gameObject.AddComponent<BeeFlyBehaviour>();
							body.BaseStrength *= 0.5f;
							body.Health *= 5f;
							body.BreakingThreshold *= 50f;
							body.transform.root.localScale *= 1.02f; 
							var arm1 = Instance.transform.Find("Head");
							var phys1 = arm1.GetComponent<PhysicalBehaviour>();
							var prop1 = ModAPI.FindPhysicalProperties("Human");
							prop1.Sharp = true;
							prop1.SharpAxes = new[]
							{
								new SharpAxis(Vector2.left, 0f, 0.45f, true, true),
							};
							phys1.Properties = prop1;                            
							
							body.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
							{
								ModAPI.LoadSound("Sfx/bee1.wav"),
								ModAPI.LoadSound("Sfx/bee2.wav"),
								ModAPI.LoadSound("Sfx/bee3.wav"),
								ModAPI.LoadSound("Sfx/bee4.wav"),
							};            

						//Destroy Stuff                       
							Instance.transform.Find("BackArm").Find("UpperArm").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
							Instance.transform.Find("BackArm").Find("LowerArm").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
							Instance.transform.Find("FrontArm").Find("UpperArmFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
							Instance.transform.Find("FrontArm").Find("LowerArmFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
							Instance.transform.Find("Body").Find("UpperBody").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
							Instance.transform.Find("Body").Find("MiddleBody").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
							Instance.transform.Find("Body").Find("LowerBody").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
							Instance.transform.Find("BackLeg").Find("UpperLeg").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
							Instance.transform.Find("BackLeg").Find("LowerLeg").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
							Instance.transform.Find("FrontLeg").Find("UpperLegFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
							Instance.transform.Find("FrontLeg").Find("LowerLegFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate(); 
							Instance.transform.Find("BackLeg").Find("Foot").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();
							Instance.transform.Find("FrontLeg").Find("FootFront").gameObject.GetComponent<PhysicalBehaviour>().Disintegrate();                       
						}

						var BeeBody = Instance.transform.Find("Head").gameObject;
						BeeBody.name = "Head";
						BeeBody.GetComponent<LimbBehaviour>().HasBrain = true;
						BeeBody.GetComponent<CirculationBehaviour>().WasInitiallyPumping = true;
						BeeBody.GetComponent<CirculationBehaviour>().IsPump = true;

						foreach(var hitbox in BeeBody.GetComponents<Collider2D>()) { GameObject.Destroy(hitbox); }
						var floop = BeeBody.AddComponent<BoxCollider2D>();
                        floop.size = new Vector2(0.2849767f, 0.1993054f);
						floop.offset = new Vector2(0.01465452f, -0.02893295f);					

						var fleebth = BeeBody.AddComponent<GripBehaviour>();
						fleebth.GripPosition = new Vector3(0.15f, -0.15f, 0.45f);  

                        AnimationBehaviour animBeh = Instance.transform.Find("Head").gameObject.AddComponent<AnimationBehaviour>();
						animBeh.AnimationDelay = 0.03f;
						animBeh.AnimationSprites = new Sprite[]
						{
							ModAPI.LoadSprite("Animations/BeeWings1.png"),
							ModAPI.LoadSprite("Animations/BeeWings2.png"),
							ModAPI.LoadSprite("Animations/BeeWings3.png"),
							ModAPI.LoadSprite("Animations/BeeWings4.png"),
							ModAPI.LoadSprite("Animations/BeeWings5.png"),
							ModAPI.LoadSprite("Animations/BeeWings6.png")
						};                    
					}
				}
			);
#endregion
#region Players
    //Kyrelink
            ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Human"),
					NameOverride = "Kyrelink",
					NameToOrderByOverride = "A",
					DescriptionOverride = "Creator. He created the first versions of this mod",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Kyrelink.png"),
					AfterSpawn = (Instance) =>
					{						
						var skin = ModAPI.LoadTexture("Mobs/Kyrelink.png");
						var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
						var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
						Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
					}
				}               
			);
    //Quickness
            ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Human"),
					NameOverride = "Quickness",
					NameToOrderByOverride = "A",
					DescriptionOverride = "Creator. Created all versions of the mod after 1.0.",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Quickness.png"),
					AfterSpawn = (Instance) =>
					{						
						var skin = ModAPI.LoadTexture("Mobs/Quickness.png");
						var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
						var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
						Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
					}
				}               
			);
    //Dream
            ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Human"),
					NameOverride = "Dream",
					NameToOrderByOverride = "A",
					DescriptionOverride = " ",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Dream.png"),
					AfterSpawn = (Instance) =>
					{						
						var skin = ModAPI.LoadTexture("Mobs/Dream.png");
						var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
						var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
						Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
					}
				}               
			);
            
    //GeorgeNotFound
            ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Human"),
					NameOverride = "GeorgeNotFound",
					NameToOrderByOverride = "A",
					DescriptionOverride = " ",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/GeorgeNotFound.png"),
					AfterSpawn = (Instance) =>
					{						
						var skin = ModAPI.LoadTexture("Mobs/GeorgeNotFound.png");
						var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
						var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
						Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
					}
				}               
			);
            
    //Sapnap
            ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Human"),
					NameOverride = "Sapnap",
					NameToOrderByOverride = "A",
					DescriptionOverride = " ",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Sapnap.png"),
					AfterSpawn = (Instance) =>
					{						
						var skin = ModAPI.LoadTexture("Mobs/Sapnap.png");
						var flesh = ModAPI.LoadTexture("Mobs/LayerFlesh.png");
						var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
						Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
					}
				}               
			);

    //BadBoyHalo
            ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Human"),
					NameOverride = "BadBoyHalo",
					NameToOrderByOverride = "A",
					DescriptionOverride = " ",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/BadBoyHalo.png"),
					AfterSpawn = (Instance) =>
					{						
						var skin = ModAPI.LoadTexture("Mobs/BadBoyHalo.png");
						var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
						var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
						Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
					}
				}               
			);

    //Antfrost
            ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Human"),
					NameOverride = "Antfrost",
					NameToOrderByOverride = "A",
					DescriptionOverride = " ",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Antfrost.png"),
					AfterSpawn = (Instance) =>
					{						
						var skin = ModAPI.LoadTexture("Mobs/Antfrost.png");
						var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
						var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
						Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
					}
				}               
			);
            
    //Awesamdude
            ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Human"),
					NameOverride = "Awesamdude",
					NameToOrderByOverride = "A",
					DescriptionOverride = " ",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Awesamdude.png"),
					AfterSpawn = (Instance) =>
					{						
						var skin = ModAPI.LoadTexture("Mobs/Awesamdude.png");
						var flesh = ModAPI.LoadTexture("Mobs/Layer Flesh.png");
						var bone = ModAPI.LoadTexture("Mobs/Layer Bone.png");
						Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone, 1);
					}
				}               
			);
#endregion
#region Ragdolls
    //Goat
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Small I-Beam"),
					NameOverride = "Goat",
					NameToOrderByOverride = "A",
					DescriptionOverride = "Goat! It's a very beautiful goat. He knows how to fight. This ragdoll has primitive layers of damage!",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Goat.png"),
					AfterSpawn = (Instance) =>
					{
						PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
						phys.Properties = ModAPI.FindPhysicalProperties("Human");
						phys.TrueInitialMass = 0.15f;
						phys.InitialMass = 0.15f;
						phys.rigidbody.mass = 0.15f;
                        phys.DisplayBloodDecals = false;
                        Instance.GetComponent<SpriteRenderer>().sortingOrder = 10;
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Bubbles"; 

						CustomCollision customCol0 = Instance.AddComponent<CustomCollision>();
						customCol0.CollisionSprite = ModAPI.LoadSprite("Mobs/Goat/Collision/Body.png", 5f);
						customCol0.Sprite = ModAPI.LoadSprite("Mobs/Goat/Textures/Body.png", 1f);

						Instance.AddComponent<NoChildrenCollide>();
						bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(1.9f * (flipped ? -1f : 1f), 1.9f);

						OnBreak onBreak0 = Instance.AddComponent<OnBreak>();
						onBreak0.startHealth = 100f;
						onBreak0.WoundedHealthThreshold = 80f;
                        onBreak0.MoreWoundedHealthThreshold = 60f;
						onBreak0.AlmostDeadHealthThreshold = 40f;
                        onBreak0.MoreAlmostDeadHealthThreshold = 20f;
						onBreak0.DeadHealthThreshold = 0f;				

						onBreak0.DeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/DeadSprite/Body.png", 1f);
                        onBreak0.MoreAlmostDeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/MoreAlmostDeadSprite/Body.png", 1f);
						onBreak0.AlmostDeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/AlmostDeadSprite/Body.png", 1f);
                        onBreak0.MoreWoundedSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/MoreWoundedSprite/Body.png", 1f); 
						onBreak0.WoundedSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/WoundedSprite/Body.png", 1f); 
						
						EntityCreator creator = Instance.AddComponent<EntityCreator>();
					//Head
						creator.OriginalItemName = "Small I-Beam";
						creator.PhysicalPropertiesName = "Human";
						creator.Sprite = ModAPI.LoadSprite("Mobs/Goat/Textures/Head.png", 1f);
						creator.CollisionSprite = ModAPI.LoadSprite("Mobs/Goat/Collision/Head.png", 1f);
                        //creator.SoundOnActivation = ModAPI.LoadSound("Sfx/Goatidle1.mp3");
						creator.sortingOrder = 11;
						creator.sortingLayerName = "Foreground";
						creator.LimbMass = 0.3f;
						creator.localPosition = new Vector3(0.2885502f, 0.08540767f);
                        creator.localAngle = new Vector3(0f, 0f, -45f);
                        creator.DisplayBloodDecals = false;
                        creator.UseLimits = true;
						creator.hingeJointBreakForce = 50000f;
						creator.hingeJointAnchor = new Vector3(-0.136f, 0.007f);
						creator.limitsMin = -5;
						creator.limitsMax = 5;
						creator.CreateNewEntity();
						GameObject head = creator.entity;
						creator.entity.AddComponent<OnHingeJointBreakSetParent>().joint = creator.entity.GetComponent<HingeJoint2D>();

						OnBreak onBreak1 = creator.entity.AddComponent<OnBreak>();
						onBreak1.startHealth = 80f;
						onBreak1.WoundedHealthThreshold = 80f;
                        onBreak1.MoreWoundedHealthThreshold = 60f;
						onBreak1.AlmostDeadHealthThreshold = 40f;
                        onBreak1.MoreAlmostDeadHealthThreshold = 20f;
						onBreak1.DeadHealthThreshold = 0f;		

						onBreak1.DeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/DeadSprite/Head.png", 1f);
                        onBreak1.MoreAlmostDeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/MoreAlmostDeadSprite/Head.png", 1f);
						onBreak1.AlmostDeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/AlmostDeadSprite/Head.png", 1f);
                        onBreak1.MoreWoundedSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/MoreWoundedSprite/Head.png", 1f); 
						onBreak1.WoundedSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/WoundedSprite/Head.png", 1f); 

                        var AS = head.AddComponent<AudioSource>();
                        AS.priority = 10;
                        AS.volume = 10f;
                        AS.minDistance = 5f; 
                        AudioClip[] clips = new AudioClip[]
                        {
                           ModAPI.LoadSound("Sfx/Goatidle1.mp3"),
                           ModAPI.LoadSound("Sfx/Goatidle2.mp3"),
                           ModAPI.LoadSound("Sfx/Goatidle3.mp3"),
                           ModAPI.LoadSound("Sfx/Goatidle4.mp3"),
                           ModAPI.LoadSound("Sfx/Goatidle5.mp3"),
                           ModAPI.LoadSound("Sfx/Goatidle6.mp3"),
                           ModAPI.LoadSound("Sfx/Goatidle7.mp3"),
                           ModAPI.LoadSound("Sfx/Goatidle8.mp3")

                        };
                        head.AddComponent<UseEventTrigger>().Action = () =>
                        {
                            AS.PlayOneShot(clips[UnityEngine.Random.Range(0, clips.Length)]);
                        };
                        
                    //Ear
						creator.OriginalItemName = "Small I-Beam";
						creator.PhysicalPropertiesName = "Human";
						creator.Sprite = ModAPI.LoadSprite("Mobs/Goat/Textures/Ear.png", 1f);
						creator.CollisionSprite = ModAPI.LoadSprite("Mobs/Goat/Collision/Ear.png", 1f);
                        creator.DisplayBloodDecals = false;
						creator.sortingOrder = 11;
						creator.sortingLayerName = "Foreground";
						creator.LimbMass = 0.3f;
						creator.localPosition = new Vector3(0.243f, 0.199f);
                        creator.localAngle = new Vector3(0f, 0f, 0f);
                        creator.UseLimits = true;
						creator.hingeJointBreakForce = 50000f;
						creator.hingeJointAnchor = new Vector3(0f, 0f);
						creator.limitsMin = -5;
						creator.limitsMax = 5;
						creator.CreateNewEntity();
                        creator.entity.AddComponent<OnHingeJointBreakSetParent>().joint = creator.entity.GetComponent<HingeJoint2D>();

						creator.entity.GetComponent<HingeJoint2D>().connectedBody = head.GetComponent<Rigidbody2D>();

						OnBreak onBreak11 = creator.entity.AddComponent<OnBreak>();
						onBreak11.startHealth = 5f;
						onBreak11.WoundedHealthThreshold = 4f;
                        onBreak11.MoreWoundedHealthThreshold = 3f;
						onBreak11.AlmostDeadHealthThreshold = 2f;
                        onBreak11.MoreAlmostDeadHealthThreshold = 1f;
						onBreak11.DeadHealthThreshold = 0f;		

						onBreak11.DeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/DeadSprite/Ear.png", 1f);
                        onBreak11.MoreAlmostDeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/MoreAlmostDeadSprite/Ear.png", 1f);
						onBreak11.AlmostDeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/AlmostDeadSprite/Ear.png", 1f);
                        onBreak11.MoreWoundedSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/MoreWoundedSprite/Ear.png", 1f); 
						onBreak11.WoundedSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/WoundedSprite/Ear.png", 1f);
					//Horn
						creator.OriginalItemName = "Small I-Beam";
						creator.PhysicalPropertiesName = "Brick";
						creator.Sprite = ModAPI.LoadSprite("Mobs/Goat/Textures/Horn.png", 1f);
						creator.CollisionSprite = ModAPI.LoadSprite("Mobs/Goat/Collision/Horn.png", 1f);
                        creator.DisplayBloodDecals = false;
						creator.sortingOrder = 8;
						creator.sortingLayerName = "Bottom";
						creator.LimbMass = 0.1f;
						creator.localPosition = new Vector3(0.217f, 0.284f);
                        creator.localAngle = new Vector3(0f, 0f, 0f);
                        creator.UseLimits = true;
						creator.hingeJointBreakForce = 5000f;
						creator.hingeJointAnchor = new Vector3(0f, -0.095f);
						creator.limitsMin = 0;
						creator.limitsMax = 0;
						creator.CreateNewEntity();
						creator.entity.AddComponent<OnHingeJointBreakSetParent>().joint = creator.entity.GetComponent<HingeJoint2D>();

						creator.entity.GetComponent<HingeJoint2D>().connectedBody = head.GetComponent<Rigidbody2D>();

						OnBreak onBreak2 = creator.entity.AddComponent<OnBreak>();
						onBreak2.startHealth = 80f;
						onBreak2.WoundedHealthThreshold = 80f;
                        onBreak2.MoreWoundedHealthThreshold = 60f;
						onBreak2.AlmostDeadHealthThreshold = 40f;
                        onBreak2.MoreAlmostDeadHealthThreshold = 20f;
						onBreak2.DeadHealthThreshold = 0f;		

						onBreak2.DeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/DeadSprite/Horn.png", 1f);
                        onBreak2.MoreAlmostDeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/MoreAlmostDeadSprite/Horn.png", 1f);
						onBreak2.AlmostDeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/AlmostDeadSprite/Horn.png", 1f);
                        onBreak2.MoreWoundedSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/MoreWoundedSprite/Horn.png", 1f); 
						onBreak2.WoundedSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/WoundedSprite/Horn.png", 1f);
                    //Horn 2
						creator.OriginalItemName = "Small I-Beam";
						creator.PhysicalPropertiesName = "Brick";
						creator.Sprite = ModAPI.LoadSprite("Mobs/Goat/Textures/Horn.png", 1f);
						creator.CollisionSprite = ModAPI.LoadSprite("Mobs/Goat/Collision/Horn.png", 1f);
                        creator.DisplayBloodDecals = false;
						creator.sortingOrder = 8;
						creator.sortingLayerName = "Bottom";
						creator.LimbMass = 0.1f;
						creator.localPosition = new Vector3(0.217f, 0.284f);
                        creator.localAngle = new Vector3(0f, 0f, 0f);
                        creator.UseLimits = true;
						creator.hingeJointBreakForce = 1500f;
						creator.hingeJointAnchor = new Vector3(0f, -0.095f);
						creator.limitsMin = 0;
						creator.limitsMax = 0;
						creator.CreateNewEntity();
						creator.entity.AddComponent<OnHingeJointBreakSetParent>().joint = creator.entity.GetComponent<HingeJoint2D>();

						creator.entity.GetComponent<HingeJoint2D>().connectedBody = head.GetComponent<Rigidbody2D>();

						OnBreak onBreak22 = creator.entity.AddComponent<OnBreak>();
						onBreak22.startHealth = 80f;
						onBreak22.WoundedHealthThreshold = 80f;
                        onBreak22.MoreWoundedHealthThreshold = 60f;
						onBreak22.AlmostDeadHealthThreshold = 40f;
                        onBreak22.MoreAlmostDeadHealthThreshold = 20f;
						onBreak22.DeadHealthThreshold = 0f;		

						onBreak22.DeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/DeadSprite/Horn.png", 1f);
                        onBreak22.MoreAlmostDeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/MoreAlmostDeadSprite/Horn.png", 1f);
						onBreak22.AlmostDeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/AlmostDeadSprite/Horn.png", 1f);
                        onBreak22.MoreWoundedSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/MoreWoundedSprite/Horn.png", 1f); 
						onBreak22.WoundedSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/WoundedSprite/Horn.png", 1f);

					//Beard
						creator.OriginalItemName = "Small I-Beam";
						creator.PhysicalPropertiesName = "Human";
						creator.Sprite = ModAPI.LoadSprite("Mobs/Goat/Textures/Beard.png", 1f);
						creator.CollisionSprite = ModAPI.LoadSprite("Mobs/Goat/Collision/Beard.png", 5f);
                        creator.DisplayBloodDecals = false;
						creator.sortingOrder = 8;
						creator.sortingLayerName = "Bottom";
						creator.LimbMass = 0.1f;
						creator.localPosition = new Vector3(0.338f, -0.07f);
                        creator.localAngle = new Vector3(0f, 0f, 0f);
                        creator.UseLimits = true;
						creator.hingeJointBreakForce = 50000000f;
						creator.hingeJointAnchor = new Vector3(-0.015f, 0.044f);
						creator.limitsMin = -2;
						creator.limitsMax = 2;
						creator.CreateNewEntity();
						creator.entity.AddComponent<OnHingeJointBreakSetParent>().joint = creator.entity.GetComponent<HingeJoint2D>();

						creator.entity.GetComponent<HingeJoint2D>().connectedBody = head.GetComponent<Rigidbody2D>();

						OnBreak onBreak3 = creator.entity.AddComponent<OnBreak>();
						onBreak3.startHealth = 5f;
						onBreak3.WoundedHealthThreshold = 4f;
                        onBreak3.MoreWoundedHealthThreshold = 3f;
						onBreak3.AlmostDeadHealthThreshold = 2f;
                        onBreak3.MoreAlmostDeadHealthThreshold = 1f;
						onBreak3.DeadHealthThreshold = 0f;		

						onBreak3.DeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/DeadSprite/Beard.png", 1f);
                        onBreak3.MoreAlmostDeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/MoreAlmostDeadSprite/Beard.png", 1f);
						onBreak3.AlmostDeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/AlmostDeadSprite/Beard.png", 1f);
                        onBreak3.MoreWoundedSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/MoreWoundedSprite/Beard.png", 1f); 
						onBreak3.WoundedSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/WoundedSprite/Beard.png", 1f);

					//Leg
						creator.OriginalItemName = "Small I-Beam";
						creator.PhysicalPropertiesName = "Human";
						creator.Sprite = ModAPI.LoadSprite("Mobs/Goat/Textures/Leg.png", 1f);
						creator.CollisionSprite = ModAPI.LoadSprite("Mobs/Goat/Collision/Leg.png", 1f);
                        creator.DisplayBloodDecals = false;
						creator.sortingOrder = 7;
						creator.sortingLayerName = "Foreground";
						creator.LimbMass = 0.7f;
						creator.localPosition = new Vector3(0.12f, -0.18f);
						creator.hingeJointBreakForce = 50000f;
						creator.hingeJointAnchor = new Vector3(0f, 0.1288f);
                        creator.UseLimits = true;
						creator.limitsMin = 75;
						creator.limitsMax = -75;
						creator.CreateNewEntity();
						creator.entity.AddComponent<OnHingeJointBreakSetParent>().joint = creator.entity.GetComponent<HingeJoint2D>();

						OnBreak onBreak4 = creator.entity.AddComponent<OnBreak>();
						onBreak4.startHealth = 80f;
						onBreak4.WoundedHealthThreshold = 80f;
                        onBreak4.MoreWoundedHealthThreshold = 60f;
						onBreak4.AlmostDeadHealthThreshold = 40f;
                        onBreak4.MoreAlmostDeadHealthThreshold = 20f;
						onBreak4.DeadHealthThreshold = 0f;		

						onBreak4.DeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/DeadSprite/Leg.png", 1f);
                        onBreak4.MoreAlmostDeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/MoreAlmostDeadSprite/Leg.png", 1f);
						onBreak4.AlmostDeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/AlmostDeadSprite/Leg.png", 1f);
                        onBreak4.MoreWoundedSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/MoreWoundedSprite/Leg.png", 1f); 
						onBreak4.WoundedSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/WoundedSprite/Leg.png", 1f);
					//Leg
						creator.OriginalItemName = "Small I-Beam";
						creator.PhysicalPropertiesName = "Human";
						creator.Sprite = ModAPI.LoadSprite("Mobs/Goat/Textures/Leg.png", 1f);
						creator.CollisionSprite = ModAPI.LoadSprite("Mobs/Goat/Collision/Leg.png", 1f);
                        creator.DisplayBloodDecals = false;
						creator.sortingOrder = 7;
						creator.sortingLayerName = "Foreground";
						creator.LimbMass = 0.7f;
						creator.localPosition = new Vector3(-0.12f, -0.18f);
						creator.hingeJointBreakForce = 50000f;
						creator.hingeJointAnchor = new Vector3(0f, 0.1288f);
                        creator.UseLimits = true;
						creator.limitsMin = 75;
						creator.limitsMax = -75;
						creator.CreateNewEntity();
						creator.entity.AddComponent<OnHingeJointBreakSetParent>().joint = creator.entity.GetComponent<HingeJoint2D>();

						OnBreak onBreak5 = creator.entity.AddComponent<OnBreak>();
						onBreak5.startHealth = 80f;
						onBreak5.WoundedHealthThreshold = 80f;
                        onBreak5.MoreWoundedHealthThreshold = 60f;
						onBreak5.AlmostDeadHealthThreshold = 40f;
                        onBreak5.MoreAlmostDeadHealthThreshold = 20f;
						onBreak5.DeadHealthThreshold = 0f;		

						onBreak5.DeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/DeadSprite/Leg.png", 1f);
                        onBreak5.MoreAlmostDeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/MoreAlmostDeadSprite/Leg.png", 1f);
						onBreak5.AlmostDeadSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/AlmostDeadSprite/Leg.png", 1f);
                        onBreak5.MoreWoundedSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/MoreWoundedSprite/Leg.png", 1f); 
						onBreak5.WoundedSprite = ModAPI.LoadSprite("Mobs/Goat/Textures/WoundedSprite/Leg.png", 1f);	
					}
				}
			);
    //Cow
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Bowling Pin"),
					NameOverride = "Cow",
					NameToOrderByOverride = "A",
					DescriptionOverride = "She loves you. Don't kill her.",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Cow.png"),
					AfterSpawn = (Instance) =>
					{
						PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Human");
                    	phys.OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
						Instance.GetComponent<SpriteRenderer>().sprite =  ModAPI.LoadSprite("Mobs/Cow/Cow/Body.png", 1f);
						foreach (var c in Instance.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Instance.AddComponent<PolygonCollider2D>();
						Instance.AddComponent<NoChildrenCollide>();
						bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(1.9f * (flipped ? -1f : 1f), 1.9f);
					//Head
						GameObject I = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						I.AddComponent<AudioSourceTimeScaleBehaviour>();
                        I.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                        I.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };

						SpriteRenderer IRenderer = I.GetComponent<SpriteRenderer>();
						IRenderer.sprite = ModAPI.LoadSprite("Mobs/Cow/Cow/Head.png", 1f);
						IRenderer.sortingOrder = 2;
						IRenderer.sortingLayerName = "Middle";
						foreach (var c in I.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						I.AddComponent<CircleCollider2D>();
						I.transform.parent = Instance.transform;
						I.transform.rotation = Instance.transform.rotation;
						I.transform.localPosition = new Vector3(0.34f, 0.1f);
						I.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D hingeJoint = I.GetOrAddComponent<HingeJoint2D>();
						hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						hingeJoint.anchor = new Vector3(0f, 0f);

						JointAngleLimits2D limits = hingeJoint.limits;
						limits.min = 30;
						limits.max = -30;
						hingeJoint.limits = limits;
						hingeJoint.useLimits = true;
										
					//Leg
						GameObject Plankk = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Plankk.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Plankk.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                        Plankk.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
						
						SpriteRenderer PlankkRenderer = Plankk.GetComponent<SpriteRenderer>();
						PlankkRenderer.sprite = ModAPI.LoadSprite("Mobs/Cow/Cow/Leg.png", 1f);
						PlankkRenderer.sortingOrder = 2;
						PlankkRenderer.sortingLayerName = "Foreground";
						foreach (var c in Plankk.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Plankk.AddComponent<BoxCollider2D>();
						Plankk.transform.parent = Instance.transform;
						Plankk.transform.rotation = Instance.transform.rotation;
						Plankk.transform.localPosition = new Vector3(-0.18f, -0.3f);
						Plankk.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D PlankkhingeJoint = Plankk.GetOrAddComponent<HingeJoint2D>();
						PlankkhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						PlankkhingeJoint.anchor = new Vector3(0f, 0.15f);
						Debug.Log("Broken");
							

						JointAngleLimits2D Plankklimits = hingeJoint.limits;
						Plankklimits.min = 90;
						Plankklimits.max = -30;
						PlankkhingeJoint.limits = limits;
						PlankkhingeJoint.useLimits = true;
					//Leg 2
						GameObject II = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						II.AddComponent<AudioSourceTimeScaleBehaviour>();
                        II.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						II.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
						
						SpriteRenderer IIRenderer = II.GetComponent<SpriteRenderer>();
						IIRenderer.sprite = ModAPI.LoadSprite("Mobs/Cow/Cow/Leg.png", 1f);
						IIRenderer.sortingOrder = 2;
						IIRenderer.sortingLayerName = "Foreground";
						foreach (var c in II.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						II.AddComponent<BoxCollider2D>();
						II.transform.parent = Instance.transform;
						II.transform.rotation = Instance.transform.rotation;
						II.transform.localPosition = new Vector3(0.18f, -0.3f);
						II.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D IIhingeJoint = II.GetOrAddComponent<HingeJoint2D>();
						IIhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						IIhingeJoint.anchor = new Vector3(0f, 0.15f);

						JointAngleLimits2D IIlimits = hingeJoint.limits;
						IIlimits.min = 90;
						IIlimits.max = -30;
						IIhingeJoint.limits = limits;
						IIhingeJoint.useLimits = true;
					}
				}
			);
    //Mooshroom
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Bowling Pin"),
					NameOverride = "Mooshroom",
					NameToOrderByOverride = "A",
					DescriptionOverride = "She loves you. Don't kill her.",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Mooshroom.png"),
					AfterSpawn = (Instance) =>
					{
						PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Human");
                    	phys.OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
						Instance.GetComponent<SpriteRenderer>().sprite =  ModAPI.LoadSprite("Mobs/Cow/Mooshroom/Body.png", 1f);
						foreach (var c in Instance.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Instance.AddComponent<PolygonCollider2D>();
						Instance.AddComponent<NoChildrenCollide>();
						bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(1.9f * (flipped ? -1f : 1f), 1.9f);
					//Head
						GameObject I = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						I.AddComponent<AudioSourceTimeScaleBehaviour>();
                        I.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                        I.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };

						SpriteRenderer IRenderer = I.GetComponent<SpriteRenderer>();
						IRenderer.sprite = ModAPI.LoadSprite("Mobs/Cow/Mooshroom/Head.png", 1f);
						IRenderer.sortingOrder = 2;
						IRenderer.sortingLayerName = "Middle";
						foreach (var c in I.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						I.AddComponent<CircleCollider2D>();
						I.transform.parent = Instance.transform;
						I.transform.rotation = Instance.transform.rotation;
						I.transform.localPosition = new Vector3(0.34f, 0.1f);
						I.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D hingeJoint = I.GetOrAddComponent<HingeJoint2D>();
						hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						hingeJoint.anchor = new Vector3(0f, 0f);

						JointAngleLimits2D limits = hingeJoint.limits;
						limits.min = 30;
						limits.max = -30;
						hingeJoint.limits = limits;
						hingeJoint.useLimits = true;
										
					//Leg
						GameObject Plankk = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Plankk.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Plankk.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                        Plankk.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
						
						SpriteRenderer PlankkRenderer = Plankk.GetComponent<SpriteRenderer>();
						PlankkRenderer.sprite = ModAPI.LoadSprite("Mobs/Cow/Mooshroom/Leg.png", 1f);
						PlankkRenderer.sortingOrder = 2;
						PlankkRenderer.sortingLayerName = "Foreground";
						foreach (var c in Plankk.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Plankk.AddComponent<BoxCollider2D>();
						Plankk.transform.parent = Instance.transform;
						Plankk.transform.rotation = Instance.transform.rotation;
						Plankk.transform.localPosition = new Vector3(-0.18f, -0.3f);
						Plankk.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D PlankkhingeJoint = Plankk.GetOrAddComponent<HingeJoint2D>();
						PlankkhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						PlankkhingeJoint.anchor = new Vector3(0f, 0.15f);
						Debug.Log("Broken");
							

						JointAngleLimits2D Plankklimits = hingeJoint.limits;
						Plankklimits.min = 90;
						Plankklimits.max = -30;
						PlankkhingeJoint.limits = limits;
						PlankkhingeJoint.useLimits = true;
					//Leg 2
						GameObject II = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						II.AddComponent<AudioSourceTimeScaleBehaviour>();
                        II.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						II.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
						
						SpriteRenderer IIRenderer = II.GetComponent<SpriteRenderer>();
						IIRenderer.sprite = ModAPI.LoadSprite("Mobs/Cow/Mooshroom/Leg.png", 1f);
						IIRenderer.sortingOrder = 2;
						IIRenderer.sortingLayerName = "Foreground";
						foreach (var c in II.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						II.AddComponent<BoxCollider2D>();
						II.transform.parent = Instance.transform;
						II.transform.rotation = Instance.transform.rotation;
						II.transform.localPosition = new Vector3(0.18f, -0.3f);
						II.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D IIhingeJoint = II.GetOrAddComponent<HingeJoint2D>();
						IIhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						IIhingeJoint.anchor = new Vector3(0f, 0.15f);

						JointAngleLimits2D IIlimits = hingeJoint.limits;
						IIlimits.min = 90;
						IIlimits.max = -30;
						IIhingeJoint.limits = limits;
						IIhingeJoint.useLimits = true;
                    //Flower
						GameObject Hand2 = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Hand2.AddComponent<AudioSourceTimeScaleBehaviour>();
						Hand2.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						
						SpriteRenderer Hand2Renderer = Hand2.GetComponent<SpriteRenderer>();
						Hand2Renderer.sprite = ModAPI.LoadSprite("Mobs/Cow/Mooshroom/red_mushroom.png", 1f);
						Hand2Renderer.sortingOrder = 2;
						Hand2Renderer.sortingLayerName = "Bottom";
						foreach (var c in Hand2.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Hand2.AddComponent<BoxCollider2D>();
						Hand2.transform.parent = Instance.transform;
						Hand2.transform.rotation = Instance.transform.rotation;
						Hand2.transform.localPosition = new Vector3(-0.15f, 0.2f);
						Hand2.transform.localScale = new Vector3(0.8f, 0.8f);

						HingeJoint2D Hand2hingeJoint = Hand2.GetOrAddComponent<HingeJoint2D>();
						Hand2hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						Hand2hingeJoint.anchor = new Vector3(0f, -0.04f);

						JointAngleLimits2D Hand2limits = hingeJoint.limits;
						Hand2limits.min = 5;
						Hand2limits.max = -5;
						Hand2hingeJoint.limits = Hand2limits;
						Hand2hingeJoint.useLimits = true;
			
					//Flower 2
						GameObject Leg2 = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Leg2.AddComponent<AudioSourceTimeScaleBehaviour>();
						Leg2.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						
						SpriteRenderer Leg2Renderer = Leg2.GetComponent<SpriteRenderer>();
						Leg2Renderer.sprite = ModAPI.LoadSprite("Mobs/Cow/Mooshroom/red_mushroom.png", 1f);
						Leg2Renderer.sortingOrder = 2;
						Leg2Renderer.sortingLayerName = "Bottom";
						foreach (var c in Leg2.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Leg2.AddComponent<BoxCollider2D>();
						Leg2.transform.parent = Instance.transform;
						Leg2.transform.rotation = Instance.transform.rotation;
						Leg2.transform.localPosition = new Vector3(0.2f, 0.2f);
						Leg2.transform.localScale = new Vector3(0.8f, 0.8f);

						HingeJoint2D Leg2hingeJoint = Leg2.GetOrAddComponent<HingeJoint2D>();
						Leg2hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						Leg2hingeJoint.anchor = new Vector3(0f, -0.04f);

						JointAngleLimits2D Leg2limits = hingeJoint.limits;
						Leg2limits.min = 5;
						Leg2limits.max = -5;
						Leg2hingeJoint.limits = Leg2limits;
						Leg2hingeJoint.useLimits = true;
					}
				}
			);
    
    //Brown Mooshroom
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Bowling Pin"),
					NameOverride = "Brown Mooshroom",
					NameToOrderByOverride = "A",
					DescriptionOverride = "She loves you. Don't kill her.",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Brown Mooshroom.png"),
					AfterSpawn = (Instance) =>
					{
						PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Human");
                    	phys.OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
						Instance.GetComponent<SpriteRenderer>().sprite =  ModAPI.LoadSprite("Mobs/Cow/Brown Mooshroom/Body.png", 1f);
						foreach (var c in Instance.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Instance.AddComponent<PolygonCollider2D>();
						Instance.AddComponent<NoChildrenCollide>();
						bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(1.9f * (flipped ? -1f : 1f), 1.9f);
					//Head
						GameObject I = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						I.AddComponent<AudioSourceTimeScaleBehaviour>();
                        I.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                        I.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };

						SpriteRenderer IRenderer = I.GetComponent<SpriteRenderer>();
						IRenderer.sprite = ModAPI.LoadSprite("Mobs/Cow/Brown Mooshroom/Head.png", 1f);
						IRenderer.sortingOrder = 2;
						IRenderer.sortingLayerName = "Middle";
						foreach (var c in I.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						I.AddComponent<CircleCollider2D>();
						I.transform.parent = Instance.transform;
						I.transform.rotation = Instance.transform.rotation;
						I.transform.localPosition = new Vector3(0.34f, 0.1f);
						I.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D hingeJoint = I.GetOrAddComponent<HingeJoint2D>();
						hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						hingeJoint.anchor = new Vector3(0f, 0f);

						JointAngleLimits2D limits = hingeJoint.limits;
						limits.min = 30;
						limits.max = -30;
						hingeJoint.limits = limits;
						hingeJoint.useLimits = true;
										
					//Leg
						GameObject Plankk = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Plankk.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Plankk.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                        Plankk.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
						
						SpriteRenderer PlankkRenderer = Plankk.GetComponent<SpriteRenderer>();
						PlankkRenderer.sprite = ModAPI.LoadSprite("Mobs/Cow/Brown Mooshroom/Leg.png", 1f);
						PlankkRenderer.sortingOrder = 2;
						PlankkRenderer.sortingLayerName = "Foreground";
						foreach (var c in Plankk.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Plankk.AddComponent<BoxCollider2D>();
						Plankk.transform.parent = Instance.transform;
						Plankk.transform.rotation = Instance.transform.rotation;
						Plankk.transform.localPosition = new Vector3(-0.18f, -0.3f);
						Plankk.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D PlankkhingeJoint = Plankk.GetOrAddComponent<HingeJoint2D>();
						PlankkhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						PlankkhingeJoint.anchor = new Vector3(0f, 0.15f);
						Debug.Log("Broken");
							

						JointAngleLimits2D Plankklimits = hingeJoint.limits;
						Plankklimits.min = 90;
						Plankklimits.max = -30;
						PlankkhingeJoint.limits = limits;
						PlankkhingeJoint.useLimits = true;
					//Leg 2
						GameObject II = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						II.AddComponent<AudioSourceTimeScaleBehaviour>();
                        II.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						II.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
						
						SpriteRenderer IIRenderer = II.GetComponent<SpriteRenderer>();
						IIRenderer.sprite = ModAPI.LoadSprite("Mobs/Cow/Brown Mooshroom/Leg.png", 1f);
						IIRenderer.sortingOrder = 2;
						IIRenderer.sortingLayerName = "Foreground";
						foreach (var c in II.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						II.AddComponent<BoxCollider2D>();
						II.transform.parent = Instance.transform;
						II.transform.rotation = Instance.transform.rotation;
						II.transform.localPosition = new Vector3(0.18f, -0.3f);
						II.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D IIhingeJoint = II.GetOrAddComponent<HingeJoint2D>();
						IIhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						IIhingeJoint.anchor = new Vector3(0f, 0.15f);

						JointAngleLimits2D IIlimits = hingeJoint.limits;
						IIlimits.min = 90;
						IIlimits.max = -30;
						IIhingeJoint.limits = limits;
						IIhingeJoint.useLimits = true;
                    //Flower
						GameObject Hand2 = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Hand2.AddComponent<AudioSourceTimeScaleBehaviour>();
						Hand2.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						
						SpriteRenderer Hand2Renderer = Hand2.GetComponent<SpriteRenderer>();
						Hand2Renderer.sprite = ModAPI.LoadSprite("Mobs/Cow/Brown Mooshroom/brown_mushroom.png", 1f);
						Hand2Renderer.sortingOrder = 2;
						Hand2Renderer.sortingLayerName = "Bottom";
						foreach (var c in Hand2.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Hand2.AddComponent<BoxCollider2D>();
						Hand2.transform.parent = Instance.transform;
						Hand2.transform.rotation = Instance.transform.rotation;
						Hand2.transform.localPosition = new Vector3(-0.15f, 0.2f);
						Hand2.transform.localScale = new Vector3(0.8f, 0.8f);

						HingeJoint2D Hand2hingeJoint = Hand2.GetOrAddComponent<HingeJoint2D>();
						Hand2hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						Hand2hingeJoint.anchor = new Vector3(0f, -0.04f);

						JointAngleLimits2D Hand2limits = hingeJoint.limits;
						Hand2limits.min = 5;
						Hand2limits.max = -5;
						Hand2hingeJoint.limits = Hand2limits;
						Hand2hingeJoint.useLimits = true;
			
					//Flower 2
						GameObject Leg2 = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Leg2.AddComponent<AudioSourceTimeScaleBehaviour>();
						Leg2.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						
						SpriteRenderer Leg2Renderer = Leg2.GetComponent<SpriteRenderer>();
						Leg2Renderer.sprite = ModAPI.LoadSprite("Mobs/Cow/Brown Mooshroom/brown_mushroom.png", 1f);
						Leg2Renderer.sortingOrder = 2;
						Leg2Renderer.sortingLayerName = "Bottom";
						foreach (var c in Leg2.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Leg2.AddComponent<BoxCollider2D>();
						Leg2.transform.parent = Instance.transform;
						Leg2.transform.rotation = Instance.transform.rotation;
						Leg2.transform.localPosition = new Vector3(0.2f, 0.2f);
						Leg2.transform.localScale = new Vector3(0.8f, 0.8f);

						HingeJoint2D Leg2hingeJoint = Leg2.GetOrAddComponent<HingeJoint2D>();
						Leg2hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						Leg2hingeJoint.anchor = new Vector3(0f, -0.04f);

						JointAngleLimits2D Leg2limits = hingeJoint.limits;
						Leg2limits.min = 5;
						Leg2limits.max = -5;
						Leg2hingeJoint.limits = Leg2limits;
						Leg2hingeJoint.useLimits = true;
					}
				}
			);
    //Moobloom
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Bowling Pin"),
					NameOverride = "Moobloom",
					NameToOrderByOverride = "A",
					DescriptionOverride = "Moobloom is a bright version of the usual cow. She loves you!",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Moobloom.png"),
					AfterSpawn = (Instance) =>
					{
						PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                    	phys.Properties = ModAPI.FindPhysicalProperties("Human");
                        phys.OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
						Instance.GetComponent<SpriteRenderer>().sprite =  ModAPI.LoadSprite("Mobs/Moobloom/Body.png", 1f);
						foreach (var c in Instance.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Instance.AddComponent<PolygonCollider2D>();
						Instance.AddComponent<NoChildrenCollide>();
						bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(1.9f * (flipped ? -1f : 1f), 1.9f);
					//Head
						GameObject I = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						I.AddComponent<AudioSourceTimeScaleBehaviour>();
						I.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");

						SpriteRenderer IRenderer = I.GetComponent<SpriteRenderer>();
						IRenderer.sprite = ModAPI.LoadSprite("Mobs/Moobloom/Head.png", 1f);
						IRenderer.sortingOrder = 2;
						IRenderer.sortingLayerName = "Middle";
                        I.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
						foreach (var c in I.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						I.AddComponent<CircleCollider2D>();
						I.transform.parent = Instance.transform;
						I.transform.rotation = Instance.transform.rotation;
						I.transform.localPosition = new Vector3(0.34f, 0.1f);
						I.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D hingeJoint = I.GetOrAddComponent<HingeJoint2D>();
						hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						hingeJoint.anchor = new Vector3(0f, 0f);

						JointAngleLimits2D limits = hingeJoint.limits;
						limits.min = 30;
						limits.max = -30;
						hingeJoint.limits = limits;
						hingeJoint.useLimits = true;
										
					//Leg
						GameObject Plankk = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Plankk.AddComponent<AudioSourceTimeScaleBehaviour>();
						Plankk.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						
						SpriteRenderer PlankkRenderer = Plankk.GetComponent<SpriteRenderer>();
						PlankkRenderer.sprite = ModAPI.LoadSprite("Mobs/Moobloom/Leg.png", 1f);
						PlankkRenderer.sortingOrder = 2;
						PlankkRenderer.sortingLayerName = "Foreground";
                        Plankk.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
						foreach (var c in Plankk.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Plankk.AddComponent<BoxCollider2D>();
						Plankk.transform.parent = Instance.transform;
						Plankk.transform.rotation = Instance.transform.rotation;
						Plankk.transform.localPosition = new Vector3(-0.18f, -0.3f);
						Plankk.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D PlankkhingeJoint = Plankk.GetOrAddComponent<HingeJoint2D>();
						PlankkhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						PlankkhingeJoint.anchor = new Vector3(0f, 0.15f);
						Debug.Log("Broken");
							

						JointAngleLimits2D Plankklimits = hingeJoint.limits;
						Plankklimits.min = 90;
						Plankklimits.max = -30;
						PlankkhingeJoint.limits = limits;
						PlankkhingeJoint.useLimits = true;
					//Leg 2
						GameObject II = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						II.AddComponent<AudioSourceTimeScaleBehaviour>();
						II.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						
						SpriteRenderer IIRenderer = II.GetComponent<SpriteRenderer>();
						IIRenderer.sprite = ModAPI.LoadSprite("Mobs/Moobloom/Leg.png", 1f);
						IIRenderer.sortingOrder = 2;
						IIRenderer.sortingLayerName = "Foreground";
                        II.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
						foreach (var c in II.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						II.AddComponent<BoxCollider2D>();
						II.transform.parent = Instance.transform;
						II.transform.rotation = Instance.transform.rotation;
						II.transform.localPosition = new Vector3(0.18f, -0.3f);
						II.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D IIhingeJoint = II.GetOrAddComponent<HingeJoint2D>();
						IIhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						IIhingeJoint.anchor = new Vector3(0f, 0.15f);

						JointAngleLimits2D IIlimits = hingeJoint.limits;
						IIlimits.min = 90;
						IIlimits.max = -30;
						IIhingeJoint.limits = limits;
						IIhingeJoint.useLimits = true;

					//Flower
						GameObject Hand2 = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Hand2.AddComponent<AudioSourceTimeScaleBehaviour>();
						Hand2.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						
						SpriteRenderer Hand2Renderer = Hand2.GetComponent<SpriteRenderer>();
						Hand2Renderer.sprite = ModAPI.LoadSprite("Mobs/Moobloom/Flower.png", 1f);
						Hand2Renderer.sortingOrder = 2;
						Hand2Renderer.sortingLayerName = "Bottom";
						foreach (var c in Hand2.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Hand2.AddComponent<BoxCollider2D>();
						Hand2.transform.parent = Instance.transform;
						Hand2.transform.rotation = Instance.transform.rotation;
						Hand2.transform.localPosition = new Vector3(-0.15f, 0.2f);
						Hand2.transform.localScale = new Vector3(0.8f, 0.8f);

						HingeJoint2D Hand2hingeJoint = Hand2.GetOrAddComponent<HingeJoint2D>();
						Hand2hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						Hand2hingeJoint.anchor = new Vector3(0f, -0.04f);

						JointAngleLimits2D Hand2limits = hingeJoint.limits;
						Hand2limits.min = 5;
						Hand2limits.max = -5;
						Hand2hingeJoint.limits = Hand2limits;
						Hand2hingeJoint.useLimits = true;

			
					//Flower 2
						GameObject Leg2 = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Leg2.AddComponent<AudioSourceTimeScaleBehaviour>();
						Leg2.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						
						SpriteRenderer Leg2Renderer = Leg2.GetComponent<SpriteRenderer>();
						Leg2Renderer.sprite = ModAPI.LoadSprite("Mobs/Moobloom/Flower 2.png", 1f);
						Leg2Renderer.sortingOrder = 2;
						Leg2Renderer.sortingLayerName = "Bottom";
						foreach (var c in Leg2.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Leg2.AddComponent<BoxCollider2D>();
						Leg2.transform.parent = Instance.transform;
						Leg2.transform.rotation = Instance.transform.rotation;
						Leg2.transform.localPosition = new Vector3(0.2f, 0.2f);
						Leg2.transform.localScale = new Vector3(0.8f, 0.8f);

						HingeJoint2D Leg2hingeJoint = Leg2.GetOrAddComponent<HingeJoint2D>();
						Leg2hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						Leg2hingeJoint.anchor = new Vector3(0f, -0.04f);

						JointAngleLimits2D Leg2limits = hingeJoint.limits;
						Leg2limits.min = 5;
						Leg2limits.max = -5;
						Leg2hingeJoint.limits = Leg2limits;
						Leg2hingeJoint.useLimits = true;
					}
				}
			);
    //Green Frog
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Bowling Pin"),
					NameOverride = "Green Frog",
					NameToOrderByOverride = "A",
					DescriptionOverride = "She loves you. Don't kill her.",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Green Frog.png"),
					AfterSpawn = (Instance) =>
					{
						PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Human");
                    	/*phys.OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
                        */
						Instance.GetComponent<SpriteRenderer>().sprite =  ModAPI.LoadSprite("Mobs/Frog/Frog1/Body.png", 1f);
						foreach (var c in Instance.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Instance.AddComponent<PolygonCollider2D>();
						Instance.AddComponent<NoChildrenCollide>();
						bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(1.9f * (flipped ? -1f : 1f), 1.9f);
					//Head
						GameObject I = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						I.AddComponent<AudioSourceTimeScaleBehaviour>();
                        I.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                       /* I.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
                        */
						SpriteRenderer IRenderer = I.GetComponent<SpriteRenderer>();
						IRenderer.sprite = ModAPI.LoadSprite("Mobs/Frog/Frog1/Leg1.png", 1f);
						IRenderer.sortingOrder = 2;
						IRenderer.sortingLayerName = "Middle";
						foreach (var c in I.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						I.AddComponent<BoxCollider2D>();
						I.transform.parent = Instance.transform;
						I.transform.rotation = Instance.transform.rotation;
						I.transform.localPosition = new Vector3(-0.08f, -0.08f);
						I.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D hingeJoint = I.GetOrAddComponent<HingeJoint2D>();
						hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						hingeJoint.anchor = new Vector3(0f, 0f);

						JointAngleLimits2D limits = hingeJoint.limits;
						limits.min = 10;
						limits.max = -10;
						hingeJoint.limits = limits;
						hingeJoint.useLimits = true;
										
					//Leg
						GameObject Plankk = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Plankk.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Plankk.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                        /*Plankk.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
						*/
						SpriteRenderer PlankkRenderer = Plankk.GetComponent<SpriteRenderer>();
						PlankkRenderer.sprite = ModAPI.LoadSprite("Mobs/Frog/Frog1/Leg2.png", 1f);
						PlankkRenderer.sortingOrder = 2;
						PlankkRenderer.sortingLayerName = "Foreground";
						foreach (var c in Plankk.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Plankk.AddComponent<BoxCollider2D>();
						Plankk.transform.parent = Instance.transform;
						Plankk.transform.rotation = Instance.transform.rotation;
						Plankk.transform.localPosition = new Vector3(0.09f, -0.08f);
						Plankk.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D PlankkhingeJoint = Plankk.GetOrAddComponent<HingeJoint2D>();
						PlankkhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						PlankkhingeJoint.anchor = new Vector3(0f, 0f);
						Debug.Log("Broken");
							

						JointAngleLimits2D Plankklimits = hingeJoint.limits;
						Plankklimits.min = 10;
						Plankklimits.max = -10;
						PlankkhingeJoint.limits = limits;
						PlankkhingeJoint.useLimits = true;
					}
				}
			);
    //Orange Frog
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Bowling Pin"),
					NameOverride = "Orange Frog",
					NameToOrderByOverride = "A",
					DescriptionOverride = "She loves you. Don't kill her.",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Orange Frog.png"),
					AfterSpawn = (Instance) =>
					{
						PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Human");
                    	/*phys.OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
                        */
						Instance.GetComponent<SpriteRenderer>().sprite =  ModAPI.LoadSprite("Mobs/Frog/Frog2/Body.png", 1f);
						foreach (var c in Instance.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Instance.AddComponent<PolygonCollider2D>();
						Instance.AddComponent<NoChildrenCollide>();
						bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(1.9f * (flipped ? -1f : 1f), 1.9f);
					//Head
						GameObject I = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						I.AddComponent<AudioSourceTimeScaleBehaviour>();
                        I.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                       /* I.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
                        */
						SpriteRenderer IRenderer = I.GetComponent<SpriteRenderer>();
						IRenderer.sprite = ModAPI.LoadSprite("Mobs/Frog/Frog2/Leg1.png", 1f);
						IRenderer.sortingOrder = 2;
						IRenderer.sortingLayerName = "Middle";
						foreach (var c in I.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						I.AddComponent<BoxCollider2D>();
						I.transform.parent = Instance.transform;
						I.transform.rotation = Instance.transform.rotation;
						I.transform.localPosition = new Vector3(-0.08f, -0.08f);
						I.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D hingeJoint = I.GetOrAddComponent<HingeJoint2D>();
						hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						hingeJoint.anchor = new Vector3(0f, 0f);

						JointAngleLimits2D limits = hingeJoint.limits;
						limits.min = 10;
						limits.max = -10;
						hingeJoint.limits = limits;
						hingeJoint.useLimits = true;
										
					//Leg
						GameObject Plankk = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Plankk.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Plankk.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                        /*Plankk.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
						*/
						SpriteRenderer PlankkRenderer = Plankk.GetComponent<SpriteRenderer>();
						PlankkRenderer.sprite = ModAPI.LoadSprite("Mobs/Frog/Frog2/Leg2.png", 1f);
						PlankkRenderer.sortingOrder = 2;
						PlankkRenderer.sortingLayerName = "Foreground";
						foreach (var c in Plankk.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Plankk.AddComponent<BoxCollider2D>();
						Plankk.transform.parent = Instance.transform;
						Plankk.transform.rotation = Instance.transform.rotation;
						Plankk.transform.localPosition = new Vector3(0.09f, -0.08f);
						Plankk.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D PlankkhingeJoint = Plankk.GetOrAddComponent<HingeJoint2D>();
						PlankkhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						PlankkhingeJoint.anchor = new Vector3(0f, 0f);
						Debug.Log("Broken");
							

						JointAngleLimits2D Plankklimits = hingeJoint.limits;
						Plankklimits.min = 10;
						Plankklimits.max = -10;
						PlankkhingeJoint.limits = limits;
						PlankkhingeJoint.useLimits = true;
					}
				}
			);
    //White Frog
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Bowling Pin"),
					NameOverride = "White Frog",
					NameToOrderByOverride = "A",
					DescriptionOverride = "She loves you. Don't kill her.",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/White Frog.png"),
					AfterSpawn = (Instance) =>
					{
						PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Human");
                    	/*phys.OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
                        */
						Instance.GetComponent<SpriteRenderer>().sprite =  ModAPI.LoadSprite("Mobs/Frog/Frog3/Body.png", 1f);
						foreach (var c in Instance.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Instance.AddComponent<PolygonCollider2D>();
						Instance.AddComponent<NoChildrenCollide>();
						bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(1.9f * (flipped ? -1f : 1f), 1.9f);
					//Head
						GameObject I = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						I.AddComponent<AudioSourceTimeScaleBehaviour>();
                        I.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                       /* I.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
                        */
						SpriteRenderer IRenderer = I.GetComponent<SpriteRenderer>();
						IRenderer.sprite = ModAPI.LoadSprite("Mobs/Frog/Frog3/Leg1.png", 1f);
						IRenderer.sortingOrder = 2;
						IRenderer.sortingLayerName = "Middle";
						foreach (var c in I.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						I.AddComponent<BoxCollider2D>();
						I.transform.parent = Instance.transform;
						I.transform.rotation = Instance.transform.rotation;
						I.transform.localPosition = new Vector3(-0.08f, -0.08f);
						I.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D hingeJoint = I.GetOrAddComponent<HingeJoint2D>();
						hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						hingeJoint.anchor = new Vector3(0f, 0f);

						JointAngleLimits2D limits = hingeJoint.limits;
						limits.min = 10;
						limits.max = -10;
						hingeJoint.limits = limits;
						hingeJoint.useLimits = true;
										
					//Leg
						GameObject Plankk = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Plankk.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Plankk.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                        /*Plankk.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
						*/
						SpriteRenderer PlankkRenderer = Plankk.GetComponent<SpriteRenderer>();
						PlankkRenderer.sprite = ModAPI.LoadSprite("Mobs/Frog/Frog3/Leg2.png", 1f);
						PlankkRenderer.sortingOrder = 2;
						PlankkRenderer.sortingLayerName = "Foreground";
						foreach (var c in Plankk.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Plankk.AddComponent<BoxCollider2D>();
						Plankk.transform.parent = Instance.transform;
						Plankk.transform.rotation = Instance.transform.rotation;
						Plankk.transform.localPosition = new Vector3(0.09f, -0.08f);
						Plankk.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D PlankkhingeJoint = Plankk.GetOrAddComponent<HingeJoint2D>();
						PlankkhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						PlankkhingeJoint.anchor = new Vector3(0f, 0f);
						Debug.Log("Broken");
							

						JointAngleLimits2D Plankklimits = hingeJoint.limits;
						Plankklimits.min = 10;
						Plankklimits.max = -10;
						PlankkhingeJoint.limits = limits;
						PlankkhingeJoint.useLimits = true;
					}
				}
			);
    //Tadpole
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Bowling Pin"),
					NameOverride = "Tadpole",
					NameToOrderByOverride = "A",
					DescriptionOverride = "She loves you. Don't kill her.",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Tadpole.png"),
					AfterSpawn = (Instance) =>
					{
						PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Human");
                    	/*phys.OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
                        */
						Instance.GetComponent<SpriteRenderer>().sprite =  ModAPI.LoadSprite("Mobs/Frog/Tadpole/Head.png", 1f);
						foreach (var c in Instance.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Instance.AddComponent<PolygonCollider2D>();
						Instance.AddComponent<NoChildrenCollide>();
						bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(1.9f * (flipped ? -1f : 1f), 1.9f);
					//Head
						GameObject I = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						I.AddComponent<AudioSourceTimeScaleBehaviour>();
                        I.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                       /* I.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Cowhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Cowhurt3.mp3"),                                                                                 
                            };
                        */
						SpriteRenderer IRenderer = I.GetComponent<SpriteRenderer>();
						IRenderer.sprite = ModAPI.LoadSprite("Mobs/Frog/Tadpole/Back.png", 1f);
						IRenderer.sortingOrder = 2;
						IRenderer.sortingLayerName = "Middle";
						foreach (var c in I.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						I.AddComponent<BoxCollider2D>();
						I.transform.parent = Instance.transform;
						I.transform.rotation = Instance.transform.rotation;
						I.transform.localPosition = new Vector3(-0.15f, 0f);
						I.transform.localScale = new Vector3(1f, 0.9f);

						HingeJoint2D hingeJoint = I.GetOrAddComponent<HingeJoint2D>();
						hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						hingeJoint.anchor = new Vector3(0.1f, 0f);

						JointAngleLimits2D limits = hingeJoint.limits;
						limits.min = 15;
						limits.max = -15;
						hingeJoint.limits = limits;
						hingeJoint.useLimits = true;
					}
				}
			);
    //Hoglin
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Bowling Pin"),
					NameOverride = "Hoglin",
					NameToOrderByOverride = "A",
					DescriptionOverride = "A huge hog that wants to eat you!",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Hoglin.png"),
					AfterSpawn = (Instance) =>
					{
						PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Human");
                    	phys.OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Hoghurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Hoghurt2.mp3"),                                                                                 
                            };
						Instance.GetComponent<SpriteRenderer>().sprite =  ModAPI.LoadSprite("Mobs/Nether/Hoglin/Body.png", 1f);
						foreach (var c in Instance.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Instance.AddComponent<PolygonCollider2D>();
						Instance.AddComponent<NoChildrenCollide>();
						bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(1.9f * (flipped ? -1f : 1f), 1.9f);
					//Head
						GameObject I = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						I.AddComponent<AudioSourceTimeScaleBehaviour>();
                        I.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                        I.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Hoghurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Hoghurt2.mp3"),                                                                                 
                            };

						SpriteRenderer IRenderer = I.GetComponent<SpriteRenderer>();
						IRenderer.sprite = ModAPI.LoadSprite("Mobs/Nether/Hoglin/Head.png", 1f);
						IRenderer.sortingOrder = 2;
						IRenderer.sortingLayerName = "Middle";
						foreach (var c in I.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						I.AddComponent<PolygonCollider2D>();
						I.transform.parent = Instance.transform;
						I.transform.rotation = Instance.transform.rotation;
						I.transform.localPosition = new Vector3(0.5f, -0.05f);
						I.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D hingeJoint = I.GetOrAddComponent<HingeJoint2D>();
						hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						hingeJoint.anchor = new Vector3(-0.1f, 0.3f);

						JointAngleLimits2D limits = hingeJoint.limits;
						limits.min = 0;
						limits.max = -50;
						hingeJoint.limits = limits;
						hingeJoint.useLimits = true;										
					//Leg
						GameObject Plankk = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Plankk.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Plankk.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                        Plankk.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Hoghurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Hoghurt2.mp3"),                                                                                  
                            };
						
						SpriteRenderer PlankkRenderer = Plankk.GetComponent<SpriteRenderer>();
						PlankkRenderer.sprite = ModAPI.LoadSprite("Mobs/Nether/Hoglin/Leg.png", 1f);
						PlankkRenderer.sortingOrder = 2;
						PlankkRenderer.sortingLayerName = "Foreground";
						foreach (var c in Plankk.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Plankk.AddComponent<BoxCollider2D>();
						Plankk.transform.parent = Instance.transform;
						Plankk.transform.rotation = Instance.transform.rotation;
						Plankk.transform.localPosition = new Vector3(-0.28f, -0.35f);
						Plankk.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D PlankkhingeJoint = Plankk.GetOrAddComponent<HingeJoint2D>();
						PlankkhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						PlankkhingeJoint.anchor = new Vector3(0f, 0.15f);
						Debug.Log("Broken");
							

						JointAngleLimits2D Plankklimits = hingeJoint.limits;
						Plankklimits.min = 90;
						Plankklimits.max = -30;
						PlankkhingeJoint.limits = limits;
						PlankkhingeJoint.useLimits = true;
					//Leg 2
						GameObject II = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						II.AddComponent<AudioSourceTimeScaleBehaviour>();
                        II.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						II.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Hoghurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Hoghurt2.mp3"),                                                                                  
                            };
						
						SpriteRenderer IIRenderer = II.GetComponent<SpriteRenderer>();
						IIRenderer.sprite = ModAPI.LoadSprite("Mobs/Nether/Hoglin/Leg.png", 1f);
						IIRenderer.sortingOrder = 2;
						IIRenderer.sortingLayerName = "Foreground";
						foreach (var c in II.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						II.AddComponent<BoxCollider2D>();
						II.transform.parent = Instance.transform;
						II.transform.rotation = Instance.transform.rotation;
						II.transform.localPosition = new Vector3(0.28f, -0.35f);
						II.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D IIhingeJoint = II.GetOrAddComponent<HingeJoint2D>();
						IIhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						IIhingeJoint.anchor = new Vector3(0f, 0.15f);

						JointAngleLimits2D IIlimits = hingeJoint.limits;
						IIlimits.min = 90;
						IIlimits.max = -30;
						IIhingeJoint.limits = limits;
						IIhingeJoint.useLimits = true;
                    //Flower
						GameObject Hand2 = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Hand2.AddComponent<AudioSourceTimeScaleBehaviour>();
						Hand2.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						
						SpriteRenderer Hand2Renderer = Hand2.GetComponent<SpriteRenderer>();
						Hand2Renderer.sprite = ModAPI.LoadSprite("Mobs/Nether/Hoglin/Spine Hair.png", 1f);
						Hand2Renderer.sortingOrder = 2;
						Hand2Renderer.sortingLayerName = "Bottom";
						foreach (var c in Hand2.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Hand2.AddComponent<BoxCollider2D>();
						Hand2.transform.parent = Instance.transform;
						Hand2.transform.rotation = Instance.transform.rotation;
						Hand2.transform.localPosition = new Vector3(0.2f, 0.2f);
						Hand2.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D Hand2hingeJoint = Hand2.GetOrAddComponent<HingeJoint2D>();
						Hand2hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						Hand2hingeJoint.anchor = new Vector3(0f, -0.04f);

						JointAngleLimits2D Hand2limits = hingeJoint.limits;
						Hand2limits.min = 1;
						Hand2limits.max = -1;
						Hand2hingeJoint.limits = Hand2limits;
						Hand2hingeJoint.useLimits = true;
					}
				}
			);
    //Zoglin
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Bowling Pin"),
					NameOverride = "Zoglin",
					NameToOrderByOverride = "A",
					DescriptionOverride = "A huge zombified hog that wants to eat you!",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Zoglin.png"),
					AfterSpawn = (Instance) =>
					{
						PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Human");
                    	phys.OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Zoghurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Zoghurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Zoghurt3.mp3"),                                                                                 
                            };
						Instance.GetComponent<SpriteRenderer>().sprite =  ModAPI.LoadSprite("Mobs/Nether/Zoglin/Body.png", 1f);
						foreach (var c in Instance.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Instance.AddComponent<PolygonCollider2D>();
						Instance.AddComponent<NoChildrenCollide>();
						bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(1.9f * (flipped ? -1f : 1f), 1.9f);
					//Head
						GameObject I = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						I.AddComponent<AudioSourceTimeScaleBehaviour>();
                        I.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                        I.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Zoghurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Zoghurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Zoghurt3.mp3"),                                                                                 
                            };

						SpriteRenderer IRenderer = I.GetComponent<SpriteRenderer>();
						IRenderer.sprite = ModAPI.LoadSprite("Mobs/Nether/Zoglin/Head.png", 1f);
						IRenderer.sortingOrder = 2;
						IRenderer.sortingLayerName = "Middle";
						foreach (var c in I.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						I.AddComponent<PolygonCollider2D>();
						I.transform.parent = Instance.transform;
						I.transform.rotation = Instance.transform.rotation;
						I.transform.localPosition = new Vector3(0.5f, -0.05f);
						I.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D hingeJoint = I.GetOrAddComponent<HingeJoint2D>();
						hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						hingeJoint.anchor = new Vector3(-0.1f, 0.3f);

						JointAngleLimits2D limits = hingeJoint.limits;
						limits.min = 0;
						limits.max = -50;
						hingeJoint.limits = limits;
						hingeJoint.useLimits = true;										
					//Leg
						GameObject Plankk = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Plankk.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Plankk.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                        Plankk.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Zoghurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Zoghurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Zoghurt3.mp3"),                                                                                 
                            };
						
						SpriteRenderer PlankkRenderer = Plankk.GetComponent<SpriteRenderer>();
						PlankkRenderer.sprite = ModAPI.LoadSprite("Mobs/Nether/Zoglin/Leg.png", 1f);
						PlankkRenderer.sortingOrder = 2;
						PlankkRenderer.sortingLayerName = "Foreground";
						foreach (var c in Plankk.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Plankk.AddComponent<BoxCollider2D>();
						Plankk.transform.parent = Instance.transform;
						Plankk.transform.rotation = Instance.transform.rotation;
						Plankk.transform.localPosition = new Vector3(-0.28f, -0.35f);
						Plankk.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D PlankkhingeJoint = Plankk.GetOrAddComponent<HingeJoint2D>();
						PlankkhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						PlankkhingeJoint.anchor = new Vector3(0f, 0.15f);
						Debug.Log("Broken");
							

						JointAngleLimits2D Plankklimits = hingeJoint.limits;
						Plankklimits.min = 90;
						Plankklimits.max = -30;
						PlankkhingeJoint.limits = limits;
						PlankkhingeJoint.useLimits = true;
					//Leg 2
						GameObject II = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						II.AddComponent<AudioSourceTimeScaleBehaviour>();
                        II.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						II.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Zoghurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Zoghurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Zoghurt3.mp3"),                                                                                 
                            };
						
						SpriteRenderer IIRenderer = II.GetComponent<SpriteRenderer>();
						IIRenderer.sprite = ModAPI.LoadSprite("Mobs/Nether/Zoglin/Leg.png", 1f);
						IIRenderer.sortingOrder = 2;
						IIRenderer.sortingLayerName = "Foreground";
						foreach (var c in II.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						II.AddComponent<BoxCollider2D>();
						II.transform.parent = Instance.transform;
						II.transform.rotation = Instance.transform.rotation;
						II.transform.localPosition = new Vector3(0.28f, -0.35f);
						II.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D IIhingeJoint = II.GetOrAddComponent<HingeJoint2D>();
						IIhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						IIhingeJoint.anchor = new Vector3(0f, 0.15f);

						JointAngleLimits2D IIlimits = hingeJoint.limits;
						IIlimits.min = 90;
						IIlimits.max = -30;
						IIhingeJoint.limits = limits;
						IIhingeJoint.useLimits = true;
					}
				}
			);
    //Blaze
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Bowling Pin"),
					NameOverride = "Blaze",
					NameToOrderByOverride = "A",
					DescriptionOverride = "Fire man!",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Blaze.png"),
					AfterSpawn = (Instance) =>
					{
						PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Human");
                    	phys.OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Blazehit1.mp3"),
                                ModAPI.LoadSound("Sfx/Blazehit2.mp3"),
                                ModAPI.LoadSound("Sfx/Blazehit3.mp3"),    
                                ModAPI.LoadSound("Sfx/Blazehit4.mp3"),                                                                              
                            };
						Instance.GetComponent<SpriteRenderer>().sprite =  ModAPI.LoadSprite("Mobs/Nether/Blaze/Body.png", 1f);
						foreach (var c in Instance.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Instance.AddComponent<PolygonCollider2D>();
						Instance.AddComponent<NoChildrenCollide>();
						bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(1.9f * (flipped ? -1f : 1f), 1.9f);
					//Head
						GameObject I = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						I.AddComponent<AudioSourceTimeScaleBehaviour>();
                        I.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                        I.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Blazehit1.mp3"),
                                ModAPI.LoadSound("Sfx/Blazehit2.mp3"),
                                ModAPI.LoadSound("Sfx/Blazehit3.mp3"),    
                                ModAPI.LoadSound("Sfx/Blazehit4.mp3"),                                                                                 
                            };

						SpriteRenderer IRenderer = I.GetComponent<SpriteRenderer>();
						IRenderer.sprite = ModAPI.LoadSprite("Mobs/Nether/Blaze/Leg.png", 1f);
						IRenderer.sortingOrder = 2;
						IRenderer.sortingLayerName = "Middle";
						foreach (var c in I.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						I.AddComponent<CircleCollider2D>();
						I.transform.parent = Instance.transform;
						I.transform.rotation = Instance.transform.rotation;
						I.transform.localPosition = new Vector3(0.18f, -0.25f);
						I.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D hingeJoint = I.GetOrAddComponent<HingeJoint2D>();
						hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						hingeJoint.anchor = new Vector3(0f, 0f);

						JointAngleLimits2D limits = hingeJoint.limits;
						limits.min = 6;
						limits.max = -6;
						hingeJoint.limits = limits;
						hingeJoint.useLimits = true;										
					//Leg
						GameObject Plankk = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Plankk.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Plankk.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                        Plankk.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Blazehit1.mp3"),
                                ModAPI.LoadSound("Sfx/Blazehit2.mp3"),
                                ModAPI.LoadSound("Sfx/Blazehit3.mp3"),    
                                ModAPI.LoadSound("Sfx/Blazehit4.mp3"),                                                                                 
                            };
						
						SpriteRenderer PlankkRenderer = Plankk.GetComponent<SpriteRenderer>();
						PlankkRenderer.sprite = ModAPI.LoadSprite("Mobs/Nether/Blaze/Leg.png", 1f);
						PlankkRenderer.sortingOrder = 2;
						PlankkRenderer.sortingLayerName = "Foreground";
						foreach (var c in Plankk.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Plankk.AddComponent<BoxCollider2D>();
						Plankk.transform.parent = Instance.transform;
						Plankk.transform.rotation = Instance.transform.rotation;
						Plankk.transform.localPosition = new Vector3(-0.18f, -0.25f);
						Plankk.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D PlankkhingeJoint = Plankk.GetOrAddComponent<HingeJoint2D>();
						PlankkhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						PlankkhingeJoint.anchor = new Vector3(0f, 0f);
						Debug.Log("Broken");
							

						JointAngleLimits2D Plankklimits = hingeJoint.limits;
						Plankklimits.min = 6;
						Plankklimits.max = -6;
						PlankkhingeJoint.limits = limits;
						PlankkhingeJoint.useLimits = true;
					//Leg 2
						GameObject II = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						II.AddComponent<AudioSourceTimeScaleBehaviour>();
                        II.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						II.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Blazehit1.mp3"),
                                ModAPI.LoadSound("Sfx/Blazehit2.mp3"),
                                ModAPI.LoadSound("Sfx/Blazehit3.mp3"),    
                                ModAPI.LoadSound("Sfx/Blazehit4.mp3"),                                                                                 
                            };
						
						SpriteRenderer IIRenderer = II.GetComponent<SpriteRenderer>();
						IIRenderer.sprite = ModAPI.LoadSprite("Mobs/Nether/Blaze/Leg.png", 1f);
						IIRenderer.sortingOrder = 2;
						IIRenderer.sortingLayerName = "Foreground";
						foreach (var c in II.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						II.AddComponent<BoxCollider2D>();
						II.transform.parent = Instance.transform;
						II.transform.rotation = Instance.transform.rotation;
						II.transform.localPosition = new Vector3(0f, -0.3f);
						II.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D IIhingeJoint = II.GetOrAddComponent<HingeJoint2D>();
						IIhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						IIhingeJoint.anchor = new Vector3(0f, 0f);

						JointAngleLimits2D IIlimits = hingeJoint.limits;
						IIlimits.min = 6;
						IIlimits.max = -6;
						IIhingeJoint.limits = limits;
						IIhingeJoint.useLimits = true;
                //Flower
						GameObject Hand2 = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Hand2.AddComponent<AudioSourceTimeScaleBehaviour>();
						Hand2.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						
						SpriteRenderer Hand2Renderer = Hand2.GetComponent<SpriteRenderer>();
						Hand2Renderer.sprite = ModAPI.LoadSprite("Mobs/Nether/Blaze/Leg.png", 1f);
						Hand2Renderer.sortingOrder = 2;
						Hand2Renderer.sortingLayerName = "Bottom";
                        Hand2.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Blazehit1.mp3"),
                                ModAPI.LoadSound("Sfx/Blazehit2.mp3"),
                                ModAPI.LoadSound("Sfx/Blazehit3.mp3"),    
                                ModAPI.LoadSound("Sfx/Blazehit4.mp3"),                                                                                 
                            };
						foreach (var c in Hand2.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Hand2.AddComponent<BoxCollider2D>();
						Hand2.transform.parent = Instance.transform;
						Hand2.transform.rotation = Instance.transform.rotation;
						Hand2.transform.localPosition = new Vector3(0.1f, -0.45f);
						Hand2.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D Hand2hingeJoint = Hand2.GetOrAddComponent<HingeJoint2D>();
						Hand2hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						Hand2hingeJoint.anchor = new Vector3(0f, 0f);

						JointAngleLimits2D Hand2limits = hingeJoint.limits;
						Hand2limits.min = 6;
						Hand2limits.max = -6;
						Hand2hingeJoint.limits = Hand2limits;
						Hand2hingeJoint.useLimits = true;
			
					//Flower 2
						GameObject Leg2 = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Leg2.AddComponent<AudioSourceTimeScaleBehaviour>();
						Leg2.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						
						SpriteRenderer Leg2Renderer = Leg2.GetComponent<SpriteRenderer>();
						Leg2Renderer.sprite = ModAPI.LoadSprite("Mobs/Nether/Blaze/Leg.png", 1f);
						Leg2Renderer.sortingOrder = 2;
						Leg2Renderer.sortingLayerName = "Bottom";
                        Leg2.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Blazehit1.mp3"),
                                ModAPI.LoadSound("Sfx/Blazehit2.mp3"),
                                ModAPI.LoadSound("Sfx/Blazehit3.mp3"),    
                                ModAPI.LoadSound("Sfx/Blazehit4.mp3"),                                                                                 
                            };
						foreach (var c in Leg2.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Leg2.AddComponent<BoxCollider2D>();
						Leg2.transform.parent = Instance.transform;
						Leg2.transform.rotation = Instance.transform.rotation;
						Leg2.transform.localPosition = new Vector3(-0.1f, -0.45f);
						Leg2.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D Leg2hingeJoint = Leg2.GetOrAddComponent<HingeJoint2D>();
						Leg2hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						Leg2hingeJoint.anchor = new Vector3(0f, 0f);

						JointAngleLimits2D Leg2limits = hingeJoint.limits;
						Leg2limits.min = 6;
						Leg2limits.max = -6;
						Leg2hingeJoint.limits = Leg2limits;
						Leg2hingeJoint.useLimits = true;
					}
				}
			);
    //Strider
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Bowling Pin"),
					NameOverride = "Strider",
					NameToOrderByOverride = "A",
					DescriptionOverride = "Red Cube.",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Strider.png"),
					AfterSpawn = (Instance) =>
					{
						PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Human");
                    	phys.OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Striderhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Striderhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Striderhurt3.mp3"),   
                                ModAPI.LoadSound("Sfx/Striderhurt4.mp3"),                                                                               
                            };
						Instance.GetComponent<SpriteRenderer>().sprite =  ModAPI.LoadSprite("Mobs/Nether/Strider/Body.png", 1f);
						foreach (var c in Instance.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Instance.AddComponent<PolygonCollider2D>();
						Instance.AddComponent<NoChildrenCollide>();
						bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(1.9f * (flipped ? -1f : 1f), 1.9f);
					//Head
						GameObject I = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						I.AddComponent<AudioSourceTimeScaleBehaviour>();
                        I.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                        I.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Striderhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Striderhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Striderhurt3.mp3"),   
                                ModAPI.LoadSound("Sfx/Striderhurt4.mp3"),                                                                               
                            };

						SpriteRenderer IRenderer = I.GetComponent<SpriteRenderer>();
						IRenderer.sprite = ModAPI.LoadSprite("Mobs/Nether/Strider/Leg.png", 1f);
						IRenderer.sortingOrder = 2;
						IRenderer.sortingLayerName = "Middle";
						foreach (var c in I.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						I.AddComponent<CircleCollider2D>();
						I.transform.parent = Instance.transform;
						I.transform.rotation = Instance.transform.rotation;
						I.transform.localPosition = new Vector3(0f, -0.43f);
						I.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D hingeJoint = I.GetOrAddComponent<HingeJoint2D>();
						hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						hingeJoint.anchor = new Vector3(0f, 0.3f);

						JointAngleLimits2D limits = hingeJoint.limits;
						limits.min = 90;
						limits.max = -30;
						hingeJoint.limits = limits;
						hingeJoint.useLimits = true;										
					//Leg
						GameObject Plankk = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Plankk.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Plankk.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                        Plankk.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Striderhurt1.mp3"),
                                ModAPI.LoadSound("Sfx/Striderhurt2.mp3"),
                                ModAPI.LoadSound("Sfx/Striderhurt3.mp3"),   
                                ModAPI.LoadSound("Sfx/Striderhurt4.mp3"),                                                                                  
                            };
						
						SpriteRenderer PlankkRenderer = Plankk.GetComponent<SpriteRenderer>();
						PlankkRenderer.sprite = ModAPI.LoadSprite("Mobs/Nether/Strider/Leg.png", 1f);
						PlankkRenderer.sortingOrder = 2;
						PlankkRenderer.sortingLayerName = "Foreground";
						foreach (var c in Plankk.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Plankk.AddComponent<BoxCollider2D>();
						Plankk.transform.parent = Instance.transform;
						Plankk.transform.rotation = Instance.transform.rotation;
						Plankk.transform.localPosition = new Vector3(0f, -0.43f);
						Plankk.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D PlankkhingeJoint = Plankk.GetOrAddComponent<HingeJoint2D>();
						PlankkhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						PlankkhingeJoint.anchor = new Vector3(0f, 0.3f);
						Debug.Log("Broken");
							

						JointAngleLimits2D Plankklimits = hingeJoint.limits;
						Plankklimits.min = 90;
						Plankklimits.max = -30;
						PlankkhingeJoint.limits = limits;
						PlankkhingeJoint.useLimits = true;
					}
				}
			);
    
    
    //Snow Golem With Pumkin
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Crossbow"),
                    NameOverride = "Snow Golem With Pumkin",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = " ",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Snow Golem With Pumkin.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
						Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Mobs/Snow Golem With Pumkin.png", 0.6f);
						foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<PolygonCollider2D>();
                        Instance.FixColliders();
						Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Snow");  
						Instance.GetComponent<SpriteRenderer>().sortingOrder = 10;

						var Proj = Instance.GetComponent<ProjectileLauncherBehaviour>();
						Proj.projectileAsset = ModAPI.FindSpawnable("Snowball");
						Proj.launchSound = ModAPI.LoadSound("Sfx/bow1.wav");
						Proj.IsAutomatic = true;
						Proj.ShouldPutProjectileInOtherLayer = false;
						Proj.RemoveLaunchedObjectsWithMe = true;
						Proj.projectileLaunchStrength = 15f;
						Proj.AutomaticInterval = 0.6f;
						Proj.recoilMultiplier = 0.05f;
						Proj.ScreenShake = 0.1f;						
                    }
                }
            );
    //Snow Golem
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Crossbow"),
                    NameOverride = "Snow Golem",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = " ",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Snow Golem.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
						Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Mobs/Snow Golem.png", 0.6f);
						foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<PolygonCollider2D>();
                        Instance.FixColliders();
						Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Snow");  
						Instance.GetComponent<SpriteRenderer>().sortingOrder = 10;
						var Proj = Instance.GetComponent<ProjectileLauncherBehaviour>();
						Proj.projectileAsset = ModAPI.FindSpawnable("Snowball");
						Proj.launchSound = ModAPI.LoadSound("Sfx/bow1.wav");
						Proj.IsAutomatic = true;
						Proj.ShouldPutProjectileInOtherLayer = false;
						Proj.RemoveLaunchedObjectsWithMe = true;
						Proj.projectileLaunchStrength = 15f;
						Proj.AutomaticInterval = 0.6f;
						Proj.recoilMultiplier = 0.05f;
						Proj.ScreenShake = 0.1f;						
                    }
                }
            );
    //Glare
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Bowling Pin"),
					NameOverride = "Glare",
					NameToOrderByOverride = "A",
					DescriptionOverride = "A glowing firefly, he is very cute and wants to make friends with you",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Glare.png"),
					AfterSpawn = (Instance) =>
					{
						PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                    	phys.Properties = ModAPI.FindPhysicalProperties("Human");

						Instance.GetComponent<SpriteRenderer>().sprite =  ModAPI.LoadSprite("Mobs/Glare/Glare.png", 1f);
						foreach (var c in Instance.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Instance.AddComponent<PolygonCollider2D>();
						Instance.AddComponent<NoChildrenCollide>();
					//Head
						GameObject I = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						
						SpriteRenderer IRenderer = I.GetComponent<SpriteRenderer>();
						IRenderer.sprite = ModAPI.LoadSprite("Mobs/Glare/Glare Vine.png", 1f);
						IRenderer.sortingOrder = 2;
						IRenderer.sortingLayerName = "Foreground";
						foreach (var c in I.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						I.AddComponent<BoxCollider2D>();
						I.transform.parent = Instance.transform;
						I.transform.rotation = Instance.transform.rotation;
						I.transform.localPosition = new Vector3(0f, -0.3f);
						I.transform.localScale = new Vector3(1f, 1f);
                        I.AddComponent<AudioSourceTimeScaleBehaviour>();
                        I.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");

						HingeJoint2D hingeJoint = I.GetOrAddComponent<HingeJoint2D>();
						hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						hingeJoint.anchor = new Vector3(0f, 0.3f);

						JointAngleLimits2D limits = hingeJoint.limits;
						limits.min = 3;
						limits.max = -3;
						hingeJoint.limits = limits;
						hingeJoint.useLimits = true;
					}
				}
			);
	//Allay
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Bowling Pin"),
					NameOverride = "Allay",
					NameToOrderByOverride = "A",
					DescriptionOverride = "Flying ghost of kindness.",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Allay.png"),
					AfterSpawn = (Instance) =>
					{
						PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                    	phys.Properties = ModAPI.FindPhysicalProperties("Human");
						Instance.GetComponent<SpriteRenderer>().sprite =  ModAPI.LoadSprite("Mobs/Allay/Allay Body.png", 1f);
						foreach (var c in Instance.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Instance.AddComponent<PolygonCollider2D>();
						Instance.AddComponent<NoChildrenCollide>();
					//Head
						GameObject I = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						
						SpriteRenderer IRenderer = I.GetComponent<SpriteRenderer>();
						IRenderer.sprite = ModAPI.LoadSprite("Mobs/Allay/Allay Head.png", 1f);
						IRenderer.sortingOrder = 2;
						IRenderer.sortingLayerName = "Foreground";
                        I.AddComponent<AudioSourceTimeScaleBehaviour>();
                        I.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						foreach (var c in I.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						I.AddComponent<BoxCollider2D>();
						I.transform.parent = Instance.transform;
						I.transform.rotation = Instance.transform.rotation;
						I.transform.localPosition = new Vector3(0f, 0.35f);
						I.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D hingeJoint = I.GetOrAddComponent<HingeJoint2D>();
						hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						hingeJoint.anchor = new Vector3(0f, -0.09f);

						JointAngleLimits2D limits = hingeJoint.limits;
						limits.min = 30;
						limits.max = -30;
						hingeJoint.limits = limits;
						hingeJoint.useLimits = true;
					//Hand
						GameObject II = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
                        SpriteRenderer IIRenderer = II.GetComponent<SpriteRenderer>();
                        IIRenderer.sprite = ModAPI.LoadSprite("Mobs/Allay/Allay Hand.png", 1f);
                        IIRenderer.sortingOrder = 2;
                        IIRenderer.sortingLayerName = "Foreground";
                        II.AddComponent<AudioSourceTimeScaleBehaviour>();
                        II.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                        foreach (var c in II.GetComponents<Collider2D>())
                        {
                            GameObject.Destroy(c);
                        }
                        II.AddComponent<BoxCollider2D>();
                        II.transform.parent = Instance.transform;
                        II.transform.rotation = Instance.transform.rotation;
                        II.transform.localPosition = new Vector3(0f, 0.0f);
                        II.transform.localScale = new Vector3(1f, 1f);
                        HingeJoint2D IIhingeJoint = II.GetOrAddComponent<HingeJoint2D>();
                        IIhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
                        IIhingeJoint.anchor = new Vector3(0f, 0.15f);
                        JointAngleLimits2D IIlimits = IIhingeJoint.limits;
                        IIlimits.min = 50;
                        IIlimits.max = -50;
                        IIhingeJoint.limits = IIlimits;
                        IIhingeJoint.useLimits = false;
					//Hand 2
						GameObject Plankk = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
                        SpriteRenderer PlankkRenderer = Plankk.GetComponent<SpriteRenderer>();
                        PlankkRenderer.sprite = ModAPI.LoadSprite("Mobs/Allay/Allay Hand.png", 1f);
                        PlankkRenderer.sortingOrder = 2;
                        PlankkRenderer.sortingLayerName = "Bottom";
                        Plankk.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Plankk.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                        foreach (var c in Plankk.GetComponents<Collider2D>())
                        {
                            GameObject.Destroy(c);
                        }
                        Plankk.AddComponent<BoxCollider2D>();
                        Plankk.transform.parent = Instance.transform;
                        Plankk.transform.rotation = Instance.transform.rotation;
                        Plankk.transform.localPosition = new Vector3(0f, 0.0f);
                        Plankk.transform.localScale = new Vector3(1f, 1f);
                        HingeJoint2D PlankkhingeJoint = Plankk.GetOrAddComponent<HingeJoint2D>();
                        PlankkhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
                        PlankkhingeJoint.anchor = new Vector3(0f, 0.15f);
                        JointAngleLimits2D Plankklimits = PlankkhingeJoint.limits;
                        Plankklimits.min = 50;
                        Plankklimits.max = -50;
                        PlankkhingeJoint.limits = Plankklimits;
                        PlankkhingeJoint.useLimits = false;
					//Wings
						GameObject IV = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
                        SpriteRenderer IVRenderer = IV.GetComponent<SpriteRenderer>();
                        IVRenderer.sprite = ModAPI.LoadSprite("Mobs/Allay/Allay Wings.png", 1f);
                        IVRenderer.sortingOrder = 2;
                        IVRenderer.sortingLayerName = "Foreground";
                        IV.AddComponent<AudioSourceTimeScaleBehaviour>();
                        IV.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
                        foreach (var c in IV.GetComponents<Collider2D>())
                        {
                            GameObject.Destroy(c);
                        }
                        IV.AddComponent<BoxCollider2D>();
                        IV.transform.parent = Instance.transform;
                        IV.transform.rotation = Instance.transform.rotation;
                        IV.transform.localPosition = new Vector3(-0.2f, 0.0f);
                        IV.transform.localScale = new Vector3(1f, 1f);
                        HingeJoint2D IVhingeJoint = IV.GetOrAddComponent<HingeJoint2D>();
                        IVhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
                        IVhingeJoint.anchor = new Vector3(0.2f, 0.1f);
                        JointAngleLimits2D IVlimits = IVhingeJoint.limits;
                        IVlimits.min = 3;
                        IVlimits.max = -3;
                        IVhingeJoint.limits = IVlimits;
                        IVhingeJoint.useLimits = true;
					}
				}
			);
	//Copper Golem
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Bowling Pin"),
					NameOverride = "Copper Golem",
					NameToOrderByOverride = "A",
					DescriptionOverride = "Copper Golem really likes to push the copper buttons... which are not there yet.",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Cooper Golem.png"),
					AfterSpawn = (Instance) =>
					{
						PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                    	phys.Properties = ModAPI.FindPhysicalProperties("Human");
						Instance.GetComponent<SpriteRenderer>().sprite =  ModAPI.LoadSprite("Mobs/Cooper Golem/Cooper Golem Body.png", 1f);
						foreach (var c in Instance.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Instance.AddComponent<PolygonCollider2D>();
						Instance.AddComponent<NoChildrenCollide>();
					//Head
						GameObject I = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						
						SpriteRenderer IRenderer = I.GetComponent<SpriteRenderer>();
						IRenderer.sprite = ModAPI.LoadSprite("Mobs/Cooper Golem/Cooper Golem Head.png", 1f);
						IRenderer.sortingOrder = 2;
						IRenderer.sortingLayerName = "Middle";
                        I.AddComponent<AudioSourceTimeScaleBehaviour>();
                        I.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						foreach (var c in I.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						I.AddComponent<CircleCollider2D>();
						I.transform.parent = Instance.transform;
						I.transform.rotation = Instance.transform.rotation;
						I.transform.localPosition = new Vector3(0f, 0.28f);
						I.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D hingeJoint = I.GetOrAddComponent<HingeJoint2D>();
						hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						hingeJoint.anchor = new Vector3(0f, -0.13f);

						JointAngleLimits2D limits = hingeJoint.limits;
						limits.min = 30;
						limits.max = -30;
						hingeJoint.limits = limits;
						hingeJoint.useLimits = true;
					
					
					//Leg
						GameObject Plankk = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
                        Plankk.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Plankk.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						//new GameObject("AllayI");
						
						SpriteRenderer PlankkRenderer = Plankk.GetComponent<SpriteRenderer>();
						PlankkRenderer.sprite = ModAPI.LoadSprite("Mobs/Cooper Golem/Cooper Golem Leg.png", 1f);
						PlankkRenderer.sortingOrder = 2;
						PlankkRenderer.sortingLayerName = "Foreground";
                        
						foreach (var c in Plankk.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Plankk.AddComponent<BoxCollider2D>();
						Plankk.transform.parent = Instance.transform;
						Plankk.transform.rotation = Instance.transform.rotation;
						Plankk.transform.localPosition = new Vector3(0f, -0.15f);
						Plankk.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D PlankkhingeJoint = Plankk.GetOrAddComponent<HingeJoint2D>();
						PlankkhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						PlankkhingeJoint.anchor = new Vector3(0f, 0.05f);

						JointAngleLimits2D Plankklimits = hingeJoint.limits;
						Plankklimits.min = 30;
						Plankklimits.max = -30;
						PlankkhingeJoint.limits = limits;
						PlankkhingeJoint.useLimits = true;
					//Hand
						GameObject II = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
                        II.AddComponent<AudioSourceTimeScaleBehaviour>();
                        II.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						//new GameObject("AllayI");
						
						SpriteRenderer IIRenderer = II.GetComponent<SpriteRenderer>();
						IIRenderer.sprite = ModAPI.LoadSprite("Mobs/Cooper Golem/Cooper Golem Hand.png", 1f);
						IIRenderer.sortingOrder = 2;
						IIRenderer.sortingLayerName = "Foreground";
						foreach (var c in II.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						II.AddComponent<BoxCollider2D>();
						II.transform.parent = Instance.transform;
						II.transform.rotation = Instance.transform.rotation;
						II.transform.localPosition = new Vector3(0f, 0f);
						II.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D IIhingeJoint = II.GetOrAddComponent<HingeJoint2D>();
						IIhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						IIhingeJoint.anchor = new Vector3(0f, 0.05f);

						JointAngleLimits2D IIlimits = hingeJoint.limits;
						IIlimits.min = 30;
						IIlimits.max = -30;
						IIhingeJoint.limits = limits;
						IIhingeJoint.useLimits = false;

					//Hand2
						GameObject Hand2 = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
                        Hand2.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Hand2.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						//new GameObject("AllayI");
						
						SpriteRenderer Hand2Renderer = Hand2.GetComponent<SpriteRenderer>();
						Hand2Renderer.sprite = ModAPI.LoadSprite("Mobs/Cooper Golem/Cooper Golem Hand.png", 1f);
						Hand2Renderer.sortingOrder = 2;
						Hand2Renderer.sortingLayerName = "Bottom";
						foreach (var c in Hand2.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Hand2.AddComponent<BoxCollider2D>();
						Hand2.transform.parent = Instance.transform;
						Hand2.transform.rotation = Instance.transform.rotation;
						Hand2.transform.localPosition = new Vector3(0f, 0f);
						Hand2.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D Hand2hingeJoint = Hand2.GetOrAddComponent<HingeJoint2D>();
						Hand2hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						Hand2hingeJoint.anchor = new Vector3(0f, 0.05f);

						JointAngleLimits2D Hand2limits = hingeJoint.limits;
						Hand2limits.min = 30;
						Hand2limits.max = -30;
						Hand2hingeJoint.limits = Hand2limits;
						Hand2hingeJoint.useLimits = false;

					
					//Leg
						GameObject Leg2 = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
                        Leg2.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Leg2.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						//new GameObject("AllayI");
						
						SpriteRenderer Leg2Renderer = Leg2.GetComponent<SpriteRenderer>();
						Leg2Renderer.sprite = ModAPI.LoadSprite("Mobs/Cooper Golem/Cooper Golem Leg.png", 1f);
						Leg2Renderer.sortingOrder = 2;
						Leg2Renderer.sortingLayerName = "Bottom";
						foreach (var c in Leg2.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Leg2.AddComponent<BoxCollider2D>();
						Leg2.transform.parent = Instance.transform;
						Leg2.transform.rotation = Instance.transform.rotation;
						Leg2.transform.localPosition = new Vector3(0f, -0.15f);
						Leg2.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D Leg2hingeJoint = Leg2.GetOrAddComponent<HingeJoint2D>();
						Leg2hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						Leg2hingeJoint.anchor = new Vector3(0f, 0.05f);

						JointAngleLimits2D Leg2limits = hingeJoint.limits;
						Leg2limits.min = 30;
						Leg2limits.max = -30;
						Leg2hingeJoint.limits = Leg2limits;
						Leg2hingeJoint.useLimits = true;
					}
				}
			);
    
	//Glow Squid
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Bowling Pin"),
					NameOverride = "Glow Squid",
					NameToOrderByOverride = "A",
					DescriptionOverride = "Glow Squid is a cute creature that lives in water. It wants to eat you.",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Glow Squid.png"),
					AfterSpawn = (Instance) =>
					{
						PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                    	phys.Properties = ModAPI.FindPhysicalProperties("Human");
						Instance.GetComponent<SpriteRenderer>().sprite =  ModAPI.LoadSprite("Mobs/Glow Squid/Body.png", 1f);
						foreach (var c in Instance.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Instance.AddComponent<PolygonCollider2D>();
						Instance.AddComponent<NoChildrenCollide>();
						bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(1.9f * (flipped ? -1f : 1f), 1.9f);
					//Head
						GameObject I = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						I.AddComponent<AudioSourceTimeScaleBehaviour>();
						I.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");

						SpriteRenderer IRenderer = I.GetComponent<SpriteRenderer>();
						IRenderer.sprite = ModAPI.LoadSprite("Mobs/Glow Squid/Leg.png", 1f);
						IRenderer.sortingOrder = 2;
						IRenderer.sortingLayerName = "Middle";
						foreach (var c in I.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						I.AddComponent<PolygonCollider2D>();
						I.transform.parent = Instance.transform;
						I.transform.rotation = Instance.transform.rotation;
						I.transform.localPosition = new Vector3(0f, -0.47f);
						I.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D hingeJoint = I.GetOrAddComponent<HingeJoint2D>();
						hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						hingeJoint.anchor = new Vector3(0f, 0.25f);

						JointAngleLimits2D limits = hingeJoint.limits;
						limits.min = 30;
						limits.max = -30;
						hingeJoint.limits = limits;
						hingeJoint.useLimits = true;
										
					//Leg
						GameObject Plankk = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Plankk.AddComponent<AudioSourceTimeScaleBehaviour>();
						Plankk.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						
						SpriteRenderer PlankkRenderer = Plankk.GetComponent<SpriteRenderer>();
						PlankkRenderer.sprite = ModAPI.LoadSprite("Mobs/Glow Squid/Leg.png", 1f);
						PlankkRenderer.sortingOrder = 2;
						PlankkRenderer.sortingLayerName = "Foreground";
						foreach (var c in Plankk.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Plankk.AddComponent<BoxCollider2D>();
						Plankk.transform.parent = Instance.transform;
						Plankk.transform.rotation = Instance.transform.rotation;
						Plankk.transform.localPosition = new Vector3(-0.1f, -0.47f);
						Plankk.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D PlankkhingeJoint = Plankk.GetOrAddComponent<HingeJoint2D>();
						PlankkhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						PlankkhingeJoint.anchor = new Vector3(0f, 0.25f);
						Debug.Log("Broken");
							

						JointAngleLimits2D Plankklimits = hingeJoint.limits;
						Plankklimits.min = 90;
						Plankklimits.max = -30;
						PlankkhingeJoint.limits = limits;
						PlankkhingeJoint.useLimits = true;
					//Leg 2
						GameObject II = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						II.AddComponent<AudioSourceTimeScaleBehaviour>();
						II.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						
						SpriteRenderer IIRenderer = II.GetComponent<SpriteRenderer>();
						IIRenderer.sprite = ModAPI.LoadSprite("Mobs/Glow Squid/Leg.png", 1f);
						IIRenderer.sortingOrder = 2;
						IIRenderer.sortingLayerName = "Foreground";
						foreach (var c in II.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						II.AddComponent<BoxCollider2D>();
						II.transform.parent = Instance.transform;
						II.transform.rotation = Instance.transform.rotation;
						II.transform.localPosition = new Vector3(0.1f, -0.47f);
						II.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D IIhingeJoint = II.GetOrAddComponent<HingeJoint2D>();
						IIhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						IIhingeJoint.anchor = new Vector3(0f, 0.25f);

						JointAngleLimits2D IIlimits = hingeJoint.limits;
						IIlimits.min = 90;
						IIlimits.max = -30;
						IIhingeJoint.limits = limits;
						IIhingeJoint.useLimits = true;
					}
				}
			);
#endregion
#region Weapons
    //Wooden Sword
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Sword"), 
                    NameOverride = "Wooden Sword", // Name.
                    NameToOrderByOverride = "B", //  
                    DescriptionOverride = "Sword. Kill hostile creatures with it",// Description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),//Category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/15.png", 5f),//Thumbnail
                    AfterSpawn = (Instance) =>
                    { 
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/Wooden_Sword.png", 9f);
                        foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<PolygonCollider2D>();
                    }
                }
            );
    //Stone Sword
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Sword"), 
                    NameOverride = "Stone Sword", // Name.
                    NameToOrderByOverride = "B", //  
                    DescriptionOverride = "Sword. Kill hostile creatures with it",// Description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),//Category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/14.png", 5f),//Thumbnail
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/Stone_Sword.png", 9f);
                        foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<PolygonCollider2D>();
                    }
                }
            );
    //Iron Sword
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Sword"), 
                    NameOverride = "Iron Sword", // Name.
                    NameToOrderByOverride = "B", //  
                    DescriptionOverride = "Sword. Kill hostile creatures with it",// Description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),//Category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/13.png", 5f),//Thumbnail
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/Iron_Sword.png", 9f);
                        foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<PolygonCollider2D>();
                    }
                }
            );
    //Golden Sword
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Sword"), 
                    NameOverride = "Golden Sword", // Name.
                    NameToOrderByOverride = "B", //  
                    DescriptionOverride = "Sword. Kill hostile creatures with it",// Description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),//Category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/12.png", 5f),//Thumbnail
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/golden_Sword.png", 9f);
                        foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<PolygonCollider2D>();
                    }
                }
            );
    //Diamond Sword
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Sword"), 
                    NameOverride = "Diamond Sword", // Name.
                    NameToOrderByOverride = "B", //  
                    DescriptionOverride = "Sword. Kill hostile creatures with it",// Description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),//Category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/9.png", 5f),//Thumbnail
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/diamond_Sword.png", 9f);
                        foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<PolygonCollider2D>();
                    }
                }
            );
    //Netherite Sword
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Sword"), 
                    NameOverride = "Netherite Sword", // Name.
                    NameToOrderByOverride = "B", //  
                    DescriptionOverride = "Sword. Kill hostile creatures with it",// Description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),//Category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/11.png", 5f),//Thumbnail
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/Netherite_Sword.png", 9f);
                        foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<PolygonCollider2D>();
                    }
                }
            );
    
    //Wooden Axe
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Axe"),
                    NameOverride = "Wooden Axe",
                    NameToOrderByOverride = "B",
                    DescriptionOverride = "Axe. Chop wood and more with it.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/43.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        bool flipped = Instance.transform.localScale.x < 0f;
                        Instance.transform.localScale = new Vector2(1.3f * (flipped ? -1f : 1f), 1.3f)  
                          ;
                        var axeHandle = Instance.transform.GetChild(0);
                        axeHandle.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/Stick_Wooden.png", 7.7f);
                        var axeHead = axeHandle.GetChild(0);
                        axeHead.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/wooden_axe.png", 7.7f);
                        axeHead.localPosition = new Vector3(0.025f, 0.26f);
                        axeHandle.gameObject.FixColliders();
                        axeHead.gameObject.FixColliders();
                        var properties = ModAPI.FindPhysicalProperties("Metal");
                        properties.Sharp = true;
                        properties.SharpAxes = new SharpAxis[]
                        {
                            new SharpAxis(Vector2.right, 0f, 0.25f, true, false)
                        };
                        axeHead.gameObject.GetComponent<PhysicalBehaviour>().Properties = properties;
                    }
                }
            );
    //Stone Axe
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Axe"),
                    NameOverride = "Stone Axe",
                    NameToOrderByOverride = "B",
                    DescriptionOverride = "Axe. Chop wood and more with it.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/34.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        bool flipped = Instance.transform.localScale.x < 0f;
                        Instance.transform.localScale = new Vector2(1.3f * (flipped ? -1f : 1f), 1.3f)  
                          ;
                        var axeHandle = Instance.transform.GetChild(0);
                        axeHandle.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/Stick_Wooden.png", 7.7f);
                        var axeHead = axeHandle.GetChild(0);
                        axeHead.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/stone_axe.png", 7.7f);
                        axeHead.localPosition = new Vector3(0.025f, 0.26f);
                        axeHandle.gameObject.FixColliders();
                        axeHead.gameObject.FixColliders();
                        var properties = ModAPI.FindPhysicalProperties("Metal");
                        properties.Sharp = true;
                        properties.SharpAxes = new SharpAxis[]
                        {
                            new SharpAxis(Vector2.right, 0f, 0.25f, true, false)
                        };
                        axeHead.gameObject.GetComponent<PhysicalBehaviour>().Properties = properties;
                    }
                }
            );
    //Iron Axe
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Axe"),
                    NameOverride = "Iron Axe",
                    NameToOrderByOverride = "B",
                    DescriptionOverride = "Axe. Chop wood and more with it.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/33.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        bool flipped = Instance.transform.localScale.x < 0f;
                        Instance.transform.localScale = new Vector2(1.3f * (flipped ? -1f : 1f), 1.3f)  
                          ;
                        var axeHandle = Instance.transform.GetChild(0);
                        axeHandle.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/Stick_Wooden.png", 7.7f);
                        var axeHead = axeHandle.GetChild(0);
                        axeHead.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/iron_axe.png", 7.7f);
                        axeHead.localPosition = new Vector3(0.025f, 0.26f);
                        axeHandle.gameObject.FixColliders();
                        axeHead.gameObject.FixColliders();
                        var properties = ModAPI.FindPhysicalProperties("Metal");
                        properties.Sharp = true;
                        properties.SharpAxes = new SharpAxis[]
                        {
                            new SharpAxis(Vector2.right, 0f, 0.25f, true, false)
                        };
                        axeHead.gameObject.GetComponent<PhysicalBehaviour>().Properties = properties;
                    }
                }
            );
    //Golden Axe
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Axe"),
                    NameOverride = "Golden Axe",
                    NameToOrderByOverride = "B",
                    DescriptionOverride = "Axe. Chop wood and more with it.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/35.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        bool flipped = Instance.transform.localScale.x < 0f;
                        Instance.transform.localScale = new Vector2(1.3f * (flipped ? -1f : 1f), 1.3f)  
                          ;
                        var axeHandle = Instance.transform.GetChild(0);
                        axeHandle.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/Stick_Wooden.png", 7.7f);
                        var axeHead = axeHandle.GetChild(0);
                        axeHead.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/golden_axe.png", 7.7f);
                        axeHead.localPosition = new Vector3(0.025f, 0.26f);
                        axeHandle.gameObject.FixColliders();
                        axeHead.gameObject.FixColliders();
                        var properties = ModAPI.FindPhysicalProperties("Metal");
                        properties.Sharp = true;
                        properties.SharpAxes = new SharpAxis[]
                        {
                            new SharpAxis(Vector2.right, 0f, 0.25f, true, false)
                        };
                        axeHead.gameObject.GetComponent<PhysicalBehaviour>().Properties = properties;
                    }
                }
            );
    //Diamond Axe
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Axe"),
                    NameOverride = "Diamond Axe",
                    NameToOrderByOverride = "B",
                    DescriptionOverride = "Axe. Chop wood and more with it.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/32.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        bool flipped = Instance.transform.localScale.x < 0f;
                        Instance.transform.localScale = new Vector2(1.3f * (flipped ? -1f : 1f), 1.3f)  
                          ;
                        var axeHandle = Instance.transform.GetChild(0);
                        axeHandle.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/Stick_Wooden.png", 7.7f);
                        var axeHead = axeHandle.GetChild(0);
                        axeHead.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/diamond_axe.png", 7.7f);
                        axeHead.localPosition = new Vector3(0.025f, 0.26f);
                        axeHandle.gameObject.FixColliders();
                        axeHead.gameObject.FixColliders();
                        var properties = ModAPI.FindPhysicalProperties("Metal");
                        properties.Sharp = true;
                        properties.SharpAxes = new SharpAxis[]
                        {
                            new SharpAxis(Vector2.right, 0f, 0.25f, true, false)
                        };
                        axeHead.gameObject.GetComponent<PhysicalBehaviour>().Properties = properties;
                    }
                }
            );
    //Netherite Axe
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Axe"),
                    NameOverride = "Axe. Chop wood and more with it.",
                    NameToOrderByOverride = "B",
                    DescriptionOverride = "strong weapon",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/36.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        bool flipped = Instance.transform.localScale.x < 0f;
                        Instance.transform.localScale = new Vector2(1.3f * (flipped ? -1f : 1f), 1.3f)  
                          ;
                        var axeHandle = Instance.transform.GetChild(0);
                        axeHandle.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/Stick_Netherite.png", 7.7f);
                        var axeHead = axeHandle.GetChild(0);
                        axeHead.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/netherite_axe.png", 7.7f);
                        axeHead.localPosition = new Vector3(0.025f, 0.26f);
                        axeHandle.gameObject.FixColliders();
                        axeHead.gameObject.FixColliders();
                        var properties = ModAPI.FindPhysicalProperties("Metal");
                        properties.Sharp = true;
                        properties.SharpAxes = new SharpAxis[]
                        {
                            new SharpAxis(Vector2.right, 0f, 0.25f, true, false)
                        };
                        axeHead.gameObject.GetComponent<PhysicalBehaviour>().Properties = properties;
                        axeHandle.gameObject.GetComponent<PhysicalBehaviour>().Properties = properties;
                    }
                }
            );
    //Wooden Shovel
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Sword"), 
                    NameOverride = "Wooden Shovel",
                    NameToOrderByOverride = "B", 
                    DescriptionOverride = "Shovel. Kill hostile creatures with it",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/wooden_shovel.png", 5f),
                    AfterSpawn = (Instance) =>
                    { 
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/wooden_shovel.png", 6f);
                        foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<PolygonCollider2D>();
                    }
                }
            );
    //Stone Shovel
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Sword"), 
                    NameOverride = "Stone Shovel",
                    NameToOrderByOverride = "B", 
                    DescriptionOverride = "Shovel. Kill hostile creatures with it",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/stone_shovel.png", 5f),
                    AfterSpawn = (Instance) =>
                    { 
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/stone_shovel.png", 6f);
                        foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<PolygonCollider2D>();
                    }
                }
            );
    //Iron Shovel
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Sword"), 
                    NameOverride = "Iron Shovel",
                    NameToOrderByOverride = "B", 
                    DescriptionOverride = "Shovel. Kill hostile creatures with it",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/iron_shovel.png", 5f),
                    AfterSpawn = (Instance) =>
                    { 
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/iron_shovel.png", 6f);
                        foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<PolygonCollider2D>();
                    }
                }
            );
    //Golden Shovel
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Sword"), 
                    NameOverride = "Golden Shovel",
                    NameToOrderByOverride = "B", 
                    DescriptionOverride = "Shovel. Kill hostile creatures with it",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/golden_shovel.png", 5f),
                    AfterSpawn = (Instance) =>
                    { 
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/golden_shovel.png", 6f);
                        foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<PolygonCollider2D>();
                    }
                }
            );
    //Diamond Shovel
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Sword"), 
                    NameOverride = "Diamond Shovel",
                    NameToOrderByOverride = "B", 
                    DescriptionOverride = "Shovel. Kill hostile creatures with it",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/diamond_shovel.png", 5f),
                    AfterSpawn = (Instance) =>
                    { 
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/diamond_shovel.png", 6f);
                        foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<PolygonCollider2D>();
                    }
                }
            );
    //Netherite Shovel
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Sword"), 
                    NameOverride = "Netherite Shovel",
                    NameToOrderByOverride = "B", 
                    DescriptionOverride = "Shovel. Kill hostile creatures with it",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/netherite_shovel.png", 5f),
                    AfterSpawn = (Instance) =>
                    { 
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/netherite_shovel.png", 6f);
                        foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<PolygonCollider2D>();
                    }
                }
            );
    //Bow
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Crossbow"),
                    NameOverride = "Bow",
                    NameToOrderByOverride = "B",
                    DescriptionOverride = "Hit targets at greater distances with this beautiful bow!",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/BowView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {                      
                        var SpriteTexture = Instance.GetComponent<SpriteRenderer>();
                        var Projectile = Instance.GetComponent<ProjectileLauncherBehaviour>();
						var phys = Instance.GetComponent<PhysicalBehaviour>();

                        SpriteTexture.sprite = ModAPI.LoadSprite("Weapons/Bow.png", 15f);

						Projectile.projectileAsset = ModAPI.FindSpawnable("Arrow");
						Projectile.launchSound = ModAPI.LoadSound("Sfx/bow1.wav");
						Projectile.barrelPosition = new Vector2(0.143f, -0.017f);
						Projectile.IsAutomatic = false;						
						Projectile.ScreenShake = 0.3f;
						Projectile.projectileLaunchStrength = 1.7f;
						phys.Properties = ModAPI.FindPhysicalProperties("Weapon");
						phys.TrueInitialMass = 0.10f;
						phys.InitialMass = 0.10f;
						phys.rigidbody.mass = 0.10f;
                        foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<PolygonCollider2D>();
                    }
                }
            );
    //Crossbow
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Crossbow"),
                    NameOverride = "Crossbow ",
                    NameToOrderByOverride = "B",
                    DescriptionOverride = "Shoot the nearest enemy with this ranged weapon. He will be pleased.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/21.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        var SpriteTexture = Instance.GetComponent<SpriteRenderer>();
                        var Projectile = Instance.GetComponent<ProjectileLauncherBehaviour>();
						var phys = Instance.GetComponent<PhysicalBehaviour>();

                        SpriteTexture.sprite = ModAPI.LoadSprite("Weapons/cross.png", 15.5f);

						Projectile.projectileAsset = ModAPI.FindSpawnable("Arrow");
						Projectile.launchSound = ModAPI.LoadSound("Sfx/sho.wav");
						Projectile.barrelPosition = new Vector2(0.143f, -0.016f);
						Projectile.IsAutomatic = false;						
						Projectile.ScreenShake = 0.5f;
						Projectile.projectileLaunchStrength = 2.5f;
						phys.Properties = ModAPI.FindPhysicalProperties("Weapon");
						phys.TrueInitialMass = 0.10f;
						phys.InitialMass = 0.10f;
						phys.rigidbody.mass = 0.10f;
                        foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<PolygonCollider2D>();
                    }
                }
            );
    //Trident
            ModAPI.Register(
                            new Modification()
                            {
                                OriginalItem = ModAPI.FindSpawnable("Sword"),
                                NameOverride = "Trident",
                                NameToOrderByOverride = "B",
                                DescriptionOverride = "Destroy fish with this weapon. And people, too.",
                                CategoryOverride = ModAPI.FindCategory("Minecraft"),
                                ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/10.png", 5f),
                                AfterSpawn = (Instance) =>
                    {
                    //Main	
						Sprite CustomSprite = ModAPI.LoadSprite("Weapons/Trident.png");
						Sprite OutlineSprite = ModAPI.LoadSprite("Weapons/Outline Sprites/Trident.png");
						float WeaponSize = 1.8f;

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        var properties = ModAPI.FindPhysicalProperties("Wood");										
					//Settings                       
						phys.TrueInitialMass = 0.10f;
						phys.InitialMass = 0.10f;
						phys.rigidbody.mass = 0.10f;
						phys.HoldingPositions = new Vector3[]
						{
							new Vector2(0f, -0.18f),
                            new Vector2(0f, 0.094f),
						};
                        properties.Sharp = true;
                        properties.SharpAxes = new SharpAxis[]
                        {
                            new SharpAxis(Vector2.up, 0f, 0.357f, true, false)
                        };
					//Other
						var OtlineSpriteRenderer = Instance.GetComponent<SpriteRenderer>();
						OtlineSpriteRenderer.sprite = OutlineSprite;
						OtlineSpriteRenderer.sortingLayerName = "Default";
						OtlineSpriteRenderer.sortingOrder = -10;
						
						GameObject NewSprite = new GameObject("CustomSprite");
            			NewSprite.transform.parent = Instance.transform;
            			NewSprite.transform.localPosition = new Vector3(0, 0f);
            			NewSprite.transform.localScale = new Vector3(1f, 1f);
            			var RendererNewSprite = NewSprite.AddComponent<SpriteRenderer>();
                        var RandomNewSprite = NewSprite.AddComponent<RandomSpriteBehaviour>();					
           			 	RandomNewSprite.sprites = new Sprite[]
                            {
                                CustomSprite,
                            };
                        RendererNewSprite.transform.localRotation = Quaternion.Euler(0f, 0f, 0f);
						RendererNewSprite.sortingLayerName = "Default";
						RendererNewSprite.sortingOrder = 0;
						foreach (Collider2D Collison in Instance.GetComponents<Collider2D>())
                        {
                            Collider2D.Destroy(Collison);
                        }
						Instance.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.000335335732f,0.442857265f),
                            new Vector2(-0.0714285746f,0.413471818f),
                            new Vector2(-0.0714285374f,0.299569368f),
                            new Vector2(-0.0145624876f,0.271131516f),
                            new Vector2(-0.0144741908f,-0.442537189f),
                            new Vector2(0.014154233f,-0.442537189f),
                            new Vector2(0.0141267776f,0.270222902f),
                            new Vector2(0.0714285374f,0.299569368f),
                            new Vector2(0.0710518733f,0.413848519f)
							               
                        });				
						bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(WeaponSize * (flipped ? -1f : 1f), WeaponSize);
					//
                }
            }
        );
    //Fishing Rod
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Fishing Rod",
                    NameToOrderByOverride = "B",
                    DescriptionOverride = "Fish or hook someone",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/fishing_rod.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/Fishing Rod.png", 6f);
                        var web = Instance.AddComponent<MaceBehaviour>();

                        Instance.AddComponent<PolygonCollider2D>();

                            PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                            phys.Properties = ModAPI.FindPhysicalProperties("Wood");
                            phys.InitialMass = 0.4f;
                            phys.TrueInitialMass = 0.4f;
                    }
                }
            );
    //Arrow
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Crossbow Bolt"), 
                    NameOverride = "Arrow",
                    NameToOrderByOverride = "B",
                    DescriptionOverride = "Arrow. Has a variety of textures",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),//Category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/ArrowView.png", 5f),//Thumbnail
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>();
                        Instance.AddComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                            {
                                ModAPI.LoadSprite("Weapons/Arrows/Arrow.png"),
                                ModAPI.LoadSprite("Weapons/Arrows/Arrow of Slowness.png"),
                                ModAPI.LoadSprite("Weapons/Arrows/Spectral Arrow.png"),
                            };
                    }
                }
            );
    //Snowball
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Brick"), 
                    NameOverride = "Snowball",
                    NameToOrderByOverride = "B", 
                    DescriptionOverride = "Minecraft Snowball",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Snowball.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/Snowball.png", 1.2f);
						Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Snow");
						foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<CircleCollider2D>();
                        Instance.FixColliders();  
                    }
                }
            );
    //Throwing Snowball
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Crossbow"), 
                    NameOverride = "Throwing Snowball",
                    NameToOrderByOverride = "B", 
                    DescriptionOverride = "Minecraft Snowball",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Throwing Snowball.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Weapons/Snowball.png", 1.2f);
						Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Snow");
						foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<CircleCollider2D>();
                        Instance.FixColliders();
						var Proj = Instance.GetComponent<ProjectileLauncherBehaviour>();
						Proj.projectileAsset = ModAPI.FindSpawnable("Snowball");
						Proj.launchSound = ModAPI.LoadSound("Sfx/Snowball.wav");
						Proj.IsAutomatic = false;
						Proj.ShouldPutProjectileInOtherLayer = false;
						Proj.RemoveLaunchedObjectsWithMe = true;
						Proj.projectileLaunchStrength = 10f;
						Proj.AutomaticInterval = 0.6f;
						Proj.recoilMultiplier = 0.05f;
						Proj.ScreenShake = 0.1f;  
                    }
                }
            );
#endregion
#region Tools
    //Spyglass
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), 
                    NameOverride = "SpyGlass", // Name.
                    NameToOrderByOverride = "C", //  
                    DescriptionOverride = "Press F for ZOOM sound",// Description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),//Category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Spyglass.png", 5f),//Thumbnail
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/Spyglass.png", 0.55f);
                        Instance.FixColliders();
                        var AS = Instance.AddComponent<AudioSource>();
                        AS.priority = 10;
                        AS.volume = 10f;
                        AS.minDistance = 5f; 
                        AS.clip = ModAPI.LoadSound("Sfx/spy.wav");
                        Instance.AddComponent<UseEventTrigger>().Action = () =>
                        {
                            AS.Play();
                        };
                    }
                }
            );        
    //Shield
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Stick"), 
                    NameOverride = "Shield",
                    NameToOrderByOverride = "C",
                    DescriptionOverride = "Get your hands on it and protect yourself from the blows!",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),//Category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Shield.png", 5f),//Thumbnail
                    AfterSpawn = (Instance) =>
                    {
                    //Main	
						Sprite CustomSprite = ModAPI.LoadSprite("Blocks&Stuff/Banners&Shields/Shields/Shield.png");
                        Sprite CustomSprite2 = ModAPI.LoadSprite("Blocks&Stuff/Banners&Shields/Shields/Snowy Shield.png");
                        Sprite CustomSprite3 = ModAPI.LoadSprite("Blocks&Stuff/Banners&Shields/Shields/Ominous Shield.png");
                        Sprite CustomSprite4 = ModAPI.LoadSprite("Blocks&Stuff/Banners&Shields/Shields/White Shield.png");
                        Sprite CustomSprite5 = ModAPI.LoadSprite("Blocks&Stuff/Banners&Shields/Shields/Black Shield.png");
                        Sprite CustomSprite6 = ModAPI.LoadSprite("Blocks&Stuff/Banners&Shields/Shields/Magenta Shield.png");

						Sprite OutlineSprite = ModAPI.LoadSprite("Blocks&Stuff/Outline Sprites/Shield.png");
						float WeaponSize = 1.5f;

                        var phys = Instance.GetComponent<PhysicalBehaviour>();										
					//Settings
						phys.Properties = ModAPI.FindPhysicalProperties("Wood");
						phys.TrueInitialMass = 0.10f;
						phys.InitialMass = 0.10f;
						phys.rigidbody.mass = 0.10f;
						phys.HoldingPositions = new Vector3[]
						{
							new Vector2(-0.0701f, 0f),
						};
					//Other
						var OtlineSpriteRenderer = Instance.GetComponent<SpriteRenderer>();
						OtlineSpriteRenderer.sprite = OutlineSprite;
						OtlineSpriteRenderer.sortingLayerName = "Default";
						OtlineSpriteRenderer.sortingOrder = -10;
						
						GameObject NewSprite = new GameObject("CustomSprite");
            			NewSprite.transform.parent = Instance.transform;
            			NewSprite.transform.localPosition = new Vector3(0, 0f);
            			NewSprite.transform.localScale = new Vector3(1f, 1f);
            			var RendererNewSprite = NewSprite.AddComponent<SpriteRenderer>();
                        var RandomNewSprite = NewSprite.AddComponent<RandomSpriteBehaviour>();					
           			 	RandomNewSprite.sprites = new Sprite[]
                            {
                                CustomSprite,
                                CustomSprite2,
                                CustomSprite3,
                                CustomSprite4,
                                CustomSprite5,
                                CustomSprite6
                            };
                        RendererNewSprite.transform.localRotation = Quaternion.Euler(0f, 0f, 0f);
						RendererNewSprite.sortingLayerName = "Default";
						RendererNewSprite.sortingOrder = 0;
						foreach (Collider2D Collison in Instance.GetComponents<Collider2D>())
                        {
                            Collider2D.Destroy(Collison);
                        }
						Instance.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.0447423235f,0.514285743f),
                            new Vector2(-0.0431288481f,0.0850400925f), 
                            new Vector2(-0.0981514454f,0.084997654f), 
                            new Vector2(-0.0986847878f,-0.0879069567f), 
                            new Vector2(-0.0428942442f,-0.0879050493f), 
                            new Vector2(-0.0413821712f,-0.513841689f), 
                            new Vector2(0.100000001f,-0.514285743f), 
                            new Vector2(0.100000001f,0.514285743f) 
							               
                        });				
						bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(WeaponSize * (flipped ? -1f : 1f), WeaponSize);
					//		
                    }
                }
            );
    //Copper Goat Horn
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Bowling Pin"),
                    NameOverride = "Copper Goat Horn",
                    NameToOrderByOverride = "C",
                    DescriptionOverride = "Copper Goat Horn Villagers, Steve and Alex use to announce something. Press F on the item to activate",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Copper Goat Horn.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite =  ModAPI.LoadSprite("Blocks&Stuff/Copper Goat Horn.png", 1f);
                        foreach (var c in Instance.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Instance.AddComponent<PolygonCollider2D>();
                        var AS = Instance.AddComponent<AudioSource>();
                        AS.priority = 10;
                        AS.volume = 10f;
                        AS.minDistance = 5f; 
                        AS.clip = ModAPI.LoadSound("Sfx/Copper Goat Horn.mp3");
                        Instance.AddComponent<UseEventTrigger>().Action = () =>
                        {
                            AS.Play();
                        };
                        Instance.GetComponent<PhysicalBehaviour>().HoldingPositions = new Vector3[]
                            {
                                new Vector3(0.0008f, -0.0412f, 0f)
                            };
                    }
                }
            );
    //Goat Horn
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Bowling Pin"),
                    NameOverride = "Goat Horn",
                    NameToOrderByOverride = "C",
                    DescriptionOverride = "Goat Horn Illagers Used to alert of danger, winnings, or anything else. Press F on the item to activate ",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Goat Horn.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite =  ModAPI.LoadSprite("Blocks&Stuff/Goat Horn.png", 1f);
                        foreach (var c in Instance.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Instance.AddComponent<PolygonCollider2D>();
                        var AS = Instance.AddComponent<AudioSource>();
                        AS.priority = 10;
                        AS.volume = 10f;
                        AS.minDistance = 5f; 
                        AS.clip = ModAPI.LoadSound("Sfx/raidhorn_02.mp3");
                        Instance.AddComponent<UseEventTrigger>().Action = () =>
                        {
                            AS.Play();
                        };
                        Instance.GetComponent<PhysicalBehaviour>().HoldingPositions = new Vector3[]
                            {
                                new Vector3(0.0008f, -0.0412f, 0f)
                            };
                    }
                }
            );
    //Poppy
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Stick"),
                    NameOverride = "Poppy",
                    NameToOrderByOverride = "C",
                    DescriptionOverride = "Poppy. Press F on the item to activate",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Poppy.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite =  ModAPI.LoadSprite("Blocks&Stuff/Goat Horn.png", 1f);
                        foreach (var c in Instance.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
                        
                        SpriteRenderer SprRen = Instance.GetComponent<SpriteRenderer>();
						SprRen.sortingLayerName = "Bubbles";
						SprRen.sprite = ModAPI.LoadSprite("Blocks&Stuff/Poppy.png", 1f);
							foreach (var ColliderBoxDeleteVar in Instance.GetComponents<Collider2D>())
								{
									GameObject.Destroy(ColliderBoxDeleteVar);
								}
							Instance.FixColliders();
						SprRen.sprite = ModAPI.LoadSprite("Blocks&Stuff/Poppy.png", 1f);

						var destructibleObjectBehaviour = Instance.GetOrAddComponent<DestructibleObjectBehaviour>();
                        destructibleObjectBehaviour.breakSounds = new AudioClip[] //Звуки при ломании объекта.
                        {
                            ModAPI.LoadSound("Sfx/Mossbreak1.mp3"),
                            ModAPI.LoadSound("Sfx/Mossbreak2.mp3"),
                            ModAPI.LoadSound("Sfx/Mossbreak3.mp3"),
                            ModAPI.LoadSound("Sfx/Mossbreak4.mp3"),  
                            ModAPI.LoadSound("Sfx/Mossbreak5.mp3")
                        };
                        destructibleObjectBehaviour.debrisCollisionSprites = new Sprite[] //Текстуры обломков.
                        {
                            ModAPI.LoadSprite("Blocks&Stuff/Debris/Poppy.png", 1f),
                            ModAPI.LoadSprite("Blocks&Stuff/Debris/Poppy2.png", 1f),
                            ModAPI.LoadSprite("Blocks&Stuff/Debris/Poppy3.png", 1f),
                            ModAPI.LoadSprite("Blocks&Stuff/Debris/Poppy4.png", 1f),
                            ModAPI.LoadSprite("Blocks&Stuff/Debris/Poppy5.png", 1f)
                        };
                        destructibleObjectBehaviour.debrisSprites = new Sprite[] //Текстуры обломков.
                        {
                            ModAPI.LoadSprite("Blocks&Stuff/Debris/Poppy.png", 1f),
                            ModAPI.LoadSprite("Blocks&Stuff/Debris/Poppy2.png", 1f),
                            ModAPI.LoadSprite("Blocks&Stuff/Debris/Poppy3.png", 1f),
                            ModAPI.LoadSprite("Blocks&Stuff/Debris/Poppy4.png", 1f),
                            ModAPI.LoadSprite("Blocks&Stuff/Debris/Poppy5.png", 1f)
                        };
                        destructibleObjectBehaviour.debrisPositions = new Vector3[] //Расположение обломков.
                        {
                            new Vector3(0f, 0f),
							new Vector3(0f, 0f),
                            new Vector3(0f, 0f),
                            new Vector3(0f, 0f),
                            new Vector3(0f, 0f)
                        };
                        destructibleObjectBehaviour.debrisProperties = ModAPI.FindPhysicalProperties("Wood"); //Материал обломков.
                        destructibleObjectBehaviour.debrisMass = 0.01f; //Масса обломков.
                        destructibleObjectBehaviour.BreakingThreshold = 1f; //Сила ломания объекта.
                        destructibleObjectBehaviour.debrisNumber = 5; //Количество обломков.
                        Instance.GetComponent<PhysicalBehaviour>().OverrideImpactSounds = new AudioClip[] 
                            {
                                ModAPI.LoadSound("Sfx/Mossbreak1.mp3"),
                                ModAPI.LoadSound("Sfx/Mossbreak2.mp3"),
                                ModAPI.LoadSound("Sfx/Mossbreak3.mp3"),
                                ModAPI.LoadSound("Sfx/Mossbreak4.mp3"),  
                                ModAPI.LoadSound("Sfx/Mossbreak5.mp3")                                                                                  
                            };
                        Instance.AddComponent<PolygonCollider2D>();
                        var AS = Instance.AddComponent<AudioSource>();
                        AS.priority = 10;
                        AS.volume = 10f;
                        AS.minDistance = 5f; 
                        AS.clip = ModAPI.LoadSound("Sfx/VillagerYES.mp3");
                        Instance.AddComponent<UseEventTrigger>().Action = () =>
                        {
                            AS.Play();
                        };
                    }
                }
            );


#endregion     
#region Stuff
    //Minecart
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Bowling Pin"),
					NameOverride = "Minecart",
					NameToOrderByOverride = "D",
					DescriptionOverride = "Ride the rails on this.",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Minecart.png"),
					AfterSpawn = (Instance) =>
					{
                        float ObjectSize = 3.5f;
                        Sprite MinecartBody = ModAPI.LoadSprite("Vehicles/Minecart.png");
                        Sprite MinecartWheel = ModAPI.LoadSprite("Vehicles/Minecart Wheel.png");
                        Sprite MinecartBackground = ModAPI.LoadSprite("Vehicles/Minecart Background.png");

						PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                        SpriteRenderer ThisSprite = Instance.GetComponent<SpriteRenderer>();
						phys.Properties = ModAPI.FindPhysicalProperties("Metal");
						phys.TrueInitialMass = 0.5f;
						phys.InitialMass = 0.5f;
						phys.rigidbody.mass = 0.5f;
                        phys.DisplayBloodDecals = false;

                        ThisSprite.sprite = MinecartBody;
                        ThisSprite.sortingOrder = 10;
                        ThisSprite.sortingLayerName = "Bubbles";

                        foreach (Collider2D Collison in Instance.GetComponents<Collider2D>())
						{
							Collider2D.Destroy(Collison);
						}
						Instance.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
						{
							new Vector2(0.200098991f,0.114285469f),
                            new Vector2(0.199133873f,-0.08527565f),  
                            new Vector2(-0.199242592f,-0.0860557556f),  
                            new Vector2(-0.199912071f,0.114285469f),  
                            new Vector2(-0.22857143f,0.114285715f),  
                            new Vector2(-0.22857143f,-0.114285715f),  
                            new Vector2(0.22857143f,-0.114285715f),  
                            new Vector2(0.22857143f,0.114285715f)          
						});	
                        #region Wheel
                        GameObject Wheel = new GameObject("Wheel");
						var ThisRend = Wheel.AddComponent<SpriteRenderer>();
						var ThisRig = Wheel.AddComponent<Rigidbody2D>();
						var ThisPhys = Wheel.gameObject.AddComponent<PhysicalBehaviour>();
						HingeJoint2D ThisJoint = Wheel.AddComponent<HingeJoint2D>();     

						Wheel.transform.parent = Instance.transform; 
						Wheel.transform.localScale = Vector3.one; 
						Wheel.transform.localPosition = new Vector3(-0.1421f, -0.09990025f);
                        Wheel.transform.localScale = new Vector3(0.9f, 0.9f);
						ThisRend.sprite = MinecartWheel;                    
						ThisRend.sortingLayerName = "Default";
						ThisRend.sortingOrder = 0; 
						ThisRend.transform.localRotation = Quaternion.Euler(0f, 0f, 0f);

						ThisPhys.Properties = ModAPI.FindPhysicalProperties("Metal");
						ThisRig.mass = 0.2f;

						ThisJoint.anchor = new Vector2(0f, 0f);			                                         
						ThisJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						ThisJoint.breakForce = Mathf.Infinity;
						ThisJoint.breakTorque = Mathf.Infinity;
						Wheel.layer = LayerMask.NameToLayer("Objects");                    
						Wheel.AddComponent<CircleCollider2D>();
                        Wheel.AddComponent<AudioSourceTimeScaleBehaviour>();
                        #endregion  
                        #region WheelTwo
                        GameObject Wheel2 = new GameObject("Wheel2");
						var ThisRend2 = Wheel2.AddComponent<SpriteRenderer>();
						var ThisRig2 = Wheel2.AddComponent<Rigidbody2D>();
						var ThisPhys2 = Wheel2.gameObject.AddComponent<PhysicalBehaviour>();
						HingeJoint2D ThisJoint2 = Wheel2.AddComponent<HingeJoint2D>();     

						Wheel2.transform.parent = Instance.transform; 
						Wheel2.transform.localScale = Vector3.one; 
						Wheel2.transform.localPosition = new Vector3(0.1421f, -0.09990025f);
                        Wheel2.transform.localScale = new Vector3(0.9f, 0.9f);
						ThisRend2.sprite = MinecartWheel;                    
						ThisRend2.sortingLayerName = "Default";
						ThisRend2.sortingOrder = 0; 
						ThisRend2.transform.localRotation = Quaternion.Euler(0f, 0f, 0f);

						ThisPhys2.Properties = ModAPI.FindPhysicalProperties("Metal");
						ThisRig2.mass = 0.2f;

						ThisJoint2.anchor = new Vector2(0f, 0f);			                                         
						ThisJoint2.connectedBody = Instance.GetComponent<Rigidbody2D>();
						ThisJoint2.breakForce = Mathf.Infinity;
						ThisJoint2.breakTorque = Mathf.Infinity;
						Wheel2.layer = LayerMask.NameToLayer("Objects");                    
						Wheel2.AddComponent<CircleCollider2D>();
                        Wheel2.AddComponent<AudioSourceTimeScaleBehaviour>();
                        #endregion  
                        #region Background
                        GameObject Background = new GameObject("Background");
						var ThisRend3 = Background.AddComponent<SpriteRenderer>();
						var ThisPhys3 = Background.gameObject.AddComponent<PhysicalBehaviour>();

						Background.transform.parent = Instance.transform; 
						Background.transform.localScale = Vector3.one; 
						Background.transform.localPosition = new Vector3(0f, 0f);
                        Background.transform.localScale = new Vector3(1f, 1f);
						ThisRend3.sprite = MinecartBackground;                    
						ThisRend3.sortingLayerName = "Background";
						ThisRend3.sortingOrder = 0; 
						ThisRend3.transform.localRotation = Quaternion.Euler(0f, 0f, 0f);

                        Background.layer = 10;                    
                        Background.AddComponent<AudioSourceTimeScaleBehaviour>(); 
                        #endregion     

                        BoxCollider2D trigger = Instance.AddComponent<BoxCollider2D>();
                        trigger.offset = new Vector2(-0.0006079674f, 0.007300496f);
                        trigger.size = new Vector2(0.4291582f, 0.2139704f);
                        trigger.isTrigger = true;

                        NoCollideWithObjectsInTrigger noCollideWithObjectsInTrigger = Instance.gameObject.AddComponent<NoCollideWithObjectsInTrigger>();
                        noCollideWithObjectsInTrigger.Colliders = new Collider2D[]
                        {
                            Wheel.gameObject.GetComponent<CircleCollider2D>(),
                            Wheel2.gameObject.GetComponent<CircleCollider2D>(),
                        };
                        noCollideWithObjectsInTrigger.Trigger = trigger;

                        bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(ObjectSize * (flipped ? -1f : 1f), ObjectSize);         
                	}
				}
			);
    //Minecart With TNT
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Bowling Pin"),
					NameOverride = "Minecart With TNT",
					NameToOrderByOverride = "D",
					DescriptionOverride = "Ride the rails on this.",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Minecart With TNT.png"),
					AfterSpawn = (Instance) =>
					{
                        float ObjectSize = 3.5f;
                        Sprite MinecartBody = ModAPI.LoadSprite("Vehicles/Minecart.png");
                        Sprite MinecartWheel = ModAPI.LoadSprite("Vehicles/Minecart Wheel.png");
                        Sprite MinecartBackground = ModAPI.LoadSprite("Vehicles/Minecart Background.png");
                        Sprite TNTBlock = ModAPI.LoadSprite("Blocks&Stuff/tnt.png");


						PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                        SpriteRenderer ThisSprite = Instance.GetComponent<SpriteRenderer>();
						phys.Properties = ModAPI.FindPhysicalProperties("Metal");
						phys.TrueInitialMass = 0.5f;
						phys.InitialMass = 0.5f;
						phys.rigidbody.mass = 0.5f;
                        phys.DisplayBloodDecals = false;

                        ThisSprite.sprite = MinecartBody;
                        ThisSprite.sortingOrder = 10;
                        ThisSprite.sortingLayerName = "Bubbles";

                        foreach (Collider2D Collison in Instance.GetComponents<Collider2D>())
						{
							Collider2D.Destroy(Collison);
						}
						Instance.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
						{
							new Vector2(0.200098991f,0.114285469f),
                            new Vector2(0.199133873f,-0.08527565f),  
                            new Vector2(-0.199242592f,-0.0860557556f),  
                            new Vector2(-0.199912071f,0.114285469f),  
                            new Vector2(-0.22857143f,0.114285715f),  
                            new Vector2(-0.22857143f,-0.114285715f),  
                            new Vector2(0.22857143f,-0.114285715f),  
                            new Vector2(0.22857143f,0.114285715f)          
						});	
                    #region Wheel
                        GameObject Wheel = new GameObject("Wheel");
						var ThisRend = Wheel.AddComponent<SpriteRenderer>();
						var ThisRig = Wheel.AddComponent<Rigidbody2D>();
						var ThisPhys = Wheel.gameObject.AddComponent<PhysicalBehaviour>();
						HingeJoint2D ThisJoint = Wheel.AddComponent<HingeJoint2D>();     

						Wheel.transform.parent = Instance.transform; 
						Wheel.transform.localScale = Vector3.one; 
						Wheel.transform.localPosition = new Vector3(-0.1421f, -0.09990025f);
                        Wheel.transform.localScale = new Vector3(0.9f, 0.9f);
						ThisRend.sprite = MinecartWheel;                    
						ThisRend.sortingLayerName = "Default";
						ThisRend.sortingOrder = 0; 
						ThisRend.transform.localRotation = Quaternion.Euler(0f, 0f, 0f);

						ThisPhys.Properties = ModAPI.FindPhysicalProperties("Metal");
						ThisRig.mass = 0.2f;

						ThisJoint.anchor = new Vector2(0f, 0f);			                                         
						ThisJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						ThisJoint.breakForce = Mathf.Infinity;
						ThisJoint.breakTorque = Mathf.Infinity;
						Wheel.layer = LayerMask.NameToLayer("Objects");                    
						Wheel.AddComponent<CircleCollider2D>();
                        Wheel.AddComponent<AudioSourceTimeScaleBehaviour>();
                    #endregion  
                    #region WheelTwo
                        GameObject Wheel2 = new GameObject("Wheel2");
						var ThisRend2 = Wheel2.AddComponent<SpriteRenderer>();
						var ThisRig2 = Wheel2.AddComponent<Rigidbody2D>();
						var ThisPhys2 = Wheel2.gameObject.AddComponent<PhysicalBehaviour>();
						HingeJoint2D ThisJoint2 = Wheel2.AddComponent<HingeJoint2D>();     

						Wheel2.transform.parent = Instance.transform; 
						Wheel2.transform.localScale = Vector3.one; 
						Wheel2.transform.localPosition = new Vector3(0.1421f, -0.09990025f);
                        Wheel2.transform.localScale = new Vector3(0.9f, 0.9f);
						ThisRend2.sprite = MinecartWheel;                    
						ThisRend2.sortingLayerName = "Default";
						ThisRend2.sortingOrder = 0; 
						ThisRend2.transform.localRotation = Quaternion.Euler(0f, 0f, 0f);

						ThisPhys2.Properties = ModAPI.FindPhysicalProperties("Metal");
						ThisRig2.mass = 0.2f;

						ThisJoint2.anchor = new Vector2(0f, 0f);			                                         
						ThisJoint2.connectedBody = Instance.GetComponent<Rigidbody2D>();
						ThisJoint2.breakForce = Mathf.Infinity;
						ThisJoint2.breakTorque = Mathf.Infinity;
						Wheel2.layer = LayerMask.NameToLayer("Objects");                    
						Wheel2.AddComponent<CircleCollider2D>();
                        Wheel2.AddComponent<AudioSourceTimeScaleBehaviour>();
                        #endregion  
                    #region Background
                        GameObject Background = new GameObject("Background");
						var ThisRend3 = Background.AddComponent<SpriteRenderer>();
						var ThisPhys3 = Background.gameObject.AddComponent<PhysicalBehaviour>();

						Background.transform.parent = Instance.transform; 
						Background.transform.localScale = Vector3.one; 
						Background.transform.localPosition = new Vector3(0f, 0f);
                        Background.transform.localScale = new Vector3(1f, 1f);
						ThisRend3.sprite = MinecartBackground;                    
						ThisRend3.sortingLayerName = "Background";
						ThisRend3.sortingOrder = 0; 
						ThisRend3.transform.localRotation = Quaternion.Euler(0f, 0f, 0f);

                        Background.layer = 10;                    
                        Background.AddComponent<AudioSourceTimeScaleBehaviour>(); 
                    #endregion 
                    #region TNT
                        GameObject TNTObject = new GameObject("Wheel");
						var ThisRend4 = Wheel.AddComponent<SpriteRenderer>();
						var ThisRig4 = Wheel.AddComponent<Rigidbody2D>();
						var ThisPhys4 = Wheel.gameObject.AddComponent<PhysicalBehaviour>();
						HingeJoint2D ThisJoint3 = Wheel.AddComponent<HingeJoint2D>();     

						TNTObject.transform.parent = Instance.transform; 
						TNTObject.transform.localScale = Vector3.one; 
						TNTObject.transform.localPosition = new Vector3(-0.1421f, -0.09990025f);
                        TNTObject.transform.localScale = new Vector3(0.9f, 0.9f);
						ThisRend4.sprite = MinecartWheel;                    
						ThisRend4.sortingLayerName = "Default";
						ThisRend4.sortingOrder = 0; 
						ThisRend4.transform.localRotation = Quaternion.Euler(0f, 0f, 0f);

						ThisPhys4.Properties = ModAPI.FindPhysicalProperties("Metal");
						ThisRig4.mass = 0.2f;

						ThisJoint3.anchor = new Vector2(0f, 0.096f);			                                         
						ThisJoint3.connectedBody = Instance.GetComponent<Rigidbody2D>();
						ThisJoint3.breakForce = Mathf.Infinity;
						ThisJoint3.breakTorque = Mathf.Infinity;
						TNTObject.layer = LayerMask.NameToLayer("Objects");                    
						TNTObject.AddComponent<CircleCollider2D>();
                        TNTObject.AddComponent<AudioSourceTimeScaleBehaviour>();
                    #endregion     

                        BoxCollider2D trigger = Instance.AddComponent<BoxCollider2D>();
                        trigger.offset = new Vector2(-0.0006079674f, 0.007300496f);
                        trigger.size = new Vector2(0.4291582f, 0.2139704f);
                        trigger.isTrigger = true;

                        NoCollideWithObjectsInTrigger noCollideWithObjectsInTrigger = Instance.gameObject.AddComponent<NoCollideWithObjectsInTrigger>();
                        noCollideWithObjectsInTrigger.Colliders = new Collider2D[]
                        {
                            Wheel.gameObject.GetComponent<CircleCollider2D>(),
                            Wheel2.gameObject.GetComponent<CircleCollider2D>(),
                            TNTObject.gameObject.GetComponent<CircleCollider2D>()
                        };
                        noCollideWithObjectsInTrigger.Trigger = trigger;

                        bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(ObjectSize * (flipped ? -1f : 1f), ObjectSize);         
                	}
				}
			);
    //Boat
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Plank"),
					NameOverride = "Boat",
					NameToOrderByOverride = "D",
					DescriptionOverride = "Travel the sea on this wonderful boat!",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Oak Boat.png"),
					AfterSpawn = (Instance) =>
					{
                    //Main	
						Sprite CustomSprite = ModAPI.LoadSprite("Vehicles/Boats/Oak Boat.png");
                        Sprite CustomSprite2 = ModAPI.LoadSprite("Vehicles/Boats/Acacia Boat.png");
                        Sprite CustomSprite3 = ModAPI.LoadSprite("Vehicles/Boats/Birch Boat.png");
                        Sprite CustomSprite4 = ModAPI.LoadSprite("Vehicles/Boats/Dark Oak Boat.png");
                        Sprite CustomSprite5 = ModAPI.LoadSprite("Vehicles/Boats/Jungle Boat.png");
                        Sprite CustomSprite6 = ModAPI.LoadSprite("Vehicles/Boats/Mangrove Boat.png");
                        Sprite CustomSprite7 = ModAPI.LoadSprite("Vehicles/Boats/Spruce Boat.png");

						Sprite OutlineSprite = ModAPI.LoadSprite("Vehicles/Outline Sprites/Boat.png"); 
						float WeaponSize = 2.4f;

                        var physOther = UnityEngine.Object.Instantiate<PhysicalProperties>(ModAPI.FindPhysicalProperties("Metal"));                    
                        Instance.GetComponent<PhysicalBehaviour>().Properties = physOther;  
                        PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();										
					//Settings
						phys.Properties = ModAPI.FindPhysicalProperties("Wood");
						phys.InitialMass = 5;
                        phys.TrueInitialMass = 5;
                        //physOther.Buoyancy = 1;
                        phys.GridRes = 3;
                        phys.LocalColliderGridPoints = new Vector2[]
						{
							//new Vector2(-0.7f, -0.114f),
                            new Vector2(0f, -0.114f),
                            //new Vector2(0.7f, -0.114f),
                            new Vector2(1.5f, 0.003f),
                            new Vector2(-1.5f, 0.003f),
						};
                        physOther.Buoyancy = 1;
					//Other
						var OtlineSpriteRenderer = Instance.GetComponent<SpriteRenderer>();
						OtlineSpriteRenderer.sprite = OutlineSprite;
						OtlineSpriteRenderer.sortingLayerName = "Default";
						OtlineSpriteRenderer.sortingOrder = -10;
						
						GameObject NewSprite = new GameObject("CustomSprite");
            			NewSprite.transform.parent = Instance.transform;
            			NewSprite.transform.localPosition = new Vector3(0, 0f);
            			NewSprite.transform.localScale = new Vector3(1f, 1f);
            			var RendererNewSprite = NewSprite.AddComponent<SpriteRenderer>();
                        var RandomNewSprite = NewSprite.AddComponent<RandomSpriteBehaviour>();					
           			 	RandomNewSprite.sprites = new Sprite[]
                            {
                                CustomSprite,
                                CustomSprite2,
                                CustomSprite3,
                                CustomSprite4,
                                CustomSprite5,
                                CustomSprite6,
                                CustomSprite7
                            };
                        RendererNewSprite.transform.localRotation = Quaternion.Euler(0f, 0f, 0f);
						RendererNewSprite.sortingLayerName = "Foreground";
						RendererNewSprite.sortingOrder = 15;
						foreach (Collider2D Collison in Instance.GetComponents<Collider2D>())
                        {
                            Collider2D.Destroy(Collison);
                        }
						Instance.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.399824262f,0.113736391f),
                            new Vector2(0.398581743f,-0.0270740986f),
                            new Vector2(0.371156454f,-0.0569317341f),
                            new Vector2(-0.37121439f,-0.0564506054f),
                            new Vector2(-0.401638269f,-0.0299460888f),
                            new Vector2(-0.399606824f,0.114285707f),
                            new Vector2(-0.45714286f,0.114285715f),
                            new Vector2(-0.45680356f,-0.0575239658f),
                            new Vector2(-0.399805039f,-0.0572872236f),
                            new Vector2(-0.399885118f,-0.114285707f),
                            new Vector2(0.399980545f,-0.114285707f),
                            new Vector2(0.39989236f,-0.0570352152f),
                            new Vector2(0.45714283f,-0.0564336777f),
                            new Vector2(0.45714286f,0.114285715f)
							               
                        });				
						bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(WeaponSize * (flipped ? -1f : 1f), WeaponSize);
					//	
                	}
				}
			);
    /*//Minecart with TNT
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Bowling Pin"),
					NameOverride = "Minecart with TNT",
					NameToOrderByOverride = "D",
					DescriptionOverride = "Ride the rails on this.",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Minecart.png"),
					AfterSpawn = (Instance) =>
					{
						PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Metal");
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Bubbles";

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Vehicles/Collisions/Minecart.png", 9f);
                            foreach (var ColliderBoxDeleteVar in Instance.GetComponents<Collider2D>())
                            {
                                GameObject.Destroy(ColliderBoxDeleteVar);
                            }
                            Instance.FixColliders();
                            Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Vehicles/Minecart.png", 0.9f);

						Instance.AddComponent<NoChildrenCollide>();
						bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(3.1f * (flipped ? -1f : 1f), 3.1f);
					//Wheel
						GameObject I = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						I.AddComponent<AudioSourceTimeScaleBehaviour>();
                        I.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");

						SpriteRenderer IRenderer = I.GetComponent<SpriteRenderer>();
						IRenderer.sprite = ModAPI.LoadSprite("Vehicles/Minecart Wheel.png", 1f);
						IRenderer.sortingOrder = 2;
						IRenderer.sortingLayerName = "Bubbles";
						foreach (var c in I.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						I.AddComponent<CircleCollider2D>();
						I.transform.parent = Instance.transform;
						I.transform.rotation = Instance.transform.rotation;
						I.transform.localPosition = new Vector3(0.145f, -0.11f);
						I.transform.localScale = new Vector3(0.8f, 0.8f);

						HingeJoint2D hingeJoint = I.GetOrAddComponent<HingeJoint2D>();
						hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						hingeJoint.anchor = new Vector3(0f, 0f);

						JointAngleLimits2D limits = hingeJoint.limits;
						limits.min = 30;
						limits.max = -30;
						hingeJoint.limits = limits;
						hingeJoint.useLimits = false;
										
				    //Leg
						GameObject Plankk = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						Plankk.AddComponent<AudioSourceTimeScaleBehaviour>();
                        Plankk.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
						
						SpriteRenderer PlankkRenderer = Plankk.GetComponent<SpriteRenderer>();
						PlankkRenderer.sprite = ModAPI.LoadSprite("Vehicles/Minecart Wheel.png", 1f);
						PlankkRenderer.sortingOrder = 2;
						PlankkRenderer.sortingLayerName = "Bubbles";
						foreach (var c in Plankk.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Plankk.AddComponent<CircleCollider2D>();
						Plankk.transform.parent = Instance.transform;
						Plankk.transform.rotation = Instance.transform.rotation;
						Plankk.transform.localPosition = new Vector3(-0.145f, -0.11f);
						Plankk.transform.localScale = new Vector3(0.8f, 0.8f);

						HingeJoint2D PlankkhingeJoint = Plankk.GetOrAddComponent<HingeJoint2D>();
						PlankkhingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						PlankkhingeJoint.anchor = new Vector3(0f, 0f);
						Debug.Log("Broken");
							

						JointAngleLimits2D Plankklimits = hingeJoint.limits;
						Plankklimits.min = 90;
						Plankklimits.max = -30;
						PlankkhingeJoint.limits = limits;
						PlankkhingeJoint.useLimits = false;

                        //New Objects

                                GameObject TNTss = GameObject.Instantiate(ModAPI.FindSpawnable("Dynamite").Prefab);   
							    SpawnObjectBehavior sobBed = TNTss.AddComponent<SpawnObjectBehavior>();                           
							    sobBed.sprite = ModAPI.LoadSprite("Blocks&Stuff/tnt.png", 1f);
							    sobBed.ObjectPosition = new Vector3(0f, 0.3f, 0f);
							    sobBed.PhysicalProperties = "Wood";
							    sobBed.SetParent = true;
							    sobBed.CreateJoint = true;
							    sobBed.CreateObject(Instance);
								
                	}
				}
			);
            */
    //Lantern
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Lantern",
                    NameToOrderByOverride = "D",
                    DescriptionOverride = "Light",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/38.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/lantern.png", 0.7f);
                        Instance.FixColliders(); 
                        var light = ModAPI.CreateLight(Instance.transform, new Color(0.89803f, 0.44705f, 0.15f, 1f), 6.5f, 5f);

          
                    }
                }
            );
    //Soul Lantern
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Soul Lantern",
                    NameToOrderByOverride = "D",
                    DescriptionOverride = "Blue Light",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/40.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/bluelantern.png", 0.7f);
                        Instance.FixColliders(); 
                        var light = ModAPI.CreateLight(Instance.transform, new Color(0.1f, 0.3f, 0.5f, 1f), 6.5f, 5f);

          
                    }
                }
            );
    //Torch
            ModAPI.Register(
                            new Modification()
                            {
                                OriginalItem = ModAPI.FindSpawnable("Torch"),
                                NameOverride = "Minecraft Torch",
                                NameToOrderByOverride = "D",
                                DescriptionOverride = "Orange Light",
                                CategoryOverride = ModAPI.FindCategory("Minecraft"),
                                ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/31.png", 5f),
                                AfterSpawn = (Instance) =>
                                {
                                    Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/Torch.png", 0.8f);
                                    Instance.FixColliders();
                                }
                            }
                        );  
    //Redstone Torch
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Redstone Torch",
                    NameToOrderByOverride = "D",
                    DescriptionOverride = "Red Light",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/42.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/redtorch.png", 0.7f);
                        Instance.FixColliders(); 
                        var light = ModAPI.CreateLight(Instance.transform, new Color(1f, 0f, 0.03921f, 1f), 1.5f, 1f);        
                    }
                }
            );
    //Candle
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Candle",
                    NameToOrderByOverride = "D",
                    DescriptionOverride = "Red Light",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Candle.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        var Candle = Instance.AddComponent<CandleBehaviour>();   
                        Candle.LightRadius = 8f;
                        Candle.LightBrightness = 0.4f;
                        Candle.Pos = new Vector3(0f, 0.5f, 0f);
                        Candle.Color = Color.yellow;
                        Candle.OffSprite = ModAPI.LoadSprite("Blocks&Stuff/CandleOff.png", 0.4f);
                        Candle.OnSprite = ModAPI.LoadSprite("Blocks&Stuff/Candle.png", 0.4f);
                        Candle.OffClip = ModAPI.LoadSound("Sfx/Extinguish.mp3");
                        Candle.OnClip = ModAPI.LoadSound("Sfx/Ignite.mp3"); 
                        
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/CandleOff.png", 0.4f);
						foreach (var c in Instance.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Instance.AddComponent<PolygonCollider2D>();
						Instance.AddComponent<NoChildrenCollide>();
						bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(1f * (flipped ? -1f : 1f), 1f);
					//Head
						GameObject I = GameObject.Instantiate(ModAPI.FindSpawnable("Bowling Pin").Prefab);
						I.AddComponent<AudioSourceTimeScaleBehaviour>();

						SpriteRenderer IRenderer = I.GetComponent<SpriteRenderer>();
						IRenderer.sprite = ModAPI.LoadSprite("Blocks&Stuff/CandleFuse.png", 0.4f);
						IRenderer.sortingOrder = 2;
						IRenderer.sortingLayerName = "Middle";
						foreach (var c in I.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						I.AddComponent<CircleCollider2D>();
						I.transform.parent = Instance.transform;
						I.transform.rotation = Instance.transform.rotation;
						I.transform.localPosition = new Vector3(0f, 0.25f);
						I.transform.localScale = new Vector3(1f, 1f);

						HingeJoint2D hingeJoint = I.GetOrAddComponent<HingeJoint2D>();
						hingeJoint.connectedBody = Instance.GetComponent<Rigidbody2D>();
						hingeJoint.anchor = new Vector3(0f, 0f);

						JointAngleLimits2D limits = hingeJoint.limits;
						limits.min = 1;
						limits.max = -1;
						hingeJoint.limits = limits;
						hingeJoint.useLimits = true;                          
                    }
                }
            );         
    //Lever
            ModAPI.Register(
                            new Modification()
                            {
                                OriginalItem = ModAPI.FindSpawnable("Stick"),
                                NameOverride = "Lever",
                                NameToOrderByOverride = "D",
                                DescriptionOverride = "Press F",
                                CategoryOverride = ModAPI.FindCategory("Minecraft"),
                                ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/28.png", 5f),
                                AfterSpawn = (Instance) =>
                                {
                                    Sprite CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/LeverLeft.png", 30f);
                                    Sprite OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/LeverRight.png", 30f);
                                    var GetSpriteR = Instance.GetComponent<SpriteRenderer>();
                                    GetSpriteR.sprite = CloseSprite;
                                    Instance.FixColliders();

                                    var AS = Instance.AddComponent<AudioSource>();
                                    AudioClip OpenSound = ModAPI.LoadSound("Sfx/lever.wav");
                                    AudioClip CloseSound = ModAPI.LoadSound("Sfx/lever.wav");
                                    bool Close = true;
                                    Instance.AddComponent<UseEventTrigger>().Action = () =>
                                    {
                                        if (Close)
                                        {
                                            GetSpriteR.sprite = OpenSprite;
                                            Instance.FixColliders();
                                            AS.PlayOneShot(OpenSound);
                                            Close = !Close;
                                        }
                                        else
                                        {
                                            GetSpriteR.sprite = CloseSprite;
                                            Instance.FixColliders();  
                                            AS.PlayOneShot(CloseSound);
                                            Close = !Close;
                                        }
                                    };
                                }
                            }
                        );
    //Banner‌
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Plank"),
					NameOverride = "Banner‌",
					NameToOrderByOverride = "D",
					DescriptionOverride = "Put up a flag in honor of your village!",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Snowy Banner‌.png"),
					AfterSpawn = (Instance) =>
					{
						PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
						phys.Properties = ModAPI.FindPhysicalProperties("Wood");
						phys.TrueInitialMass = 0.05f;
						phys.InitialMass = 0.05f;
						phys.rigidbody.mass = 0.05f;
                        phys.DisplayBloodDecals = true;
                        Instance.GetComponent<SpriteRenderer>().sortingOrder = 10;
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Bottom"; 

						CustomCollision customCol0 = Instance.AddComponent<CustomCollision>();
						customCol0.CollisionSprite = ModAPI.LoadSprite("Blocks&Stuff/Banners&Shields//Collision/Stand.png", 1f);
						customCol0.Sprite = ModAPI.LoadSprite("Blocks&Stuff/Banners&Shields//Banners/Stand.png", 1f);

						Instance.AddComponent<NoChildrenCollide>();
						bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(1.5f * (flipped ? -1f : 1f), 1.5f);
						EntityCreator creator = Instance.AddComponent<EntityCreator>();
					//Banner
						creator.OriginalItemName = "Plank";
						creator.PhysicalPropertiesName = "Wood";
                        creator.CollisionSprite = ModAPI.LoadSprite("Blocks&Stuff/Banners&Shields/Collision/Snowy Banner‌.png", 1f);
                        creator.RandomSprites = new Sprite[]
                            {
                                ModAPI.LoadSprite("Blocks&Stuff/Banners&Shields/Banners/Snowy Banner‌.png", 1f),
                                ModAPI.LoadSprite("Blocks&Stuff/Banners&Shields/Banners/Ominous Banner‌.png", 1f),
                                ModAPI.LoadSprite("Blocks&Stuff/Banners&Shields/Banners/Magenta Banner.png", 1f),
                                ModAPI.LoadSprite("Blocks&Stuff/Banners&Shields/Banners/White Banner‌.png", 1f),
                                ModAPI.LoadSprite("Blocks&Stuff/Banners&Shields/Banners/Black Banner‌.png", 1f),
                            };
						creator.sortingLayerName = "Background";
						creator.LimbMass = 0.05f;
						creator.localPosition = new Vector3(0f, 0.059f);
                        creator.localAngle = new Vector3(0f, 0f, 0f);
                        creator.DisplayBloodDecals = true;
                        creator.UseLimits = false;
						creator.hingeJointBreakForce = 2000f;
						creator.hingeJointAnchor = new Vector3(0f, 0.4572f);
						creator.limitsMin = -80;
						creator.limitsMax = 80;
						creator.CreateNewEntity();
                    //Stand 2
						creator.OriginalItemName = "Plank";
						creator.PhysicalPropertiesName = "Wood";
                        creator.RandomSprites = new Sprite[]
                            {
                                ModAPI.LoadSprite("Blocks&Stuff/Banners&Shields//Collision/Stand2.png", 1f),
                            };
						creator.CollisionSprite = ModAPI.LoadSprite("Blocks&Stuff/Banners&Shields//Banners/Stand2.png", 1f);
						creator.sortingOrder = 11;
						creator.sortingLayerName = "Bottom";
						creator.LimbMass = 0.05f;
						creator.localPosition = new Vector3(-0.1f, 0.5143f);
                        creator.localAngle = new Vector3(0f, 0f, 0f);
                        creator.DisplayBloodDecals = true;
                        creator.UseLimits = true;
						creator.hingeJointBreakForce = 50000f;
						creator.hingeJointAnchor = new Vector3(-0.136f, 0.007f);
						creator.limitsMin = -1;
						creator.limitsMax = 1;
						creator.CreateNewEntity();
						creator.entity.AddComponent<OnHingeJointBreakSetParent>().joint = creator.entity.GetComponent<HingeJoint2D>();
					}
				}
			);
     //Wither Skeleton Head
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Rod"),
					NameOverride = "Wither Skeleton Head",
					NameToOrderByOverride = "D",
					DescriptionOverride = "D",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Wither Skeleton Head.png"),
					AfterSpawn = (Instance) =>
					{
						Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/Wither Skeleton Head.png", 1f);
						Instance.FixColliders();

						var phys = Instance.GetComponent<PhysicalBehaviour>();
						phys.TrueInitialMass = 0.001f;
						phys.InitialMass = 0.001f;
						phys.rigidbody.mass = 0.001f;

						ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();
						armor.stabResistance = 5f;
						armor.armorPiece = "Head";
						armor.armorTier = 0;
					}
				}
			);
    //Skeleton Head
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Rod"),
					NameOverride = "Skeleton Head",
					NameToOrderByOverride = "D",
					DescriptionOverride = " ",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Skeleton Head.png"),
					AfterSpawn = (Instance) =>
					{
						Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/Skeleton Head.png", 1f);
						Instance.FixColliders();

						var phys = Instance.GetComponent<PhysicalBehaviour>();
						phys.TrueInitialMass = 0.001f;
						phys.InitialMass = 0.001f;
						phys.rigidbody.mass = 0.001f;

						ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();
						armor.stabResistance = 5f;
						armor.armorPiece = "Head";
						armor.armorTier = 0;
					}
				}
			);
    //Wither Skeleton Head
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Rod"),
					NameOverride = "Wither Skeleton Head",
					NameToOrderByOverride = "D",
					DescriptionOverride = " ",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Wither Skeleton Head.png"),
					AfterSpawn = (Instance) =>
					{
						Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/Wither Skeleton Head.png", 1f);
						Instance.FixColliders();

						var phys = Instance.GetComponent<PhysicalBehaviour>();
						phys.TrueInitialMass = 0.001f;
						phys.InitialMass = 0.001f;
						phys.rigidbody.mass = 0.001f;

						ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();
						armor.stabResistance = 5f;
						armor.armorPiece = "Head";
						armor.armorTier = 0;
					}
				}
			);
    //Zombie Head
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Rod"),
					NameOverride = "Zombie Head",
					NameToOrderByOverride = "D",
					DescriptionOverride = " ",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Zombie Head.png"),
					AfterSpawn = (Instance) =>
					{
						Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/Zombie Head.png", 1f);
						Instance.FixColliders();

						var phys = Instance.GetComponent<PhysicalBehaviour>();
						phys.TrueInitialMass = 0.001f;
						phys.InitialMass = 0.001f;
						phys.rigidbody.mass = 0.001f;

						ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();
						armor.stabResistance = 5f;
						armor.armorPiece = "Head";
						armor.armorTier = 0;
					}
				}
			);
    //Steve Head
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Rod"),
					NameOverride = "Steve Head",
					NameToOrderByOverride = "D",
					DescriptionOverride = " ",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Steve Head.png"),
					AfterSpawn = (Instance) =>
					{
						Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/Steve Head.png", 1f);
						Instance.FixColliders();

						var phys = Instance.GetComponent<PhysicalBehaviour>();
						phys.TrueInitialMass = 0.001f;
						phys.InitialMass = 0.001f;
						phys.rigidbody.mass = 0.001f;

						ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();
						armor.stabResistance = 5f;
						armor.armorPiece = "Head";
						armor.armorTier = 0;
					}
				}
			);
    //Creeper Head
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Rod"),
					NameOverride = "Creeper Head",
					NameToOrderByOverride = "D",
					DescriptionOverride = " ",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Creeper Head.png"),
					AfterSpawn = (Instance) =>
					{
						Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/Creeper Head.png", 1f);
						Instance.FixColliders();

						var phys = Instance.GetComponent<PhysicalBehaviour>();
						phys.TrueInitialMass = 0.001f;
						phys.InitialMass = 0.001f;
						phys.rigidbody.mass = 0.001f;

						ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();
						armor.stabResistance = 5f;
						armor.armorPiece = "Head";
						armor.armorTier = 0;
					}
				}
			);
    //Pumpkin Head
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Stick"), 
                    NameOverride = "Pumpkin Head",
                    NameToOrderByOverride = "D", 
                    DescriptionOverride = "Minecraft Block",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Pumpkin Head.png", 1f),
                    AfterSpawn = (Instance) =>
                    {
                         Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/Pumpkin.png", 1.1f);
						Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<BoxCollider2D>();
                        Instance.FixColliders(); 
                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();
						armor.stabResistance = 0f;
						armor.armorPiece = "Head";
						armor.armorTier = 0;  
                    }
                }
            );
    //Carved Pumpkin Head
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Stick"), 
                    NameOverride = "Carved Pumpkin Head",
                    NameToOrderByOverride = "D", 
                    DescriptionOverride = "Minecraft Block",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Carved Pumpkin Head.png", 1f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/Carved Pumpkin.png", 1.1f);
						Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<BoxCollider2D>();
                        Instance.FixColliders(); 
                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();
						armor.stabResistance = 0f;
						armor.armorPiece = "Head";
						armor.armorTier = 0;  
                    }
                }
            );
    //Jack o Lantern Head
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Stick"), 
                    NameOverride = "Jack o Lantern Head",
                    NameToOrderByOverride = "D", 
                    DescriptionOverride = "Minecraft Block",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Jack o Lantern Head.png", 1f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/Jack o Lantern.png", 1.1f);
						Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<BoxCollider2D>();
                        Instance.FixColliders(); 
                        var light = ModAPI.CreateLight(Instance.transform, new Color(0.89803f, 0.44705f, 0.15f, 1f), 6.5f, 5f);
                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();
						armor.stabResistance = 0f;
						armor.armorPiece = "Head";
						armor.armorTier = 0;  
                    }
                }
            );
    //Melon Head
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Stick"), 
                    NameOverride = "Melon Head",
                    NameToOrderByOverride = "D", 
                    DescriptionOverride = "Minecraft Block",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Melon Head.png", 1f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/Melon.png", 1.1f);
						Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Human");
						foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<BoxCollider2D>();
                        Instance.FixColliders();  
                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();
						armor.stabResistance = 0f;
						armor.armorPiece = "Head";
						armor.armorTier = 0; 
                    }
                }
            );
#endregion
#region Blocks
    //Anvil
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Plank"),
					NameOverride = "Anvil",
					NameToOrderByOverride = "E",
					DescriptionOverride = "It's a very heavy thing. The main thing is not to throw it on your friend's head.",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Anvil.png"),
					AfterSpawn = (Instance) =>
					{
                    //Main	
						Sprite CustomSprite = ModAPI.LoadSprite("Blocks&Stuff/Anvil.png");

						Sprite OutlineSprite = ModAPI.LoadSprite("Blocks&Stuff/Outline Sprites/Anvil.png"); 
						float WeaponSize = 2.4f;

                        var physOther = UnityEngine.Object.Instantiate<PhysicalProperties>(ModAPI.FindPhysicalProperties("Metal"));                    
                        Instance.GetComponent<PhysicalBehaviour>().Properties = physOther;  
                        PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();										
					//Settings
						phys.Properties = ModAPI.FindPhysicalProperties("Metal");
						phys.InitialMass = 15;
                        phys.TrueInitialMass = 15;  
                        phys.OverrideImpactSounds = new AudioClip[] 
                        {
                            ModAPI.LoadSound("Sfx/Anvil Land.mp3"),                                                                                 
                        };              
					//Other
						var OtlineSpriteRenderer = Instance.GetComponent<SpriteRenderer>();
						OtlineSpriteRenderer.sprite = OutlineSprite;
						OtlineSpriteRenderer.sortingLayerName = "Default";
						OtlineSpriteRenderer.sortingOrder = -10;
						
						GameObject NewSprite = new GameObject("CustomSprite");
            			NewSprite.transform.parent = Instance.transform;
            			NewSprite.transform.localPosition = new Vector3(0, 0f);
            			NewSprite.transform.localScale = new Vector3(1f, 1f);
            			var RendererNewSprite = NewSprite.AddComponent<SpriteRenderer>();
                        var RandomNewSprite = NewSprite.AddComponent<RandomSpriteBehaviour>();					
           			 	RandomNewSprite.sprites = new Sprite[]
                            {
                                CustomSprite
                            };
                        RendererNewSprite.transform.localRotation = Quaternion.Euler(0f, 0f, 0f);
						RendererNewSprite.sortingLayerName = "Default";
						RendererNewSprite.sortingOrder = 0;
						foreach (Collider2D Collison in Instance.GetComponents<Collider2D>())
                        {
                            Collider2D.Destroy(Collison);
                        }
						Instance.AddComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.22857143f,0.22857143f),
                            new Vector2(-0.228571415f,0.0566592216f),
                            new Vector2(-0.114254147f,0.0560207367f),
                            new Vector2(-0.11585027f,-0.0851886272f),
                            new Vector2(-0.170417577f,-0.115473032f),
                            new Vector2(-0.172370747f,-0.229848638f),
                            new Vector2(0.173009396f,-0.227932706f),
                            new Vector2(0.170902789f,-0.117455482f),
                            new Vector2(0.11697416f,-0.0839004517f),
                            new Vector2(0.119363278f,0.057297945f),
                            new Vector2(0.22857143f,0.0579366684f),
                            new Vector2(0.22857143f,0.22857143f)						               
                        });				
						bool flipped = Instance.transform.localScale.x < 0f;
						Instance.transform.localScale = new Vector2(WeaponSize * (flipped ? -1f : 1f), WeaponSize);
					//	
                	}
				}
			);
    //Bed
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Stick"),
                    NameOverride = "Bed",
                    NameToOrderByOverride = "E",
                    DescriptionOverride = "Red Bed",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/41.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/bed.png", 0.4f);
                        Instance.FixColliders(); 
       
                    }
                }
            );
    //Bee Nest
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Crossbow"),
                    NameOverride = "Bee Nest",
                    NameToOrderByOverride = "E",
                    DescriptionOverride = "Bee nest with bees.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Bee Nest.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Sprite ThisSprite = ModAPI.LoadSprite("Blocks&Stuff/Bee Nest.png", 0.4f);
                        
                        var SpriteTexture = Instance.GetComponent<SpriteRenderer>();
                        var Projectile = Instance.GetComponent<ProjectileLauncherBehaviour>();
						var phys = Instance.GetComponent<PhysicalBehaviour>();

                        SpriteTexture.sprite = ThisSprite;

						Projectile.projectileAsset = ModAPI.FindSpawnable("Bee");
						Projectile.launchSound = ModAPI.LoadSound("Sfx/sho.wav");
						Projectile.barrelPosition = new Vector2(0.5f, 0f);
						Projectile.IsAutomatic = false;						
						Projectile.ScreenShake = 0f;
						Projectile.projectileLaunchStrength = 0f;
						phys.Properties = ModAPI.FindPhysicalProperties("Weapon");
						phys.TrueInitialMass = 0.10f;
						phys.InitialMass = 0.10f;
						phys.rigidbody.mass = 0.10f;
                        foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<PolygonCollider2D>();     
                    }
                }
            );
    //TNT
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Dynamite"),
                    NameOverride = "TNT",
                    NameToOrderByOverride = "E",
                    DescriptionOverride = "BOOM",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/30.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/tnt.png", 0.4f);
                        foreach (var c in Instance.GetComponents<Collider2D>())
                              { 
                                 GameObject.Destroy(c);
                              }
                          Instance.AddComponent<PolygonCollider2D>();   
                        Instance.FixColliders();                
                    }
                }
            );
    //Dispenser
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Crossbow"),
                    NameOverride = "Dispenser",
                    NameToOrderByOverride = "E",
                    DescriptionOverride = "Block that launches arrows",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/29.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        var SpriteTexture = Instance.GetComponent<SpriteRenderer>();
                        var Projectile = Instance.GetComponent<ProjectileLauncherBehaviour>();
						var phys = Instance.GetComponent<PhysicalBehaviour>();

                        SpriteTexture.sprite = ModAPI.LoadSprite("Blocks&Stuff/disp.png", 0.4f);

						Projectile.projectileAsset = ModAPI.FindSpawnable("Arrow");
						Projectile.launchSound = ModAPI.LoadSound("Sfx/bow2.wav");
						Projectile.barrelPosition = new Vector2(0.1329999f, -0.07825994f);
						Projectile.IsAutomatic = false;						
						Projectile.ScreenShake = 0.3f;
						Projectile.projectileLaunchStrength = 1.7f;
						phys.Properties = ModAPI.FindPhysicalProperties("Weapon");
						phys.TrueInitialMass = 0.10f;
						phys.InitialMass = 0.10f;
						phys.rigidbody.mass = 0.10f;
                        foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<PolygonCollider2D>();
 
                    }
                }
            );
    //Cake
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Cake",
                    NameToOrderByOverride = "E",
                    DescriptionOverride = " ",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Cake.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/Cake/Cake.png", 0.4f);
						foreach (var c in Instance.GetComponents<Collider2D>())
						{
							GameObject.Destroy(c);
						}
						Instance.AddComponent<BoxCollider2D>();
                        var phys = Instance.GetComponent<PhysicalBehaviour>(); 
                        phys.Properties = ModAPI.FindPhysicalProperties("Human");
						phys.TrueInitialMass = 0.10f;
						phys.InitialMass = 0.10f;
						phys.rigidbody.mass = 0.10f;

                        var Stage = Instance.AddComponent<StageBehaviour>();
                        Stage.StartStage = ModAPI.LoadSprite("Blocks&Stuff/Cake/Cake.png", 0.4f);
                        Stage.Stage1 = ModAPI.LoadSprite("Blocks&Stuff/Cake/Cake2.png", 0.4f);
                        Stage.Stage2 = ModAPI.LoadSprite("Blocks&Stuff/Cake/Cake3.png", 0.4f);
                        Stage.Stage3 = ModAPI.LoadSprite("Blocks&Stuff/Cake/Cake4.png", 0.4f);
                        Stage.Stage4 = ModAPI.LoadSprite("Blocks&Stuff/Cake/Cake5.png", 0.4f);
                        Stage.Switching = ModAPI.LoadSound("Sfx/Eat.mp3");

                        /*var Candle = Instance.AddComponent<CandleBehaviour>();   
                        Candle.LightRadius = 0f;
                        Candle.LightBrightness = 0.4f;
                        Candle.Pos = new Vector3(0f, 0.5f, 0f);
                        Candle.Color = Color.yellow;
                        Candle.OffSprite = ModAPI.LoadSprite("Blocks&Stuff/CakeEat.png", 0.4f);
                        Candle.OnSprite = ModAPI.LoadSprite("Blocks&Stuff/CakeEat.png", 0.4f);
                        Candle.OffClip = ModAPI.LoadSound("Sfx/Eat.mp3");
                        Candle.OnClip = ModAPI.LoadSound("Sfx/Eat.mp3");  
                        */                                         
                    }
                }
            ); 
    //Chest
            ModAPI.Register(
                            new Modification()
                            {
                                OriginalItem = ModAPI.FindSpawnable("Plank"),
                                NameOverride = "Chest",
                                NameToOrderByOverride = "E",
                                DescriptionOverride = "Press F to open",
                                CategoryOverride = ModAPI.FindCategory("Minecraft"),
                                ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Chest.png"),
                                AfterSpawn = (Instance) =>
                                {
                                    Sprite CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Chest.png", 0.4f);
                                    Sprite OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/ChestOp.png", 0.4f);

                                    Sprite CloseSpriteCol = ModAPI.LoadSprite("Blocks&Stuff/Collision/Chest.png", 4.0f);
                                    Sprite OpenSpriteCol = ModAPI.LoadSprite("Blocks&Stuff/Collision/ChestOp.png", 4.0f);

                                    var GetSpriteR = Instance.GetComponent<SpriteRenderer>();
                                    GetSpriteR.sortingLayerName = "Foreground";
                                    GetSpriteR.sprite = CloseSprite;
                                    Instance.FixColliders();

                                    var AS = Instance.AddComponent<AudioSource>();
                                    AudioClip OpenSound = ModAPI.LoadSound("Sfx/chest1.wav");
                                    AudioClip CloseSound = ModAPI.LoadSound("Sfx/chest2.wav");
                                    bool Close = true;
                                    Instance.AddComponent<UseEventTrigger>().Action = () =>
                                    {
                                        if (Close)
                                        {
                                            
                                            GetSpriteR.sprite = OpenSpriteCol;                                         
                                            foreach (var ColliderBoxDeleteVar in Instance.GetComponents<Collider2D>())
                                                {
                                                    GameObject.Destroy(ColliderBoxDeleteVar);
                                                }
                                            Instance.FixColliders();
                                            GetSpriteR.sprite = OpenSprite;
                                            GetSpriteR.sortingLayerName = "Bottom";
                                            AS.PlayOneShot(OpenSound);
                                            Close = !Close;
                                        }
                                        else
                                        {                                           
                                            GetSpriteR.sprite = CloseSpriteCol;
                                            foreach (var ColliderBoxDeleteVar in Instance.GetComponents<Collider2D>())
                                                {
                                                    GameObject.Destroy(ColliderBoxDeleteVar);
                                                }
                                            Instance.FixColliders();
                                            GetSpriteR.sprite = CloseSprite;
                                            GetSpriteR.sortingLayerName = "Foreground";
                                            AS.PlayOneShot(CloseSound);
                                            Close = !Close;
                                        }
                                    }; 
                                }                        
                            }
                        );
    
    //Trapped Chest
            ModAPI.Register(
                            new Modification()
                            {
                                OriginalItem = ModAPI.FindSpawnable("Plank"),
                                NameOverride = "Trapped Chest",
                                NameToOrderByOverride = "E",
                                DescriptionOverride = "Press F to open",
                                CategoryOverride = ModAPI.FindCategory("Minecraft"),
                                ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Trapped Chest.png"),
                                AfterSpawn = (Instance) =>
                                {
                                   Sprite CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Trapped Chest.png", 0.4f);
                                    Sprite OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Trapped ChestOp.png", 0.4f);

                                    Sprite CloseSpriteCol = ModAPI.LoadSprite("Blocks&Stuff/Collision/Chest.png", 4.0f);
                                    Sprite OpenSpriteCol = ModAPI.LoadSprite("Blocks&Stuff/Collision/ChestOp.png", 4.0f);
                                    var GetSpriteR = Instance.GetComponent<SpriteRenderer>();
                                    GetSpriteR.sortingLayerName = "Foreground";
                                    GetSpriteR.sprite = CloseSprite;
                                    Instance.FixColliders();

                                    var AS = Instance.AddComponent<AudioSource>();
                                    AudioClip OpenSound = ModAPI.LoadSound("Sfx/chest1.wav");
                                    AudioClip CloseSound = ModAPI.LoadSound("Sfx/chest2.wav");
                                    bool Close = true;
                                    Instance.AddComponent<UseEventTrigger>().Action = () =>
                                    {
                                        if (Close)
                                        {
                                            
                                            GetSpriteR.sprite = OpenSpriteCol;                                         
                                            foreach (var ColliderBoxDeleteVar in Instance.GetComponents<Collider2D>())
                                                {
                                                    GameObject.Destroy(ColliderBoxDeleteVar);
                                                }
                                            Instance.FixColliders();
                                            GetSpriteR.sprite = OpenSprite;
                                            GetSpriteR.sortingLayerName = "Bottom";
                                            AS.PlayOneShot(OpenSound);
                                            Close = !Close;
                                        }
                                        else
                                        {
                                            
                                            GetSpriteR.sprite = CloseSpriteCol;
                                            foreach (var ColliderBoxDeleteVar in Instance.GetComponents<Collider2D>())
                                                {
                                                    GameObject.Destroy(ColliderBoxDeleteVar);
                                                }
                                            Instance.FixColliders();
                                            GetSpriteR.sprite = CloseSprite;
                                            GetSpriteR.sortingLayerName = "Foreground";
                                            AS.PlayOneShot(CloseSound);
                                            Close = !Close;
                                        }
                                    }; 
                                }                        
                            }
                        );
    //Red Christmas Chest
            ModAPI.Register(
                            new Modification()
                            {
                                OriginalItem = ModAPI.FindSpawnable("Plank"),
                                NameOverride = "Red Christmas Chest",
                                NameToOrderByOverride = "E",
                                DescriptionOverride = "Press F to open",
                                CategoryOverride = ModAPI.FindCategory("Minecraft"),
                                ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Red Christmas Chest.png"),
                                AfterSpawn = (Instance) =>
                                {
                                    Sprite CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Red Christmas Chest.png", 0.4f);
                                    Sprite OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Red Christmas ChestOp.png", 0.4f);

                                    Sprite CloseSpriteCol = ModAPI.LoadSprite("Blocks&Stuff/Collision/Chest.png", 4.0f);
                                    Sprite OpenSpriteCol = ModAPI.LoadSprite("Blocks&Stuff/Collision/ChestOp.png", 4.0f);
                                    var GetSpriteR = Instance.GetComponent<SpriteRenderer>();
                                    GetSpriteR.sortingLayerName = "Foreground";
                                    GetSpriteR.sprite = CloseSprite;
                                    Instance.FixColliders();

                                    var AS = Instance.AddComponent<AudioSource>();
                                    AudioClip OpenSound = ModAPI.LoadSound("Sfx/chest1.wav");
                                    AudioClip CloseSound = ModAPI.LoadSound("Sfx/chest2.wav");
                                    bool Close = true;
                                    Instance.AddComponent<UseEventTrigger>().Action = () =>
                                    {
                                        if (Close)
                                        {
                                            
                                            GetSpriteR.sprite = OpenSpriteCol;                                         
                                            foreach (var ColliderBoxDeleteVar in Instance.GetComponents<Collider2D>())
                                                {
                                                    GameObject.Destroy(ColliderBoxDeleteVar);
                                                }
                                            Instance.FixColliders();
                                            GetSpriteR.sprite = OpenSprite;
                                            GetSpriteR.sortingLayerName = "Bottom";
                                            AS.PlayOneShot(OpenSound);
                                            Close = !Close;
                                        }
                                        else
                                        {
                                            
                                            GetSpriteR.sprite = CloseSpriteCol;
                                            foreach (var ColliderBoxDeleteVar in Instance.GetComponents<Collider2D>())
                                                {
                                                    GameObject.Destroy(ColliderBoxDeleteVar);
                                                }
                                            Instance.FixColliders();
                                            GetSpriteR.sprite = CloseSprite;
                                            GetSpriteR.sortingLayerName = "Foreground";
                                            AS.PlayOneShot(CloseSound);
                                            Close = !Close;
                                        }
                                    }; 
                                }                        
                            }
                        );
    //Green Christmas Chest
            ModAPI.Register(
                            new Modification()
                            {
                                OriginalItem = ModAPI.FindSpawnable("Plank"),
                                NameOverride = "Green Christmas Chest",
                                NameToOrderByOverride = "E",
                                DescriptionOverride = "Press F to open",
                                CategoryOverride = ModAPI.FindCategory("Minecraft"),
                                ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Green Christmas Chest.png"),
                                AfterSpawn = (Instance) =>
                                {
                                    Sprite CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Green Christmas Chest.png", 0.4f);
                                    Sprite OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Green Christmas ChestOp.png", 0.4f);

                                    Sprite CloseSpriteCol = ModAPI.LoadSprite("Blocks&Stuff/Collision/Chest.png", 4.0f);
                                    Sprite OpenSpriteCol = ModAPI.LoadSprite("Blocks&Stuff/Collision/ChestOp.png", 4.0f);
                                    var GetSpriteR = Instance.GetComponent<SpriteRenderer>();
                                    GetSpriteR.sortingLayerName = "Foreground";
                                    GetSpriteR.sprite = CloseSprite;
                                    Instance.FixColliders();

                                    var AS = Instance.AddComponent<AudioSource>();
                                    AudioClip OpenSound = ModAPI.LoadSound("Sfx/chest1.wav");
                                    AudioClip CloseSound = ModAPI.LoadSound("Sfx/chest2.wav");
                                    bool Close = true;
                                    Instance.AddComponent<UseEventTrigger>().Action = () =>
                                    {
                                        if (Close)
                                        {
                                            
                                            GetSpriteR.sprite = OpenSpriteCol;                                         
                                            foreach (var ColliderBoxDeleteVar in Instance.GetComponents<Collider2D>())
                                                {
                                                    GameObject.Destroy(ColliderBoxDeleteVar);
                                                }
                                            Instance.FixColliders();
                                            GetSpriteR.sprite = OpenSprite;
                                            GetSpriteR.sortingLayerName = "Bottom";
                                            AS.PlayOneShot(OpenSound);
                                            Close = !Close;
                                        }
                                        else
                                        {
                                            
                                            GetSpriteR.sprite = CloseSpriteCol;
                                            foreach (var ColliderBoxDeleteVar in Instance.GetComponents<Collider2D>())
                                                {
                                                    GameObject.Destroy(ColliderBoxDeleteVar);
                                                }
                                            Instance.FixColliders();
                                            GetSpriteR.sprite = CloseSprite;
                                            GetSpriteR.sortingLayerName = "Foreground";
                                            AS.PlayOneShot(CloseSound);
                                            Close = !Close;
                                        }
                                    }; 
                                }                        
                            }
                        );        
    //Ender Chest
            ModAPI.Register(
                            new Modification()
                            {
                                OriginalItem = ModAPI.FindSpawnable("Plank"),
                                NameOverride = "Ender Chest",
                                NameToOrderByOverride = "E",
                                DescriptionOverride = "Press F to open",
                                CategoryOverride = ModAPI.FindCategory("Minecraft"),
                                ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Ender Chest.png"),
                                AfterSpawn = (Instance) =>
                                {
                                   Sprite CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Ender Chest.png", 0.4f);
                                    Sprite OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Ender ChestOp.png", 0.4f);

                                    Sprite CloseSpriteCol = ModAPI.LoadSprite("Blocks&Stuff/Collision/Chest.png", 4.0f);
                                    Sprite OpenSpriteCol = ModAPI.LoadSprite("Blocks&Stuff/Collision/ChestOp.png", 4.0f);
                                    var GetSpriteR = Instance.GetComponent<SpriteRenderer>();
                                    GetSpriteR.sortingLayerName = "Foreground";
                                    GetSpriteR.sprite = CloseSprite;
                                    Instance.FixColliders();

                                    var AS = Instance.AddComponent<AudioSource>();
                                    AudioClip OpenSound = ModAPI.LoadSound("Sfx/Enderopen.mp3");
                                    AudioClip CloseSound = ModAPI.LoadSound("Sfx/Enderclose.mp3");
                                    bool Close = true;
                                    Instance.AddComponent<UseEventTrigger>().Action = () =>
                                    {
                                        if (Close)
                                        {
                                            
                                            GetSpriteR.sprite = OpenSpriteCol;                                         
                                            foreach (var ColliderBoxDeleteVar in Instance.GetComponents<Collider2D>())
                                                {
                                                    GameObject.Destroy(ColliderBoxDeleteVar);
                                                }
                                            Instance.FixColliders();
                                            GetSpriteR.sprite = OpenSprite;
                                            GetSpriteR.sortingLayerName = "Bottom";
                                            AS.PlayOneShot(OpenSound);
                                            Close = !Close;
                                        }
                                        else
                                        {
                                            
                                            GetSpriteR.sprite = CloseSpriteCol;
                                            foreach (var ColliderBoxDeleteVar in Instance.GetComponents<Collider2D>())
                                                {
                                                    GameObject.Destroy(ColliderBoxDeleteVar);
                                                }
                                            Instance.FixColliders();
                                            GetSpriteR.sprite = CloseSprite;
                                            GetSpriteR.sortingLayerName = "Foreground";
                                            AS.PlayOneShot(CloseSound);
                                            Close = !Close;
                                        }
                                    }; 
                                }                        
                            }
                        );
    //Door
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Plank"),
					NameOverride = "Door  ",
					NameToOrderByOverride = "E",
					DescriptionOverride = "Create any door and open as many as you can. Press F to open.",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Door.png"),
					AfterSpawn = (Instance) =>
					{
                    //Main
                        float Mass = 0.75f;
                        string SortingLayerName = "Objects";
                    //Script
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Doors/Oak1.png", 0.4f);
                        foreach (var c in Instance.GetComponents<Collider2D>())
                        {
                            GameObject.Destroy(c);
                        }
                        var Collision1 = Instance.AddComponent<BoxCollider2D>();
                        Collision1.size = new Vector2(0.2134342f, 2.285714f);
                        Collision1.offset = new Vector2(-0.4637852f, 0f);

                        var Collision2 = Instance.AddComponent<BoxCollider2D>();
                        Collision2.size = new Vector2(0f, 0f);
                        Collision2.offset = new Vector2(0f, 0f);
                        //Instance.FixColliders();
                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Metal");
                        ImprovedDoorBehaviour ImprovedDoorBehaviour = Instance.GetOrAddComponent<ImprovedDoorBehaviour>();
                        ImprovedDoorBehaviour.Options = new ImprovedDoorBehaviour.DoorOptions[]
                        {
                            new ImprovedDoorBehaviour.DoorOptions
                            {
                                CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Doors/Oak1.png", 0.4f),
                                OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Doors/Oak2.png", 0.4f),
                                CloseSound = ModAPI.LoadSound("Sfx/Wooden2.mp3"),
                                OpenSound = ModAPI.LoadSound("Sfx/Wooden.mp3"),
                                PhysicalProperties = "Wood",
                            },
                            new ImprovedDoorBehaviour.DoorOptions
                            {
                                CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Doors/Iron1.png", 0.4f),
                                OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Doors/Iron2.png", 0.4f),
                                CloseSound = ModAPI.LoadSound("Sfx/Iron2.mp3"),
                                OpenSound = ModAPI.LoadSound("Sfx/Iron.mp3"),
                                PhysicalProperties = "Metal",
                            },
                            new ImprovedDoorBehaviour.DoorOptions
                            {
                                CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Doors/Spruce1.png", 0.4f),
                                OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Doors/Spruce2.png", 0.4f),
                                CloseSound = ModAPI.LoadSound("Sfx/Wooden2.mp3"),
                                OpenSound = ModAPI.LoadSound("Sfx/Wooden.mp3"),
                                PhysicalProperties = "Wood",
                            },
                            new ImprovedDoorBehaviour.DoorOptions
                            {
                                CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Doors/Birch1.png", 0.4f),
                                OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Doors/Birch2.png", 0.4f),
                                CloseSound = ModAPI.LoadSound("Sfx/Wooden2.mp3"),
                                OpenSound = ModAPI.LoadSound("Sfx/Wooden.mp3"),
                                PhysicalProperties = "Wood",
                            },
                            new ImprovedDoorBehaviour.DoorOptions
                            {
                                CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Doors/Acacia1.png", 0.4f),
                                OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Doors/Acacia2.png", 0.4f),
                                CloseSound = ModAPI.LoadSound("Sfx/Wooden2.mp3"),
                                OpenSound = ModAPI.LoadSound("Sfx/Wooden.mp3"),
                                PhysicalProperties = "Wood",
                            },
                            new ImprovedDoorBehaviour.DoorOptions
                            {
                                CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Doors/DarkOak1.png", 0.4f),
                                OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Doors/DarkOak2.png", 0.4f),
                                CloseSound = ModAPI.LoadSound("Sfx/Wooden2.mp3"),
                                OpenSound = ModAPI.LoadSound("Sfx/Wooden.mp3"),
                                PhysicalProperties = "Wood",
                            },
                            new ImprovedDoorBehaviour.DoorOptions
                            {
                                CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Doors/Mangrove1.png", 0.4f),
                                OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Doors/Mangrove2.png", 0.4f),
                                CloseSound = ModAPI.LoadSound("Sfx/Wooden2.mp3"),
                                OpenSound = ModAPI.LoadSound("Sfx/Wooden.mp3"),
                                PhysicalProperties = "Wood",
                            },
                            new ImprovedDoorBehaviour.DoorOptions
                            {
                                CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Doors/Crimson1.png", 0.4f),
                                OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Doors/Crimson2.png", 0.4f),
                                CloseSound = ModAPI.LoadSound("Sfx/Wooden2.mp3"),
                                OpenSound = ModAPI.LoadSound("Sfx/Wooden.mp3"),
                                PhysicalProperties = "Wood",
                            },
                            new ImprovedDoorBehaviour.DoorOptions
                            {
                                CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Doors/Warped1.png", 0.4f),
                                OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Doors/Warped2.png", 0.4f),
                                CloseSound = ModAPI.LoadSound("Sfx/Wooden2.mp3"),
                                OpenSound = ModAPI.LoadSound("Sfx/Wooden.mp3"),
                                PhysicalProperties = "Wood",
                            } 
                        };
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = Mass;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = Mass;
                        Instance.GetComponent<PhysicalBehaviour>().rigidbody.mass = Mass;
                    //
					}
				}
			);
    //Trapdoor
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Plank"),
					NameOverride = "Trapdoor  ",
					NameToOrderByOverride = "E",
					DescriptionOverride = "Create any trapdoor and open as many as you can. Press F to open.",
					CategoryOverride = ModAPI.FindCategory("Minecraft"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Trapdoor.png"),
					AfterSpawn = (Instance) =>
					{
                    //Main
                        float Mass = 0.75f;
                        string SortingLayerName = "Objects";
                    //Script
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Trapdoors/FirstPosition/OakClosed.png", 0.4f);
                        foreach (var c in Instance.GetComponents<Collider2D>())
                        {
                            GameObject.Destroy(c);
                        }
                        var Collision1 = Instance.AddComponent<BoxCollider2D>();
                        Collision1.offset = new Vector2(-0.001027107f, -0.4644399f);
                        Collision1.size = new Vector2(1.142517f, 0.2154379f);

                        var Collision2 = Instance.AddComponent<BoxCollider2D>();
                        Collision2.offset = new Vector2(0f, 0f);
                        Collision2.size = new Vector2(0f, 0f);                        
                        //Instance.FixColliders();
                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Metal");
                        ImprovedDoorBehaviour ImprovedDoorBehaviour = Instance.GetOrAddComponent<ImprovedDoorBehaviour>();
                        ImprovedDoorBehaviour.Options = new ImprovedDoorBehaviour.DoorOptions[]
                        {
                            new ImprovedDoorBehaviour.DoorOptions
                            {
                                CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Trapdoors/FirstPosition/OakClosed.png", 0.4f),
                                OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Trapdoors/FirstPosition/OakOpen.png", 0.4f),
                                CloseSound = ModAPI.LoadSound("Sfx/WoodenTrapdoorClose.mp3"),
                                OpenSound = ModAPI.LoadSound("Sfx/WoodenTrapdoorOpen.mp3"),
                                PhysicalProperties = "Wood",
                            },
                            new ImprovedDoorBehaviour.DoorOptions
                            {
                                CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Trapdoors/FirstPosition/IronClosed.png", 0.4f),
                                OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Trapdoors/FirstPosition/IronOpen.png", 0.4f),
                                CloseSound = ModAPI.LoadSound("Sfx/IronTrapdoorClose.mp3"),
                                OpenSound = ModAPI.LoadSound("Sfx/IronTrapdoorOpen.mp3"),
                                PhysicalProperties = "Metal",
                            },
                            new ImprovedDoorBehaviour.DoorOptions
                            {
                                CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Trapdoors/FirstPosition/SpruceClosed.png", 0.4f),
                                OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Trapdoors/FirstPosition/SpruceOpen.png", 0.4f),
                                CloseSound = ModAPI.LoadSound("Sfx/WoodenTrapdoorClose.mp3"),
                                OpenSound = ModAPI.LoadSound("Sfx/WoodenTrapdoorOpen.mp3"),
                                PhysicalProperties = "Wood",
                            },
                            new ImprovedDoorBehaviour.DoorOptions
                            {
                                CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Trapdoors/FirstPosition/BirchClosed.png", 0.4f),
                                OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Trapdoors/FirstPosition/BirchOpen.png", 0.4f),
                                CloseSound = ModAPI.LoadSound("Sfx/WoodenTrapdoorClose.mp3"),
                                OpenSound = ModAPI.LoadSound("Sfx/WoodenTrapdoorOpen.mp3"),
                                PhysicalProperties = "Wood",
                            },
                            new ImprovedDoorBehaviour.DoorOptions
                            {
                                CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Trapdoors/FirstPosition/JungleClosed.png", 0.4f),
                                OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Trapdoors/FirstPosition/JungleOpen.png", 0.4f),
                                CloseSound = ModAPI.LoadSound("Sfx/WoodenTrapdoorClose.mp3"),
                                OpenSound = ModAPI.LoadSound("Sfx/WoodenTrapdoorOpen.mp3"),
                                PhysicalProperties = "Wood",
                            },
                            new ImprovedDoorBehaviour.DoorOptions
                            {
                                CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Trapdoors/FirstPosition/AcaciaClosed.png", 0.4f),
                                OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Trapdoors/FirstPosition/AcaciaOpen.png", 0.4f),
                                CloseSound = ModAPI.LoadSound("Sfx/WoodenTrapdoorClose.mp3"),
                                OpenSound = ModAPI.LoadSound("Sfx/WoodenTrapdoorOpen.mp3"),
                                PhysicalProperties = "Wood",
                            },
                            new ImprovedDoorBehaviour.DoorOptions
                            {
                                CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Trapdoors/FirstPosition/DarkOakClosed.png", 0.4f),
                                OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Trapdoors/FirstPosition/DarkOakOpen.png", 0.4f),
                                CloseSound = ModAPI.LoadSound("Sfx/WoodenTrapdoorClose.mp3"),
                                OpenSound = ModAPI.LoadSound("Sfx/WoodenTrapdoorOpen.mp3"),
                                PhysicalProperties = "Wood",
                            },
                            new ImprovedDoorBehaviour.DoorOptions
                            {
                                CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Trapdoors/FirstPosition/MangroveClosed.png", 0.4f),
                                OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Trapdoors/FirstPosition/MangroveOpen.png", 0.4f),
                                CloseSound = ModAPI.LoadSound("Sfx/WoodenTrapdoorClose.mp3"),
                                OpenSound = ModAPI.LoadSound("Sfx/WoodenTrapdoorOpen.mp3"),
                                PhysicalProperties = "Wood",
                            },
                            new ImprovedDoorBehaviour.DoorOptions
                            {
                                CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Trapdoors/FirstPosition/CrimsonClosed.png", 0.4f),
                                OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Trapdoors/FirstPosition/CrimsonOpen.png", 0.4f),
                                CloseSound = ModAPI.LoadSound("Sfx/WoodenTrapdoorClose.mp3"),
                                OpenSound = ModAPI.LoadSound("Sfx/WoodenTrapdoorOpen.mp3"),
                                PhysicalProperties = "Wood",
                            },
                            new ImprovedDoorBehaviour.DoorOptions
                            {
                                CloseSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Trapdoors/FirstPosition/WarpedClosed.png", 0.4f),
                                OpenSprite = ModAPI.LoadSprite("Blocks&Stuff/Doors&Trapdoors/Trapdoors/FirstPosition/WarpedOpen.png", 0.4f),
                                CloseSound = ModAPI.LoadSound("Sfx/WoodenTrapdoorClose.mp3"),
                                OpenSound = ModAPI.LoadSound("Sfx/WoodenTrapdoorOpen.mp3"),
                                PhysicalProperties = "Wood",
                            }
                        };
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = Mass;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = Mass;
                        Instance.GetComponent<PhysicalBehaviour>().rigidbody.mass = Mass;
                    //
					}
				}
			);
    //Sea Lantern
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Sea Lantern",
                    NameToOrderByOverride = "E",
                    DescriptionOverride = "Blue light",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/39.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/sealantern.png", 0.4f);
                        Instance.FixColliders(); 
                        var light = ModAPI.CreateLight(Instance.transform, new Color(0.1f, 0.3f, 0.5f, 1f), 7.5f, 7f);

          
                    }
                }
            );
    //Snow Block
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Brick"), 
                    NameOverride = "Snow Block",
                    NameToOrderByOverride = "E", 
                    DescriptionOverride = "Minecraft Block",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Snow.png", 1f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/Snow.png", 0.4f);
						Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Snow");
						foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<BoxCollider2D>();
                        Instance.FixColliders();  
                    }
                }
            );
    //Pumpkins & Melon
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Brick"), 
                    NameOverride = "Pumpkins & Melon",
                    NameToOrderByOverride = "E", 
                    DescriptionOverride = "Okay, there's also the texture of melon.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Pumpkin.png", 1f),
                    AfterSpawn = (Instance) =>
                    {
                    //Main
                        float Mass = 0.75f;
                        string SortingLayerName = "Objects";
                    //Script
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/Pumpkin.png", 0.4f);
                        Instance.FixColliders();                        
                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Metal");
                        RandomItemsBehaviour RandomItemsBehaviour = Instance.GetOrAddComponent<RandomItemsBehaviour>();
                        RandomItemsBehaviour.Options = new RandomItemsBehaviour.ObjectOptions[]
                        {
                            new RandomItemsBehaviour.ObjectOptions
                            {
                                NewSprite = ModAPI.LoadSprite("Blocks&Stuff/Pumpkin.png", 0.4f),
                                SoundOnActivation = null,
                                PhysicalProperties = "Human",
                                SortingLayerName = "Default",
                                LightColor = new Color(0.89803f, 0.44705f, 0.15f, 1f),
                                Brightnesss = 0,
                                Radius = 15f,
                                LocalPosition = new Vector3(0, 0, 0),
                            },
                            new RandomItemsBehaviour.ObjectOptions
                            {
                                NewSprite = ModAPI.LoadSprite("Blocks&Stuff/Carved Pumpkin.png", 0.4f),
                                SoundOnActivation = null,
                                PhysicalProperties = "Human",
                                SortingLayerName = "Default",
                                LightColor = new Color(0.89803f, 0.44705f, 0.15f, 1f),
                                Brightnesss = 0,
                                Radius = 15f,
                                LocalPosition = new Vector3(0, 0, 0),
                            },
                            new RandomItemsBehaviour.ObjectOptions
                            {
                                NewSprite = ModAPI.LoadSprite("Blocks&Stuff/Jack o Lantern.png", 0.4f),
                                SoundOnActivation = null,
                                PhysicalProperties = "Human",
                                SortingLayerName = "Default",
                                LightColor = new Color(0.89803f, 0.44705f, 0.15f, 1f),
                                Brightnesss = 2,
                                Radius = 6f,
                                LocalPosition = new Vector3(0, 0, 0),
                            },
                            new RandomItemsBehaviour.ObjectOptions
                            {
                                NewSprite = ModAPI.LoadSprite("Blocks&Stuff/Melon.png", 0.4f),
                                SoundOnActivation = null,
                                PhysicalProperties = "Human",
                                SortingLayerName = "Default",
                                LightColor = new Color(0.89803f, 0.44705f, 0.15f, 1f),
                                Brightnesss = 0,
                                Radius = 15f,
                                LocalPosition = new Vector3(0, 0, 0),
                            }
                        };
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = Mass;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = Mass;
                        Instance.GetComponent<PhysicalBehaviour>().rigidbody.mass = Mass;
                    //
                    }
                }
            );
    //Froglight
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Brick"), 
                    NameOverride = "Froglight",
                    NameToOrderByOverride = "E", 
                    DescriptionOverride = "Frog and Light = Froglight.",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Verdant Froglight.png", 1f),
                    AfterSpawn = (Instance) =>
                    {
                    //Main
                        float Mass = 0.75f;
                        string SortingLayerName = "Objects";
                    //Script
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Blocks&Stuff/Ochre Froglight.png", 0.4f);
                        Instance.FixColliders();                        
                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Metal");
                        RandomItemsBehaviour RandomItemsBehaviour = Instance.GetOrAddComponent<RandomItemsBehaviour>();
                        RandomItemsBehaviour.Options = new RandomItemsBehaviour.ObjectOptions[]
                        {
                            new RandomItemsBehaviour.ObjectOptions
                            {
                                NewSprite = ModAPI.LoadSprite("Blocks&Stuff/Ochre Froglight.png", 0.4f),
                                SoundOnActivation = null,
                                PhysicalProperties = "Human",
                                SortingLayerName = "Default",
                                LightColor = Color.yellow,
                                Brightnesss = 1,
                                Radius = 5f,
                                LocalPosition = new Vector3(0, 0, 0),
                            },
                            new RandomItemsBehaviour.ObjectOptions
                            {
                                NewSprite = ModAPI.LoadSprite("Blocks&Stuff/Pearlescent Froglight.png", 0.4f),
                                SoundOnActivation = null,
                                PhysicalProperties = "Human",
                                SortingLayerName = "Default",
                                LightColor = Color.magenta,
                                Brightnesss = 1,
                                Radius = 5f,
                                LocalPosition = new Vector3(0, 0, 0),
                            },
                            new RandomItemsBehaviour.ObjectOptions
                            {
                                NewSprite = ModAPI.LoadSprite("Blocks&Stuff/Verdant Froglight.png", 0.4f),
                                SoundOnActivation = null,
                                PhysicalProperties = "Human",
                                SortingLayerName = "Default",
                                LightColor = Color.green,
                                Brightnesss = 1,
                                Radius = 5f,
                                LocalPosition = new Vector3(0, 0, 0),
                            },
                        };
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = Mass;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = Mass;
                        Instance.GetComponent<PhysicalBehaviour>().rigidbody.mass = Mass;
                    //
                    }
                }
            );
    //Rail
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Brick"), 
                    NameOverride = "Rail",
                    NameToOrderByOverride = "E", 
                    DescriptionOverride = "Minecraft Block",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Rail.png", 1f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Vehicles/Rail.png", 0.4f);
						Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Soft");
						foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<BoxCollider2D>();
                        Instance.FixColliders();  
                    }
                }
                );
    //Long Rail
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Brick"), 
                    NameOverride = "Long Rail",
                    NameToOrderByOverride = "E", 
                    DescriptionOverride = "Minecraft Block",
                    CategoryOverride = ModAPI.FindCategory("Minecraft"),
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Long Rail.png", 1f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Vehicles/Rail Long.png", 0.4f);
						Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Soft");
						foreach (var c in Instance.GetComponents<Collider2D>())
              				{ 
                     			GameObject.Destroy(c);
              				}
              			Instance.AddComponent<BoxCollider2D>();
                        Instance.FixColliders();  
                    }
                }
            );
#endregion     
#region Armor
        ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Iron Chestplate Armor M+", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "ZZZZZZZ",
                    DescriptionOverride = "Minecraft Armor", //new item description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Ir1.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("armors/Iron/chestplate.png"); //get the SpriteRenderer and replace its sprite with a custom one
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.2f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.2f;
                        int PartCount = 4;

                        if (!Instance.GetComponent<ArmorBehaviourTest>())
                            Instance.AddComponent<ArmorBehaviourTest>();

                        ArmorBehaviourTest armor = Instance.GetComponent<ArmorBehaviourTest>();

                        ArmorProperties prop = new ArmorProperties();
                        
                        prop.armorPiece = "UpperBody";
                        prop.armorTier = 3;
                        prop.sprite = "armors/Iron/chestplate.png";

                        armor.prop = prop;
                        armor.SetProperties();

                        ArmorProperties[] armProp = new ArmorProperties[PartCount - 1];

                        armProp[0].sprite = "armors/Iron/chestplate2.png";
                        armProp[0].armorTier = 3;
                        armProp[0].armorPiece = "MiddleBody";

                        armProp[1].sprite = "armors/Iron/chestplate3.png";
                        armProp[1].armorTier = 3;
                        armProp[1].armorPiece = "LowerBody";

                        armProp[2].sprite = "armors/Iron/Arm.png";
                        armProp[2].armorPiece = "UpperArm";
                        armProp[2].clone = true;
                        armProp[2].armorTier = 3;

                        if (armor.spawn)
                        {
                            armor.SetPieces = new ArmorBehaviourTest[PartCount - 1];
                            armor.SpawnOtherParts(armProp);
                        }

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Iron Helmet -MC", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "ZZZZZZZ",
                    DescriptionOverride = "Minecraft", //new item description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Ir2.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("armors/Iron/Helmet.png"); 
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.2f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.2f;

                        int PartCount = 1;

                        if (!Instance.GetComponent<ArmorBehaviourTest>())
                            Instance.AddComponent<ArmorBehaviourTest>();

                        ArmorBehaviourTest armor = Instance.GetComponent<ArmorBehaviourTest>();

                        ArmorProperties prop = new ArmorProperties();
                        
                        prop.armorPiece = "Head";
                        prop.armorTier = 3;
                        prop.clone = true;
                        prop.sprite = "armors/Iron/Helmet.png";

                        armor.prop = prop;
                        armor.SetProperties();

                        ArmorProperties[] armProp = new ArmorProperties[PartCount - 1];

                        if (armor.spawn)
                        {
                            armor.SetPieces = new ArmorBehaviourTest[PartCount - 1];
                            armor.SpawnOtherParts(armProp);
                        }

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Turtle Shell Helmet -MC", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "ZZZZZZZ",
                    DescriptionOverride = "Minecraft", //new item description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Tur4.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("armors/Golden/turtle.png"); 
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.2f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.2f;

                        int PartCount = 1;

                        if (!Instance.GetComponent<ArmorBehaviourTest>())
                            Instance.AddComponent<ArmorBehaviourTest>();

                        ArmorBehaviourTest armor = Instance.GetComponent<ArmorBehaviourTest>();

                        ArmorProperties prop = new ArmorProperties();
                        
                        prop.armorPiece = "Head";
                        prop.armorTier = 3;
                        prop.clone = true;
                        prop.sprite = "armors/Golden/turtle.png";

                        armor.prop = prop;
                        armor.SetProperties();

                        ArmorProperties[] armProp = new ArmorProperties[PartCount - 1];

                        if (armor.spawn)
                        {
                            armor.SetPieces = new ArmorBehaviourTest[PartCount - 1];
                            armor.SpawnOtherParts(armProp);
                        }

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Iron leggings M+", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "ZZZZZZZ",
                    DescriptionOverride = "Minecraft Armor", //new item description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Ir3.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("armors/Iron/none.png"); //get the SpriteRenderer and replace its sprite with a custom one
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.2f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.2f;
                        int PartCount = 3;

                        if (!Instance.GetComponent<ArmorBehaviourTest>())
                            Instance.AddComponent<ArmorBehaviourTest>();

                        ArmorBehaviourTest armor = Instance.GetComponent<ArmorBehaviourTest>();

                        ArmorProperties prop = new ArmorProperties();
                        
                        prop.armorPiece = "UpperLeg";
                        prop.armorTier = 3;
                        prop.sprite = "armors/Iron/none.png";
                        prop.clone = true;

                        armor.prop = prop;
                        armor.SetProperties();

                        ArmorProperties[] armProp = new ArmorProperties[PartCount - 1];

                        armProp[0].sprite = "armors/Iron/Leg1.png";
                        armProp[0].armorTier = 3;
                        armProp[0].armorPiece = "UpperLeg";
                        armProp[0].clone = true;

                        armProp[1].sprite = "armors/Iron/Leg2.png";
                        armProp[1].armorTier = 3;
                        armProp[1].armorPiece = "LowerLeg";
                        armProp[1].clone = true;



                        if (armor.spawn)
                        {
                            armor.SetPieces = new ArmorBehaviourTest[PartCount - 1];
                            armor.SpawnOtherParts(armProp);
                        }

                        Instance.FixColliders();
                    }
                }
            );

            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Diamond Chestplate Armor M+", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "ZZZZZZZ",
                    DescriptionOverride = "Minecraft Armor", //new item description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Di1.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("armors/Diamond/chestplate.png"); //get the SpriteRenderer and replace its sprite with a custom one
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.2f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.2f;
                        int PartCount = 4;

                        if (!Instance.GetComponent<ArmorBehaviourTest>())
                            Instance.AddComponent<ArmorBehaviourTest>();

                        ArmorBehaviourTest armor = Instance.GetComponent<ArmorBehaviourTest>();

                        ArmorProperties prop = new ArmorProperties();
                        
                        prop.armorPiece = "UpperBody";
                        prop.armorTier = 3;
                        prop.sprite = "armors/Diamond/chestplate.png";

                        armor.prop = prop;
                        armor.SetProperties();

                        ArmorProperties[] armProp = new ArmorProperties[PartCount - 1];

                        armProp[0].sprite = "armors/Diamond/chestplate2.png";
                        armProp[0].armorTier = 3;
                        armProp[0].armorPiece = "MiddleBody";

                        armProp[1].sprite = "armors/Diamond/chestplate3.png";
                        armProp[1].armorTier = 3;
                        armProp[1].armorPiece = "LowerBody";

                        armProp[2].sprite = "armors/Diamond/Arm.png";
                        armProp[2].armorPiece = "UpperArm";
                        armProp[2].clone = true;
                        armProp[2].armorTier = 3;

                        if (armor.spawn)
                        {
                            armor.SetPieces = new ArmorBehaviourTest[PartCount - 1];
                            armor.SpawnOtherParts(armProp);
                        }

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Diamond Helmet -MC", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "ZZZZZZZ",
                    DescriptionOverride = "Minecraft", //new item description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Di2.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("armors/Diamond/Helmet.png"); 
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.2f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.2f;

                        int PartCount = 1;

                        if (!Instance.GetComponent<ArmorBehaviourTest>())
                            Instance.AddComponent<ArmorBehaviourTest>();

                        ArmorBehaviourTest armor = Instance.GetComponent<ArmorBehaviourTest>();

                        ArmorProperties prop = new ArmorProperties();
                        
                        prop.armorPiece = "Head";
                        prop.armorTier = 3;
                        prop.clone = true;
                        prop.sprite = "armors/Diamond/Helmet.png";

                        armor.prop = prop;
                        armor.SetProperties();

                        ArmorProperties[] armProp = new ArmorProperties[PartCount - 1];

                        if (armor.spawn)
                        {
                            armor.SetPieces = new ArmorBehaviourTest[PartCount - 1];
                            armor.SpawnOtherParts(armProp);
                        }

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Diamond leggings M+", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "ZZZZZZZ",
                    DescriptionOverride = "Minecraft Armor", //new item description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Di3.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("armors/Diamond/none.png"); //get the SpriteRenderer and replace its sprite with a custom one
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.2f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.2f;
                        int PartCount = 3;

                        if (!Instance.GetComponent<ArmorBehaviourTest>())
                            Instance.AddComponent<ArmorBehaviourTest>();

                        ArmorBehaviourTest armor = Instance.GetComponent<ArmorBehaviourTest>();

                        ArmorProperties prop = new ArmorProperties();
                        
                        prop.armorPiece = "UpperLeg";
                        prop.armorTier = 3;
                        prop.sprite = "armors/Diamond/none.png";
                        prop.clone = true;

                        armor.prop = prop;
                        armor.SetProperties();

                        ArmorProperties[] armProp = new ArmorProperties[PartCount - 1];

                        armProp[0].sprite = "armors/Diamond/Leg1.png";
                        armProp[0].armorTier = 3;
                        armProp[0].armorPiece = "UpperLeg";
                        armProp[0].clone = true;

                        armProp[1].sprite = "armors/Diamond/Leg2.png";
                        armProp[1].armorTier = 3;
                        armProp[1].armorPiece = "LowerLeg";
                        armProp[1].clone = true;



                        if (armor.spawn)
                        {
                            armor.SetPieces = new ArmorBehaviourTest[PartCount - 1];
                            armor.SpawnOtherParts(armProp);
                        }

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Golden Chestplate Armor M+", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "ZZZZZZZ",
                    DescriptionOverride = "Minecraft Armor", //new item description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Go1.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("armors/Golden/chestplate.png"); //get the SpriteRenderer and replace its sprite with a custom one
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.2f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.2f;
                        int PartCount = 4;

                        if (!Instance.GetComponent<ArmorBehaviourTest>())
                            Instance.AddComponent<ArmorBehaviourTest>();

                        ArmorBehaviourTest armor = Instance.GetComponent<ArmorBehaviourTest>();

                        ArmorProperties prop = new ArmorProperties();
                        
                        prop.armorPiece = "UpperBody";
                        prop.armorTier = 3;
                        prop.sprite = "armors/Golden/chestplate.png";

                        armor.prop = prop;
                        armor.SetProperties();

                        ArmorProperties[] armProp = new ArmorProperties[PartCount - 1];

                        armProp[0].sprite = "armors/Golden/chestplate2.png";
                        armProp[0].armorTier = 3;
                        armProp[0].armorPiece = "MiddleBody";

                        armProp[1].sprite = "armors/Golden/chestplate3.png";
                        armProp[1].armorTier = 3;
                        armProp[1].armorPiece = "LowerBody";

                        armProp[2].sprite = "armors/Golden/Arm.png";
                        armProp[2].armorPiece = "UpperArm";
                        armProp[2].clone = true;
                        armProp[2].armorTier = 3;

                        if (armor.spawn)
                        {
                            armor.SetPieces = new ArmorBehaviourTest[PartCount - 1];
                            armor.SpawnOtherParts(armProp);
                        }

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Golden Helmet -MC", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "ZZZZZZZ",
                    DescriptionOverride = "Minecraft", //new item description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Go2.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("armors/Golden/Helmet.png"); 
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.2f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.2f;

                        int PartCount = 1;

                        if (!Instance.GetComponent<ArmorBehaviourTest>())
                            Instance.AddComponent<ArmorBehaviourTest>();

                        ArmorBehaviourTest armor = Instance.GetComponent<ArmorBehaviourTest>();

                        ArmorProperties prop = new ArmorProperties();
                        
                        prop.armorPiece = "Head";
                        prop.armorTier = 3;
                        prop.clone = true;
                        prop.sprite = "armors/Golden/Helmet.png";

                        armor.prop = prop;
                        armor.SetProperties();

                        ArmorProperties[] armProp = new ArmorProperties[PartCount - 1];

                        if (armor.spawn)
                        {
                            armor.SetPieces = new ArmorBehaviourTest[PartCount - 1];
                            armor.SpawnOtherParts(armProp);
                        }

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Golden leggings M+", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "ZZZZZZZ",
                    DescriptionOverride = "Minecraft Armor", //new item description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Go3.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("armors/Golden/none.png"); //get the SpriteRenderer and replace its sprite with a custom one
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.2f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.2f;
                        int PartCount = 3;

                        if (!Instance.GetComponent<ArmorBehaviourTest>())
                            Instance.AddComponent<ArmorBehaviourTest>();

                        ArmorBehaviourTest armor = Instance.GetComponent<ArmorBehaviourTest>();

                        ArmorProperties prop = new ArmorProperties();
                        
                        prop.armorPiece = "UpperLeg";
                        prop.armorTier = 3;
                        prop.sprite = "armors/Golden/none.png";
                        prop.clone = true;

                        armor.prop = prop;
                        armor.SetProperties();

                        ArmorProperties[] armProp = new ArmorProperties[PartCount - 1];

                        armProp[0].sprite = "armors/Golden/Leg1.png";
                        armProp[0].armorTier = 3;
                        armProp[0].armorPiece = "UpperLeg";
                        armProp[0].clone = true;

                        armProp[1].sprite = "armors/Golden/Leg2.png";
                        armProp[1].armorTier = 3;
                        armProp[1].armorPiece = "LowerLeg";
                        armProp[1].clone = true;



                        if (armor.spawn)
                        {
                            armor.SetPieces = new ArmorBehaviourTest[PartCount - 1];
                            armor.SpawnOtherParts(armProp);
                        }

                        Instance.FixColliders();
                    }
                }
            );
             ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Leather Chestplate Armor M+", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "ZZZZZZZ",
                    DescriptionOverride = "Minecraft Armor", //new item description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Le1.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("armors/Leather/chestplate.png"); //get the SpriteRenderer and replace its sprite with a custom one
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.2f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.2f;
                        int PartCount = 4;

                        if (!Instance.GetComponent<ArmorBehaviourTest>())
                            Instance.AddComponent<ArmorBehaviourTest>();

                        ArmorBehaviourTest armor = Instance.GetComponent<ArmorBehaviourTest>();

                        ArmorProperties prop = new ArmorProperties();
                        
                        prop.armorPiece = "UpperBody";
                        prop.armorTier = 1;
                        prop.sprite = "armors/Leather/chestplate.png";

                        armor.prop = prop;
                        armor.SetProperties();

                        ArmorProperties[] armProp = new ArmorProperties[PartCount - 1];

                        armProp[0].sprite = "armors/Leather/chestplate2.png";
                        armProp[0].armorTier = 1;
                        armProp[0].armorPiece = "MiddleBody";

                        armProp[1].sprite = "armors/Leather/chestplate3.png";
                        armProp[1].armorTier = 1;
                        armProp[1].armorPiece = "LowerBody";

                        armProp[2].sprite = "armors/Leather/Arm.png";
                        armProp[2].armorPiece = "UpperArm";
                        armProp[2].clone = true;
                        armProp[2].armorTier = 1;

                        if (armor.spawn)
                        {
                            armor.SetPieces = new ArmorBehaviourTest[PartCount - 1];
                            armor.SpawnOtherParts(armProp);
                        }

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Leather Helmet -MC", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "ZZZZZZZ",
                    DescriptionOverride = "Minecraft", //new item description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Le2.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("armors/Leather/Helmet.png"); 
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.2f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.2f;

                        int PartCount = 1;

                        if (!Instance.GetComponent<ArmorBehaviourTest>())
                            Instance.AddComponent<ArmorBehaviourTest>();

                        ArmorBehaviourTest armor = Instance.GetComponent<ArmorBehaviourTest>();

                        ArmorProperties prop = new ArmorProperties();
                        
                        prop.armorPiece = "Head";
                        prop.armorTier = 1;
                        prop.clone = true;
                        prop.sprite = "armors/Leather/Helmet.png";

                        armor.prop = prop;
                        armor.SetProperties();

                        ArmorProperties[] armProp = new ArmorProperties[PartCount - 1];

                        if (armor.spawn)
                        {
                            armor.SetPieces = new ArmorBehaviourTest[PartCount - 1];
                            armor.SpawnOtherParts(armProp);
                        }

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Leather leggings M+", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "ZZZZZZZ",
                    DescriptionOverride = "Minecraft Armor", //new item description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Le3.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("armors/Leather/none.png"); //get the SpriteRenderer and replace its sprite with a custom one
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.2f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.2f;
                        int PartCount = 3;

                        if (!Instance.GetComponent<ArmorBehaviourTest>())
                            Instance.AddComponent<ArmorBehaviourTest>();

                        ArmorBehaviourTest armor = Instance.GetComponent<ArmorBehaviourTest>();

                        ArmorProperties prop = new ArmorProperties();
                        
                        prop.armorPiece = "UpperLeg";
                        prop.armorTier = 1;
                        prop.sprite = "armors/Leather/none.png";
                        prop.clone = true;

                        armor.prop = prop;
                        armor.SetProperties();

                        ArmorProperties[] armProp = new ArmorProperties[PartCount - 1];

                        armProp[0].sprite = "armors/Leather/Leg1.png";
                        armProp[0].armorTier = 1;
                        armProp[0].armorPiece = "UpperLeg";
                        armProp[0].clone = true;

                        armProp[1].sprite = "armors/Leather/Leg2.png";
                        armProp[1].armorTier = 1;
                        armProp[1].armorPiece = "LowerLeg";
                        armProp[1].clone = true;



                        if (armor.spawn)
                        {
                            armor.SetPieces = new ArmorBehaviourTest[PartCount - 1];
                            armor.SpawnOtherParts(armProp);
                        }

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Netherite Chestplate Armor M+", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "ZZZZZZZ",
                    DescriptionOverride = "Minecraft Armor", //new item description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Ne1.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("armors/Netherite/chestplate.png"); //get the SpriteRenderer and replace its sprite with a custom one
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.2f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.2f;
                        int PartCount = 4;

                        if (!Instance.GetComponent<ArmorBehaviourTest>())
                            Instance.AddComponent<ArmorBehaviourTest>();

                        ArmorBehaviourTest armor = Instance.GetComponent<ArmorBehaviourTest>();

                        ArmorProperties prop = new ArmorProperties();
                        
                        prop.armorPiece = "UpperBody";
                        prop.armorTier = 3;
                        prop.sprite = "armors/Netherite/chestplate.png";

                        armor.prop = prop;
                        armor.SetProperties();

                        ArmorProperties[] armProp = new ArmorProperties[PartCount - 1];

                        armProp[0].sprite = "armors/Netherite/chestplate2.png";
                        armProp[0].armorTier = 3;
                        armProp[0].armorPiece = "MiddleBody";

                        armProp[1].sprite = "armors/Netherite/chestplate3.png";
                        armProp[1].armorTier = 3;
                        armProp[1].armorPiece = "LowerBody";

                        armProp[2].sprite = "armors/Netherite/Arm.png";
                        armProp[2].armorPiece = "UpperArm";
                        armProp[2].clone = true;
                        armProp[2].armorTier = 3;

                        if (armor.spawn)
                        {
                            armor.SetPieces = new ArmorBehaviourTest[PartCount - 1];
                            armor.SpawnOtherParts(armProp);
                        }

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Netherite Helmet -MC", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "ZZZZZZZ",
                    DescriptionOverride = "Minecraft", //new item description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Ne2.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("armors/Netherite/Helmet.png"); 
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.2f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.2f;

                        int PartCount = 1;

                        if (!Instance.GetComponent<ArmorBehaviourTest>())
                            Instance.AddComponent<ArmorBehaviourTest>();

                        ArmorBehaviourTest armor = Instance.GetComponent<ArmorBehaviourTest>();

                        ArmorProperties prop = new ArmorProperties();
                        
                        prop.armorPiece = "Head";
                        prop.armorTier = 3;
                        prop.clone = true;
                        prop.sprite = "armors/Netherite/Helmet.png";

                        armor.prop = prop;
                        armor.SetProperties();

                        ArmorProperties[] armProp = new ArmorProperties[PartCount - 1];

                        if (armor.spawn)
                        {
                            armor.SetPieces = new ArmorBehaviourTest[PartCount - 1];
                            armor.SpawnOtherParts(armProp);
                        }

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Netherite leggings M+", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "ZZZZZZZ",
                    DescriptionOverride = "Minecraft Armor", //new item description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Ne3.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("armors/Netherite/none.png"); //get the SpriteRenderer and replace its sprite with a custom one
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.2f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.2f;
                        int PartCount = 3;

                        if (!Instance.GetComponent<ArmorBehaviourTest>())
                            Instance.AddComponent<ArmorBehaviourTest>();

                        ArmorBehaviourTest armor = Instance.GetComponent<ArmorBehaviourTest>();

                        ArmorProperties prop = new ArmorProperties();
                        
                        prop.armorPiece = "UpperLeg";
                        prop.armorTier = 3;
                        prop.sprite = "armors/Netherite/none.png";
                        prop.clone = true;

                        armor.prop = prop;
                        armor.SetProperties();

                        ArmorProperties[] armProp = new ArmorProperties[PartCount - 1];

                        armProp[0].sprite = "armors/Netherite/Leg1.png";
                        armProp[0].armorTier = 3;
                        armProp[0].armorPiece = "UpperLeg";
                        armProp[0].clone = true;

                        armProp[1].sprite = "armors/Netherite/Leg2.png";
                        armProp[1].armorTier = 3;
                        armProp[1].armorPiece = "LowerLeg";
                        armProp[1].clone = true;



                        if (armor.spawn)
                        {
                            armor.SetPieces = new ArmorBehaviourTest[PartCount - 1];
                            armor.SpawnOtherParts(armProp);
                        }

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Chainmail Chestplate Armor M+", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "ZZZZZZZ",
                    DescriptionOverride = "Minecraft Armor", //new item description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Ch1.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("armors/Chainmail/chestplate.png"); //get the SpriteRenderer and replace its sprite with a custom one
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.2f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.2f;
                        int PartCount = 4;

                        if (!Instance.GetComponent<ArmorBehaviourTest>())
                            Instance.AddComponent<ArmorBehaviourTest>();

                        ArmorBehaviourTest armor = Instance.GetComponent<ArmorBehaviourTest>();

                        ArmorProperties prop = new ArmorProperties();
                        
                        prop.armorPiece = "UpperBody";
                        prop.armorTier = 1;
                        prop.sprite = "armors/Chainmail/chestplate.png";

                        armor.prop = prop;
                        armor.SetProperties();

                        ArmorProperties[] armProp = new ArmorProperties[PartCount - 1];

                        armProp[0].sprite = "armors/Chainmail/chestplate2.png";
                        armProp[0].armorTier = 1;
                        armProp[0].armorPiece = "MiddleBody";

                        armProp[1].sprite = "armors/Chainmail/chestplate3.png";
                        armProp[1].armorTier = 1;
                        armProp[1].armorPiece = "LowerBody";

                        armProp[2].sprite = "armors/Chainmail/Arm.png";
                        armProp[2].armorPiece = "UpperArm";
                        armProp[2].clone = true;
                        armProp[2].armorTier = 1;

                        if (armor.spawn)
                        {
                            armor.SetPieces = new ArmorBehaviourTest[PartCount - 1];
                            armor.SpawnOtherParts(armProp);
                        }

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Chainmail Helmet -MC", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "ZZZZZZZ",
                    DescriptionOverride = "Minecraft", //new item description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Ch2.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("armors/Chainmail/Helmet.png"); 
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.2f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.2f;

                        int PartCount = 1;

                        if (!Instance.GetComponent<ArmorBehaviourTest>())
                            Instance.AddComponent<ArmorBehaviourTest>();

                        ArmorBehaviourTest armor = Instance.GetComponent<ArmorBehaviourTest>();

                        ArmorProperties prop = new ArmorProperties();
                        
                        prop.armorPiece = "Head";
                        prop.armorTier = 1;
                        prop.clone = true;
                        prop.sprite = "armors/Chainmail/Helmet.png";

                        armor.prop = prop;
                        armor.SetProperties();

                        ArmorProperties[] armProp = new ArmorProperties[PartCount - 1];

                        if (armor.spawn)
                        {
                            armor.SetPieces = new ArmorBehaviourTest[PartCount - 1];
                            armor.SpawnOtherParts(armProp);
                        }

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Chainmail leggings M+", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "ZZZZZZZ",
                    DescriptionOverride = "Minecraft Armor", //new item description
                    CategoryOverride = ModAPI.FindCategory("Minecraft"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Ch3.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("armors/Chainmail/none.png"); //get the SpriteRenderer and replace its sprite with a custom one
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.2f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.2f;
                        int PartCount = 3;

                        if (!Instance.GetComponent<ArmorBehaviourTest>())
                            Instance.AddComponent<ArmorBehaviourTest>();

                        ArmorBehaviourTest armor = Instance.GetComponent<ArmorBehaviourTest>();

                        ArmorProperties prop = new ArmorProperties();
                        
                        prop.armorPiece = "UpperLeg";
                        prop.armorTier = 1;
                        prop.sprite = "armors/Chainmail/none.png";
                        prop.clone = true;

                        armor.prop = prop;
                        armor.SetProperties();

                        ArmorProperties[] armProp = new ArmorProperties[PartCount - 1];

                        armProp[0].sprite = "armors/Chainmail/Leg1.png";
                        armProp[0].armorTier = 1;
                        armProp[0].armorPiece = "UpperLeg";
                        armProp[0].clone = true;

                        armProp[1].sprite = "armors/Chainmail/Leg2.png";
                        armProp[1].armorTier = 1;
                        armProp[1].armorPiece = "LowerLeg";
                        armProp[1].clone = true;



                        if (armor.spawn)
                        {
                            armor.SetPieces = new ArmorBehaviourTest[PartCount - 1];
                            armor.SpawnOtherParts(armProp);
                        }

                        Instance.FixColliders();
                    }
                }
            );
#endregion
        }
    }
//MaceBehaviour
    public class MaceBehaviour : MonoBehaviour
    {
        private GameObject gameobjectBall;

        public Vector2 ChainConnector = new Vector2(-0.07f, 0.42f);
        public Vector2 ballConnector = new Vector2(0.02f, 0.07f);

        Sprite Ball;
        private void Awake()
        {
            var mace = ModAPI.CreatePhysicalObject("Float", ModAPI.LoadSprite("Weapons/Fishing Hook.png"));
            foreach(var hitbox in mace.GetComponents<Collider2D>()) { Destroy(hitbox); }
            mace.AddComponent<CircleCollider2D>();           
            mace.transform.SetParent(gameObject.transform);
            mace.transform.localPosition = new Vector3(ChainConnector.x, ChainConnector.y + 1f);
            

            DistanceJoint2D joint = gameObject.AddComponent<DistanceJoint2D>();
            joint.maxDistanceOnly = true;
            joint.enableCollision = false;
            joint.distance = 1.5f;
            joint.autoConfigureDistance = false;
            joint.breakForce = Mathf.Infinity;
            joint.breakTorque = Mathf.Infinity;
            joint.anchor = ChainConnector;
            joint.connectedBody = mace.GetComponent<Rigidbody2D>();
            joint.connectedAnchor = ballConnector;
            JointCreated(joint);

            PhysicalBehaviour phys = mace.GetComponent<PhysicalBehaviour>();
            phys.Properties = ModAPI.FindPhysicalProperties("Wood");
            phys.InitialMass = 0.2f;
            phys.TrueInitialMass = 0.2f;
            
        }
        void JointCreated(DistanceJoint2D joint)
        {
            EnergyWireBehaviour energyWireBehaviour = joint.gameObject.AddComponent<EnergyWireBehaviour>();
            energyWireBehaviour.WireColor = Color.black;
            energyWireBehaviour.WireMaterial = Resources.Load<Material>("Materials/Wire");
            energyWireBehaviour.WireWidth = 0.025f;
            energyWireBehaviour.typedJoint = joint;
        }
        }
//DoorBehaviour
    class DoorBehaviour : MonoBehaviour
    {
       private void Start()
       {
           renderer = gameObject.GetComponent<SpriteRenderer>();
           UpdateState();
       }
       void UpdateState()
       {
           if(Activated)
           {
               renderer.sprite = B;
               renderer.sortingLayerName = "Bottom";
               gameObject.layer = 10;
           }
           else
           {
               renderer.sprite = A;
               renderer.sortingLayerName = "Objects";
               gameObject.layer = LayerMask.NameToLayer("Objects");
           }
       }
       private void Use() {
         if (!base.enabled) {
            return;
         }
         
         if(Activated)
         {
            audio.PlayOneShot(audioclipB, 1f);
         }
         else
         {
            audio.PlayOneShot(audioclipA, 1f);
         }
         
         
         SetActivated(!Activated);
      }
      private void SetActivated(bool activated) {
         Activated = activated;
         UpdateState();
      }

      public bool Activated = false;
      [SkipSerialisation]
      public SpriteRenderer renderer;
      [SkipSerialisation]
      public Sprite A;
      [SkipSerialisation]
      public Sprite B;
      
      [SkipSerialisation]
      public AudioSource audio;
      [SkipSerialisation]
      public AudioClip audioclipA;
      [SkipSerialisation]
      public AudioClip audioclipB;

     }
//RemoveAfterParentDie
    public class RemoveAfterParentDie : MonoBehaviour  //Used to remove additional sprites of human
    {
        private LimbBehaviour parentLimb;
        private void Awake()
        {
            parentLimb = transform.parent.gameObject.GetComponent<LimbBehaviour>();
        }
        private void Update()
        {
            if (parentLimb.Health <= 0f)
            {
                GameObject.Destroy(gameObject);
            }
        }
    }
	public class CustomCollision : MonoBehaviour
	{
		public SpriteRenderer SpriteRenderer;
		public Sprite Sprite;
		public Sprite CollisionSprite;
		
		private void Start()
		{         
			gameObject.GetComponent<SpriteRenderer>().sprite = CollisionSprite;

			foreach (var ColliderBoxDeleteVar in gameObject.GetComponents<Collider2D>())
			{
				GameObject.Destroy(ColliderBoxDeleteVar);
			}
			gameObject.FixColliders();

			gameObject.GetComponent<SpriteRenderer>().sprite = Sprite;

			gameObject.GetComponent<PhysicalBehaviour>().RefreshOutline();
			gameObject.GetComponent<PhysicalBehaviour>().BakeColliderGridPoints();
		}
	}
	public class OnHingeJointBreakSetParent : MonoBehaviour
	{
		[SkipSerialisation]
		public HingeJoint2D joint;

		private void OnJointBreak2D(Joint2D brokenJoint)
		{
			if (brokenJoint == joint)
			{
				gameObject.transform.SetParent(null);
			}
		}
	}
	public class OnBreak : MonoBehaviour, Messages.IShot, Messages.IStabbed
	{
		public float startHealth = 4f;
		private float Health;
		
		[SkipSerialisation]
		private SpriteRenderer render;
		[SkipSerialisation]
		public Sprite DeadSprite;
        public Sprite MoreAlmostDeadSprite;
		public Sprite AlmostDeadSprite;
        public Sprite MoreWoundedSprite;
		public Sprite WoundedSprite; 

		public float DeadHealthThreshold = 0;
        public float MoreAlmostDeadHealthThreshold = 5;
		public float AlmostDeadHealthThreshold = 10;
        public float MoreWoundedHealthThreshold = 15;
		public float WoundedHealthThreshold = 20;
		
		private void Awake()
		{
			render = gameObject.GetComponent<SpriteRenderer>();
		}
		private void Start()
        {
            Health = startHealth;
        }
		public void Shot(Shot shot)
		{
			Health -= 1f;
			CheckHealth();
		}
		public void Stabbed(Stabbing stabbing)
		{
			Health -= 0.5f;
			CheckHealth();
		}
		public void OnFragmentHit(float force)
		{
			Health -= 5f;
			CheckHealth();	
		}
		private void CheckHealth()
		{
			if (Health <= DeadHealthThreshold)
			{
				render.sprite = DeadSprite;
				Debug.Log(Health);
				//ModAPI.CreateParticleEffect("BrokenElectronicsSpark", gameObject.transform.position);
				//действия при смерти
			}
            else if (Health <= MoreAlmostDeadHealthThreshold)
			{
				render.sprite = MoreAlmostDeadSprite;
				Debug.Log(Health);
			}
			else if (Health <= AlmostDeadHealthThreshold)
			{
				render.sprite = AlmostDeadSprite;
				Debug.Log(Health);
			}
            else if (Health <= MoreWoundedHealthThreshold)
			{
				render.sprite = MoreWoundedSprite;
				Debug.Log(Health);
			}
			else if (Health <= WoundedHealthThreshold)
			{
				render.sprite = WoundedSprite;
				Debug.Log(Health);
			}
		}
	}
    public class AnimationBehaviour : MonoBehaviour
    {
        private int SpriteNumber = 0;

        private bool nowAlive = false;

        private GameObject AnimationObject;
        [SkipSerialisation]
        private SpriteRenderer render;
        [SkipSerialisation]
        private PersonBehaviour Person;
        public float AnimationDelay = 0.03f;

        public Sprite[] AnimationSprites; //спрайты анимации
        
        private void Awake()
        {
            AnimationObject = new GameObject();
            AnimationObject.transform.SetParent(gameObject.transform);
            AnimationObject.transform.localPosition = Vector3.zero;
            AnimationObject.transform.localEulerAngles = Vector3.zero;
            AnimationObject.transform.localScale = Vector3.one;

            render = AnimationObject.AddComponent<SpriteRenderer>();
            render.sortingLayerName = "Bubbles";
            //render.color = new Color(1f, 1f, 1f, 0.5f);

            Person = gameObject.GetComponent<LimbBehaviour>().Person;
            //body.PhysicalBehaviour.MakeWeightless();
        }
        private IEnumerator Animation()
        {
            while (nowAlive)
            {
                if (SpriteNumber >= AnimationSprites.Length) //перезапуск при последнем спрайте анимации
                {
                    SpriteNumber = 0;
                }
                render.sprite = AnimationSprites[SpriteNumber];
                SpriteNumber++;
                yield return new WaitForSeconds(AnimationDelay); //задержка между кадрами анимации
            }
        }
        private void Update()
        {
            if (Person.IsAlive() && !nowAlive)        
            {
                nowAlive = true;
                StartCoroutine(Animation());
            }
            else if (!Person.IsAlive() && nowAlive)
            {
                nowAlive = false;
            }
        }
    }
        public class BeeFlyBehaviour : MonoBehaviour
    {
        private bool nowDead = true;
        private void Update()
        {            
            if (!nowDead && !gameObject.GetComponent<LimbBehaviour>().Person.IsAlive()) //если умрет
            {
                nowDead = true;
                gameObject.GetComponent<PhysicalBehaviour>().MakeWeightful();
            }
            else if (nowDead && gameObject.GetComponent<LimbBehaviour>().Person.IsAlive()) //если оживет
            {
                nowDead = false;
                gameObject.GetComponent<PhysicalBehaviour>().MakeWeightless();
            }
        }
    }
    public class RoomDoorBehavior : MonoBehaviour
	{
		private bool Opened = false;   
		[SkipSerialisation]
		private SpriteRenderer renderer;
		[SkipSerialisation]
		private AudioSource AS;
		
		//Changeable variables
		[SkipSerialisation]
		public Sprite OpenS;
		[SkipSerialisation]
		public Sprite CloseS;
		
		[SkipSerialisation]
		public AudioClip OpenClip;
		[SkipSerialisation]
		public AudioClip CloseClip;
		
		[SkipSerialisation]
		public Vector3 DoorPosition = default(Vector3); //позиция двери относительбно комнаты ///x, y, 0
		[SkipSerialisation]
		public string PhysicalProperties = "Wood";
		[SkipSerialisation]
		//public bool FlipObject = false;

		public void CreateDoor(GameObject Instance, GameObject Parent)
		{  
			AS = gameObject.AddComponent<AudioSource>();

			PhysicalBehaviour phys = gameObject.GetComponent<PhysicalBehaviour>();  
			phys.SendUserPropagation = false;
			phys.Properties = ModAPI.FindPhysicalProperties(PhysicalProperties);
			//phys.MainAudioSource.volume = 0.5f;

			renderer = gameObject.GetComponent<SpriteRenderer>();
			renderer.sprite = CloseS; 
			//renderer.flipY = FlipObject;       
			gameObject.FixColliders();
			
			//Attach    
			gameObject.transform.SetParent(Instance.transform);

			gameObject.transform.localEulerAngles = Vector3.zero;
			gameObject.transform.localPosition = DoorPosition;     
			gameObject.transform.localScale = Vector3.one;
			
			gameObject.transform.SetParent(Instance.transform);
			
			Instance.transform.SetParent(Parent.transform);
			gameObject.transform.SetParent(Parent.transform);

			FixedJoint2D fixedJoint2D = gameObject.AddComponent<FixedJoint2D>();
			fixedJoint2D.connectedBody = Instance.GetComponent<Rigidbody2D>();
			//fixedJoint2D.breakForce = 1000; //прочность соединения          
		}    
		private void Use()
		{
			Opened = !Opened;
			if(Opened)
			{
				renderer.sprite = OpenS;
				renderer.sortingLayerName = "Bottom";
				gameObject.layer = 10;
				AS.PlayOneShot(OpenClip);
			}
			else
			{
				renderer.sprite = CloseS;
				renderer.sortingLayerName = "Objects";
				gameObject.layer = LayerMask.NameToLayer("Objects");
				AS.PlayOneShot(CloseClip);
			}
		}
	}
    public class SpawnObjectBehavior : MonoBehaviour
	{
		//Changeable variables
		public Sprite sprite;
		public Sprite CollisionSprite;

		[SkipSerialisation]
		public Vector3 ObjectPosition = default(Vector3);//позиция относительбно комнаты ///x, y, 0

		public string PhysicalProperties = "Wood";
		public bool SetParent = true;
		public bool CreateJoint = true;
		//public bool FlipObject = false;

		public void CreateObject(GameObject Instance)
		{  
			PhysicalBehaviour phys = gameObject.GetComponent<PhysicalBehaviour>();            
			phys.Properties = ModAPI.FindPhysicalProperties(PhysicalProperties);

			gameObject.GetComponent<SpriteRenderer>().sprite = sprite;
			foreach (var ColliderBoxDeleteVar in gameObject.GetComponents<PolygonCollider2D>())
			{
				GameObject.Destroy(ColliderBoxDeleteVar);
			}
            gameObject.AddComponent<PolygonCollider2D>();
			gameObject.FixColliders();

			
			//Attach
			if (SetParent)
			{
				gameObject.transform.SetParent(Instance.transform);    
			}                
			gameObject.transform.eulerAngles = new Vector3(0f, 0f, 0f); //0, 0,  угол
			gameObject.transform.position = Instance.transform.position + ObjectPosition; 
			gameObject.transform.localScale = Vector3.one;

			if (CreateJoint)
			{
				FixedJoint2D fixedJoint2D = gameObject.AddComponent<FixedJoint2D>();
				fixedJoint2D.connectedBody = Instance.GetComponent<Rigidbody2D>();
				//fixedJoint2D.breakForce = 1000; //прочность соединения
			}                
		}        
	}
    public class ReplaceObjectBehaviour : MonoBehaviour
	{
		public GameObject Object;
		public string ObjectName;
		public string ObjectOriginalItem;
		[SkipSerialisation]
		public Sprite ObjectSprite;
		public Vector3 ObjectSpawnPosition = new Vector3(0f, 0f);

		public int AddComponentNumber = 0;

		private void Use()
		{
			Object = GameObject.Instantiate(ModAPI.FindSpawnable(ObjectOriginalItem).Prefab);
			//Object = ModAPI.CreatePhysicalObject(ObjectOriginalItem, ObjectSprite);
			Object.transform.name = ObjectName;
			Object.transform.position = this.transform.position + ObjectSpawnPosition;
			
			Object.GetComponent<SpriteRenderer>().sprite = ObjectSprite;			
			Object.FixColliders();

			CheckAddedComponent();

			Destroy(gameObject);
		}
		private void CheckAddedComponent()
		{
			if (AddComponentNumber != 0)
			{
				switch (AddComponentNumber)
				{
					case 1: //Ultimate Accelerant
					{
						var phys = Object.GetOrAddComponent<PhysicalBehaviour>();
						phys.Properties = ModAPI.FindPhysicalProperties("Soft");
						phys.TrueInitialMass = 0.01f;
						phys.InitialMass = 0.01f;
						phys.rigidbody.mass = 0.01f;
						break;
					}
					case 2: //Mobile Respawn Beacon
					{
						var phys = Object.GetOrAddComponent<PhysicalBehaviour>();
						phys.TrueInitialMass = 25f;
						phys.InitialMass = 25f;
						phys.rigidbody.mass = 25f;
						break;
					}
					default:
						break;
				}
				Debug.Log("<b>ReplaceObjectBehaviour:</b> Added Component " + AddComponentNumber);
			}
		}
	}
    public class DestructibleObjectBehaviour : MonoBehaviour
    {
        private void OnCollisionStay2D(Collision2D collision)
        {
            EvaluateCollision(collision);
        }
        private void OnCollisionEnter2D(Collision2D collision)
        {
            EvaluateCollision(collision);
        }
        private void EvaluateCollision(Collision2D collision)
        {
            int contacts = collision.GetContacts(contactBuffer);
            float num = Utils.GetMinImpulse(contactBuffer, contacts);
            Vector2 point = contactBuffer[0].point;
            if(num < BreakingThreshold)
            {
                return;
            }
            Broken = true;
            Break();
        }
        private void Shot(Shot shot)
        {
            if(ShotDamage)
            {
                Break();
            }
        }
        private void OnFragmentHit()
        {
            Break();
        }
        private void Break()
        {
            GameObject audioObject = new GameObject("Audio Object");
            audioObject.transform.position = gameObject.transform.position;
            AudioSource audioSource = audioObject.AddComponent<AudioSource>();
            audioSource.spread = 0f;
            audioSource.volume = 0.8f;
            audioSource.minDistance = 10f;
            audioSource.spatialBlend = 1f;
            audioSource.dopplerLevel = 1f;
            audioSource.PlayOneShot(breakSounds.PickRandom(), 1f);
            audioObject.AddComponent<AudioSourceTimeScaleBehaviour>();
            for(int i = 0; i < debrisNumber; i++)
            {
                debris = ModAPI.CreatePhysicalObject("Debris "+i.ToString(), debrisSprites[i]);
                debris.transform.localScale = gameObject.transform.localScale;
                debris.transform.position = gameObject.transform.position + debrisPositions[i];
                debris.GetComponent<Rigidbody2D>().velocity = gameObject.GetComponent<Rigidbody2D>().velocity + UnityEngine.Random.insideUnitCircle * 2f;
                debris.GetComponent<Rigidbody2D>().angularVelocity = gameObject.GetComponent<Rigidbody2D>().angularVelocity + UnityEngine.Random.Range(-200, 200);
                debris.GetOrAddComponent<DebrisComponent>();
                var debrisPhys = debris.GetComponent<PhysicalBehaviour>();
                debrisPhys.Properties = debrisProperties;
                debrisPhys.InitialMass = debrisMass;
                debrisPhys.rigidbody.mass = debrisMass;
                debrisPhys.TrueInitialMass = debrisPhys.InitialMass;
                var collision = debris.GetOrAddComponent<CustomCollision>();
                collision.SpriteRenderer = debris.GetComponent<SpriteRenderer>();
                collision.Sprite = debrisSprites[i];
                collision.CollisionSprite = debrisCollisionSprites[i];
				debris.layer = LayerMask.NameToLayer("Objects");
                //debris.layer = 10;
                debris.AddComponent<ActOnDestroy>().DestroyWith = new GameObject[]
                {
                    audioObject
                };
            }
            gameObject.SetActive(false);
        }

        public bool Broken = false;
        public bool ShotDamage = true;
        public GameObject debris;
        private ContactPoint2D[] contactBuffer = new ContactPoint2D[12];
        public AudioClip[] breakSounds;
        public Sprite[] debrisCollisionSprites;
        public Sprite[] debrisSprites;
        public Vector3[] debrisPositions;
        public PhysicalProperties debrisProperties;
        public float debrisMass;
        public float BreakingThreshold;
        public int debrisNumber;
    }
    public class NoCollideWithObjectsInTrigger : MonoBehaviour
    {
        private void Start()
        {
            collidersWithoutTrigger = GetComponents<Collider2D>().ToList();
            collidersWithoutTrigger.Remove(Trigger);
        }
        private void OnTriggerEnter2D(Collider2D collider)
        {
            if (collider.isTrigger || noCollides.ContainsKey(collider))
            {
                return;
            }
            NoCollide noCollide = gameObject.AddComponent<NoCollide>();
            noCollide.NoCollideSetA = Colliders;
            noCollide.NoCollideSetB = new Collider2D[]
            { 
                collider 
            };
            noCollides.Add(collider, noCollide);
        }
        private void OnTriggerExit2D(Collider2D collider)
        {
            if (!noCollides.ContainsKey(collider))
            {
                return;
            }
            Destroy(noCollides[collider]);
            noCollides.Remove(collider);
        }
        public void RemoveCollider(int index)
        {
            List<Collider2D> colliders = Colliders.ToList();
            colliders.RemoveAt(index);
            Colliders = colliders.ToArray();
            Dictionary<Collider2D, NoCollide>.KeyCollection otherColliders = noCollides.Keys;
            foreach (Collider2D collider in otherColliders.ToList())
            {
                OnTriggerExit2D(collider);
                OnTriggerEnter2D(collider);
            }
        }
        private void OnCableConnect(WireBehaviour wire)
        {
            if (wire.untypedJoint.enableCollision)
            {
                return;
            }
            wire.untypedJoint.enableCollision = true;
            if (wire.untypedJoint.connectedBody == GetComponent<Rigidbody2D>())
            {
                colliders.Add(wire, wire.GetComponents<Collider2D>());
            }
            else
            {
                colliders.Add(wire, wire.untypedJoint.connectedBody.GetComponents<Collider2D>());
            }
            foreach (Collider2D collider in collidersWithoutTrigger)
            {
                foreach (Collider2D otherCollider in colliders[wire])
                {
                    IgnoreCollisionStackController.IgnoreCollisionSubstituteMethod(collider, otherCollider);
                }
            }
            connectedWires.Add(wire);
        }
        private void OnCableDisconnect(WireBehaviour wire)
        {
            if (!connectedWires.Contains(wire))
            {
                return;
            }
            foreach (Collider2D collider in collidersWithoutTrigger)
            {
                foreach (Collider2D otherCollider in colliders[wire])
                {
                    IgnoreCollisionStackController.IgnoreCollisionSubstituteMethod(collider, otherCollider, false);
                }
            }
            connectedWires.Remove(wire);
            colliders.Remove(wire);
        }

        public Collider2D[] Colliders;
        public Collider2D Trigger;
        private List<Collider2D> collidersWithoutTrigger;
        private List<WireBehaviour> connectedWires = new List<WireBehaviour>();
        private Dictionary<WireBehaviour, Collider2D[]> colliders = new Dictionary<WireBehaviour, Collider2D[]>();
        private Dictionary<Collider2D, NoCollide> noCollides = new Dictionary<Collider2D, NoCollide>();
    }
    public class UseEventTrigger2 : MonoBehaviour
    {
        [SkipSerialisation]
        public System.Action Action;

        private void Use()
        {
            if (Action != null)    
                Action();
        }
    }
    
}
// Originally uploaded by 'Kyrelink, Quickness'. Do not reupload without their explicit permission.
